import { Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTree } from '../../models/DatasourceTree.model';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { ActivatedRoute } from '@angular/router';
import { DataGridComponent } from 'src/app/vitis/data-grid/data-grid.component';
import { DataGridStudioService } from 'src/app/vitis/data-grid/services/data-grid-studio.service';
import { DataGridOptions } from 'src/app/vitis/data-grid/data-grid.model';

@Component({
  selector: 'app-admin-datasource-data-grid',
  templateUrl: './admin-datasource-data-grid.component.html',
  styleUrls: ['./admin-datasource-data-grid.component.scss']
})
export class AdminDatasourceDataGridComponent implements OnInit, OnDestroy {

  @ViewChild(DataGridComponent, { static: false, read: DataGridComponent }) set oDataGrid(oDataGrid: DataGridComponent) {
    setTimeout(() => {
      if (this.doInitGrid_(oDataGrid)) {
        this.getValues().then(() => {
          this.initDataGrid(oDataGrid);
          this.setValueInDataGrid();
          this.isInit = true;
        });
      }
    });
  }

  private aTree: DatasourceTree;
  private treeSubscription: any;
  private routeDataSubscription: any;
  private routeParamsSubscription: any;
  private oParams: any;
  private oOptions: any;
  private isInit: boolean = false;

  private dataTableInstance: any;
  private oDatatableOptions: any = {};
  private aValues: Array<any>;

  constructor(
    private route: ActivatedRoute,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private adminDatasourceService: AdminDatasourceService,
    private dataGridStudioService: DataGridStudioService,
  ) { }

  ngOnInit(): void {

    this.routeDataSubscription = this.route.data.subscribe(oOptions => {
      this.oOptions = oOptions;
    });
    this.routeParamsSubscription = this.route.params.subscribe(oParams => {
      // Avant la réasignation de this.oParams
      // Si un objet était précédement selectionné on enlève la sélection (pour un objet sur la même branche)
      this.adminDatasourceService.setObjectSelected(false, this.oParams, this.aTree);

      this.oParams = oParams;

      // Pour que initform soit lancé il faut l'arbre ainsi que l'id
      if (this.doInitGrid_()) {
        this.getValues().then(() => {
              this.initDataGrid(this.oDataGrid);
              this.setValueInDataGrid();
              this.isInit = true;
            });
      }
    });
    this.treeSubscription = this.adminDatasourceService.getTreeObs().subscribe((aTree) => {
      this.aTree = aTree;

      // Pour que initform soit lancé il faut l'arbre ainsi que l'id
      if (this.doInitGrid_()) {
        this.getValues().then(() => {
          this.initDataGrid(this.oDataGrid);
          this.setValueInDataGrid();
          this.isInit = true;
        });
      }
    });
  }

  ngOnDestroy() {
    // Enlève la sélection quand on change de branche
    this.adminDatasourceService.setObjectSelected(false, this.oParams, this.aTree);
    if (this.treeSubscription) {
      this.treeSubscription.unsubscribe();
    }
    if (this.routeDataSubscription) {
      this.routeDataSubscription.unsubscribe();
    }
    if (this.routeParamsSubscription) {
      this.routeParamsSubscription.unsubscribe();
    }
  }

  /**
   * Initialise les données dans la grille
   */
  private setValueInDataGrid() {
    if (this.vitisCheck.isDefAndNotNull(this.dataTableInstance)) {
      if (this.vitisCheck.isDefAndNotNull(this.aValues)) {
        // Mise à jour des données de la grille.
        this.dataTableInstance.clear();
        this.dataTableInstance.rows.add(this.aValues).draw();
      }
    }
  }

  /**
   * Initialise la grille de données.
   * @param {oDataGrid} DataGridComponent
   * @returns void
   */
  private initDataGrid(oDataGrid: DataGridComponent) {

    if (this.vitisCheck.isDefAndNotNull(this.oDatatableOptions)) {
      // Paramètres du plugin DataTable.
      const oDataGridOptions: DataGridOptions = {
        datatableOptions: {
          searching: true,
          paging: true,
          columns: this.oDatatableOptions.columns,
          scrollX: true,
        },
        datagridOptions: {}
      };
      if (!this.vitisCheck.isDefAndNotNull(oDataGridOptions.datatableOptions.columns)) {
        oDataGridOptions.datatableOptions.columns = [];
        console.error('Aucune colonne déclaré pour la grille');
      }

      // Création de la grille.
      this.dataTableInstance = oDataGrid.initTable(oDataGridOptions);
      // Sauve l'instance dans la définition du champ.
      this.dataTableInstance = this.dataTableInstance;
    }
  }

  /**
   * Récupère les valeurs (et eventuellement les colonnes) pour la grille
   */
  private getValues(): Promise<any> {
    return new Promise((resolve, reject) => {
      const oObject = this.adminDatasourceService.getObjectById(this.oParams.treeid);
      if (this.vitisCheck.isDefAndNotNull(oObject)) {
        if (this.vitisCheck.isDefAndNotNull(this.oOptions.get_url_func)) {
          if (this.vitisCheck.isFunction(oObject[this.oOptions.get_url_func])) {
            const oUrlAndParams = oObject[this.oOptions.get_url_func]();

            const params: any = oUrlAndParams.params;
            params.get_columns = true;
            params.limit = 100;
            this.vitisRequest.ajaxRequest({
              method: 'GET',
              url: oUrlAndParams.url,
              params: params,
              success: async (resp: any) => {
                if (this.vitisCheck.isDefAndNotNull(resp.data)) {
                  if (resp.data.status == 1) {
                    if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {
                      this.aValues = resp.data.data;
                    }

                    if (!this.vitisCheck.isDefAndNotNull(this.oDatatableOptions.columns) &&
                      this.vitisCheck.isDefAndNotNull(resp.data.columns)) {
                      this.oDatatableOptions.columns = resp.data.columns;
                    }
                    resolve(true);
                  }
                }
              }
            });
          } else {
            console.error('La fonction ' + this.oOptions.get_url_func + ' (param get_url_func) n\'existe pas dans l\'objet ', oObject);
          }
        }
      }
    });
  }

  /**
   * Si tout les paramètres requis pour initialisé la grille sont chargés.
   * @param oDataGrid
   * @returns boolean
   */
  private doInitGrid_(oDataGrid?: any): boolean {
    return this.vitisCheck.isDefAndNotNull(this.aTree) &&
      this.vitisCheck.isDefAndNotNull(this.oParams) &&
      this.vitisCheck.isDefAndNotNull(this.oParams.treeid) &&
      (this.vitisCheck.isDefAndNotNull(this.oDataGrid) || this.vitisCheck.isDefAndNotNull(oDataGrid)) &&
      !this.isInit;
  }
}
