import { Component, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute, NavigationEnd, Router } from '@angular/router';
import { VitisClone, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTree } from '../../models/DatasourceTree.model';
import { AdminDatasourceService } from '../../services/admin-datasource.service';

/**
 * TODO améliorer commentaire
 * Les onglets sont créé à partir de la liste de route enfant
 * Les routes enfants doivent avoir au minimum le paramètre 'treeid' pour pouvoir situer l'objet dans l'arbre
 * et elles doivent contenir un data.parentButton
 */
@Component({
  selector: 'app-admin-datasource-tabs',
  templateUrl: './admin-datasource-tabs.component.html',
  styleUrls: ['./admin-datasource-tabs.component.scss']
})
export class AdminDatasourceTabsComponent implements OnInit, OnDestroy {

  childrenParams: any;
  buttons: Array<any>;
  childrenUrlFirst: string;
  aTree: DatasourceTree;
  routeSubscription: any;
  treeSubscription: any;
  oObject: any;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private vitisClone: VitisClone,
    private vitisCheck: VitisCheck,
    private adminDatasourceService: AdminDatasourceService,
  ) { }

  ngOnInit(): void {

    let child = this.vitisClone.clone(this.route.firstChild);
    this.routeSubscription = this.router.events.subscribe(e => {
      if (e instanceof NavigationEnd && child !== this.route.firstChild) {
        child = this.vitisClone.clone(this.route.firstChild);
        this.childrenUrlFirst = this.route.firstChild.snapshot.url[0].path;
        // Avant la réasignation de this.oParams
        // Si un objet était précédement selectionné on enlève la sélection (pour un objet sur la même branche)
        this.adminDatasourceService.setObjectSelected(false, this.childrenParams, this.aTree);
        this.childrenParams = this.vitisClone.clone(this.route.firstChild.snapshot.params);
        this.adminDatasourceService.setObjectSelected(true, this.childrenParams, this.aTree);
      }
    });

    this.childrenUrlFirst = this.route.firstChild.snapshot.url[0].path;
    this.childrenParams = this.vitisClone.clone(this.route.firstChild.snapshot.params);

    this.initButtons(this.route.routeConfig.children);

    this.treeSubscription = this.adminDatasourceService.getTreeObs().subscribe((aTree: DatasourceTree) => {
      this.aTree = aTree;

      this.adminDatasourceService.setObjectSelected(true, this.childrenParams, this.aTree);
    });
  }

  ngOnDestroy() {
    // Enlève la sélection quand on change de branche
    this.adminDatasourceService.setObjectSelected(false, this.childrenParams, this.aTree);
    if (this.routeSubscription) {
      this.routeSubscription.unsubscribe();
    }
    if (this.treeSubscription) {
      this.treeSubscription.unsubscribe();
    }
  }

  /**
   * Change de page
   * @param btn
   */
  public changementPage(btn: any): void {

    const params = [btn.url];

    let oObject: any;

    for (let i = 0; i < btn.params.length; i++) {
      switch (btn.params[i]) {
        // formName & treeid sont des valeurs récupéré de la route, les autres sont récupéré dans l'objet
        case 'formName':
          params.push(this.getFormName());
          break;
        case 'treeid': // treeid requis mais mis dans le tableau pour géré l'emplacement
          params.push(this.childrenParams.treeid);
          break;
        default:
          // Si oObject n'existe pas on essaye de le charger
          if (!this.vitisCheck.isDefAndNotNull(oObject)) {
            oObject = this.adminDatasourceService.getObjectById(this.childrenParams.treeid);
            // Si après le chargement il n'existe pas c'est une erreur
            if (!this.vitisCheck.isDefAndNotNull(oObject)) {
              console.error('Erreur lors du chargement de l\'objet', this.childrenParams.treeid);
              break;
            }
          }
          params.push(oObject[btn.params[i]]);
          break;
      }
    }

    this.router.navigate(params, { relativeTo: this.route, queryParamsHandling: 'merge' });
  }

  /**
   * Vérifie si le bouton est actif dans le mode de formulaire actuel
   * @param btn
   */
  public btnIsActive(btn: any): boolean {
    const currentFormName = this.getFormName();
    let isBtnInForm = currentFormName === 'display' ? true : false; // Les onglets sans 'active' sont disponibles en display
    if (this.vitisCheck.isDefAndNotNull(btn.active)) {
      isBtnInForm = btn.active.indexOf(currentFormName) != -1;
    }
    const haveCondition = btn.condition;
    let isFulfilledCondition: boolean = false;
    if (haveCondition) {
      isFulfilledCondition = this.isFulfilledCondition(btn.condition);
    }
    return isBtnInForm && (!haveCondition || (haveCondition && isFulfilledCondition));
  }

  /**
   * Créer la liste des boutons à partir des childrens de la route
   */
  private initButtons(childrenRoute: any) {
    this.buttons = [];
    for (const key in childrenRoute) {
      if (Object.prototype.hasOwnProperty.call(childrenRoute, key)) {
        const route = childrenRoute[key];
        if (this.vitisCheck.isDefAndNotNull(route.data) && this.vitisCheck.isDefAndNotNull(route.data.parentButton)) {
          this.buttons.push(route.data.parentButton);
        }
      }
    }
  }

  /**
   * Vérifie si la condition est rempli
   * @param sCondition
   */
  private isFulfilledCondition(sCondition: string) {
    if (this.vitisCheck.isDefAndNotNull(this.childrenParams) && this.vitisCheck.isDefAndNotNull(this.aTree)) {
      const oObject = this.adminDatasourceService.getObjectById(this.childrenParams.treeid);
      if (this.vitisCheck.isDefAndNotNull(oObject)) {
        return this.adminDatasourceService.isFulfilledCondition(sCondition, oObject);
      }
    }
  }

  /**
   * Get formName
   */
  private getFormName() {
    return this.vitisCheck.isDefAndNotNull(this.childrenParams.formName) ? this.childrenParams.formName : 'display';
  }
}
