import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { WmsService } from 'src/modules/vmap/components/admin-datasource/services/wms.service';
import { TranslateService } from '@ngx-translate/core';

export class DatasourceTreeviewWMSConnectionFormJS {

  private oObject: any;
  // Services
  private vitisCheck: VitisCheck;
  private wmsService: WmsService;
  private translate: TranslateService;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {object} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: any,
    injector: any
  ) {
    this.oObject = oObject;
    this.vitisCheck = injector.get(VitisCheck);
    this.wmsService = injector.get(WmsService);
    this.translate = injector.get(TranslateService);
  }

  /**
   * BeforeEvent => Executer avant la sauvegarde du formulaire pour vérifier que le flux fonctionne
   * @param oFormValues
   */
  public refreshCapabilities(oFormValues): Promise<boolean> {
    return new Promise((resolve, reject) => {
      if (this.vitisCheck.isDefAndNotNull(this.oObject)) {
        this.wmsService.getAjaxCapabilitiesWMS({
          flux_url: oFormValues.flux_url,
          flux_version: oFormValues.flux_version,
          flux_login: oFormValues.flux_login,
          flux_password: oFormValues.flux_password
        }).then((oCapabilities) => {
          if (oCapabilities !== false) {
            this.oObject.oCapabilities = oCapabilities;
            resolve(true);
          } else {
            this.translate.get('CANT_REFRESH_CAPABILITIES').subscribe((sTranslate: string) => {
              vitisAlert.fireToast({
                type: 'warning',
                title: sTranslate
              });
            });
            reject();
          }
        });
      }
    });
  }
}