import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { WmtsService } from 'src/modules/vmap/components/admin-datasource/services/wmts.service';

export class DatasourceTreeviewWMTSConnectionFormJS {

  private oObject: any;
  // Services
  private vitisCheck: VitisCheck;
  private wmtsService: WmtsService;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {object} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: any,
    injector: any
  ) {
    this.oObject = oObject;

    this.vitisCheck = injector.get(VitisCheck);
    this.wmtsService = injector.get(WmtsService);
  }

  /**
   * Evenement lancé par AdminDatasourceFormComponent juste avant l'envoi du formulaire
   * @param oFormValues
   * @param sFormDefinitionName
   */
  public prepareData(oFormValues: any): Promise <boolean> {
    return new Promise((resolve, reject) => {
      const fluxUrl = oFormValues.flux_url;
      const fluxVersion = oFormValues.flux_version;
      let WMTSType = oFormValues.flux_type;
      const fluxName = oFormValues.name;

      if (this.vitisCheck.isObject(WMTSType)) {
        if (this.vitisCheck.isDefAndNotNull(WMTSType['selectedOption'])) {
          if (this.vitisCheck.isDefAndNotNull(WMTSType['selectedOption']['value'])) {
            WMTSType = WMTSType['selectedOption']['value'];
          }
        }
      }

      if (!this.vitisCheck.isDefAndNotNull(fluxName) || fluxName === '') {
        this.wmtsService.printErrorMsg('DATASOURCE_FORM_WMTS_EVENT_ERROR_MISSING_NAME');
        resolve(false);
      }
      if (!this.vitisCheck.isDefAndNotNull(fluxUrl) || fluxUrl === '') {
        this.wmtsService.printErrorMsg('DATASOURCE_FORM_WMTS_EVENT_ERROR_MISSING_LINK');
        resolve(false);
      }
      if (!this.vitisCheck.isDefAndNotNull(fluxVersion) || fluxVersion === '') {
        this.wmtsService.printErrorMsg('DATASOURCE_FORM_WMTS_EVENT_ERROR_MISSING_VERSION');
        resolve(false);
      }
      if (!this.vitisCheck.isDefAndNotNull(WMTSType) || WMTSType === '') {
        this.wmtsService.printErrorMsg('DATASOURCE_FORM_WMTS_EVENT_ERROR_MISSING_TYPE');
        resolve(false);
      }
      if (fluxUrl.toLowerCase().indexOf('service=') > 0) {
        this.wmtsService.printErrorMsg('DATASOURCE_FORM_WMTS_EVENT_ERROR_DON_T_USE_SERVICE');
        resolve(false);
      }
      if (fluxUrl.toLowerCase().indexOf('request=') > 0) {
        this.wmtsService.printErrorMsg('DATASOURCE_FORM_WMTS_EVENT_ERROR_DON_T_USE_REQUEST');
        resolve(false);
      }
      if (fluxUrl.toLowerCase().indexOf('version=') > 0) {
        this.wmtsService.printErrorMsg('DATASOURCE_FORM_WMTS_EVENT_ERROR_DON_T_USE_VERSION');
        resolve(false);
      }

      this.wmtsService.getAjaxCapabilitiesWMTS({
        flux_url: fluxUrl,
        flux_version: fluxVersion,
        flux_type: WMTSType
      }).then((oCapabilities: any) => {
        // Flux valide
        if (this.vitisCheck.isDefAndNotNull(oCapabilities) && this.vitisCheck.isObject(oCapabilities)) {
          if (this.vitisCheck.isDefAndNotNull(this.oObject)) {
            this.oObject.oCapabilities = oCapabilities;
          }
          resolve(true);
        }
      });
    });
  }
}