import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { MultiTranslateHttpLoader } from 'ngx-translate-multi-http-loader';
import { environment } from 'src/environments/environment';
import { SharedModule } from 'src/app/vitis/shared/shared.module';
import { FormreaderModule } from 'src/app/vitis/formreader/formreader.module';
import { routing } from './carto.route';
import { CartoComponent } from './carto.component';
import { MapCoreModule } from 'src/app/vitis/map-core/map-core.module';
import { VmapControlsComponent } from './components/vmap-controls/vmap-controls.component';
import { VmapControlsMapInfoComponent } from './components/vmap-controls/components/vmap-controls-map-info/vmap-controls-map-info.component';
import { VmapControlsOverviewComponent } from './components/vmap-controls/components/vmap-controls-overview/vmap-controls-overview.component';
import { VmapControlsLeftButtonsComponent } from './components/vmap-controls/components/vmap-controls-left-buttons/vmap-controls-left-buttons.component';
import { VmapControlsVitisButtonComponent } from './components/vmap-controls/components/vmap-controls-left-buttons/components/vmap-controls-vitis-button/vmap-controls-vitis-button.component';
import { VmapControlsLayersButtonComponent } from './components/vmap-controls/components/vmap-controls-left-buttons/components/vmap-controls-layers-button/vmap-controls-layers-button.component';
import { VmapControlsMapsButtonComponent } from './components/vmap-controls/components/vmap-controls-left-buttons/components/vmap-controls-maps-button/vmap-controls-maps-button.component';
import { VmapMapsPanelComponent } from './components/vmap-maps-panel/vmap-maps-panel.component';
import { VmapLayersPanelComponent } from './components/vmap-layers-panel/vmap-layers-panel.component';
import { VmapLayersListComponent } from './components/vmap-layers-panel/components/vmap-layers-list/vmap-layers-list.component';
import { VmapLayersTreeComponent } from './components/vmap-layers-panel/components/vmap-layers-tree/vmap-layers-tree.component';
import { VmapLayersLegendComponent } from './components/vmap-layers-panel/components/vmap-layers-legend/vmap-layers-legend.component';
import { VmapLayersSettingsComponent } from './components/vmap-layers-panel/components/vmap-layers-settings/vmap-layers-settings.component';
import { VmapToolsComponent } from './components/vmap-tools/vmap-tools.component';
import { VmapToolsMeasureComponent } from './components/vmap-tools/components/vmap-tools-measure/vmap-tools-measure.component';
import { VmapToolsSelectComponent } from './components/vmap-tools/components/vmap-tools-select/vmap-tools-select.component';
import { VmapToolsSelectRightPanelComponent } from './components/vmap-tools/components/vmap-tools-select-right-panel/vmap-tools-select-right-panel.component';
import { LayerSymbologyModule } from '../layer-symbology/layer-symbology.module';
import { VmapToolsFormComponent } from './components/vmap-tools/components/vmap-tools-select-right-panel/vmap-tools-form/vmap-tools-form.component';
import { VmapToolsGeometryFormComponent } from './components/vmap-tools/components/vmap-tools-select-right-panel/vmap-tools-geometry-form/vmap-tools-geometry-form.component';
import { VmapToolsRequestorModalComponent } from './components/vmap-tools/components/vmap-tools-select-right-panel/vmap-tools-requestor-modal/vmap-tools-requestor-modal.component';
import { DataGridModule } from 'src/app/vitis/data-grid/data-grid.module';
import { VmapToolsPrintComponent } from './components/vmap-tools/components/vmap-tools-print/vmap-tools-print.component';
import { VmapToolsAddRightPanelComponent } from './components/vmap-tools/components/vmap-tools-add-right-panel/vmap-tools-add-right-panel.component';
import { VmapToolsModifyRightPanelComponent } from './components/vmap-tools/components/vmap-tools-modify-right-panel/vmap-tools-modify-right-panel.component';
import { VmapToolsSearchComponent } from './components/vmap-tools/components/vmap-tools-search/vmap-tools-search.component';
import { VmapToolsFormComponent as VmpToolsFormSearchComponent } from './components/vmap-tools/components/vmap-tools-search/vmap-tools-form/vmap-tools-form.component';
import { VmapToolsRefreshComponent } from './components/vmap-tools/components/vmap-tools-refresh/vmap-tools-refresh.component';
import { VmapToolsCompareMapComponent } from './components/vmap-tools/components/vmap-tools-compare-map/vmap-tools-compare-map.component';
import { VmapToolsMapListComponent } from './components/vmap-tools/components/vmap-tools-map-list/vmap-tools-map-list.component';
import { VmapToolsFilterComponent } from './components/vmap-tools/components/vmap-tools-filter/vmap-tools-filter.component';
import { VmapLayersMapsPanelMobileComponent } from './components/vmap-mobile/vmap-layers-maps-panel-mobile/vmap-layers-maps-panel-mobile.component';
import { VmapToolsSearchMobileComponent } from './components/vmap-mobile/vmap-tools-search-mobile/vmap-tools-search-mobile.component';
import { VmapToolsGeolocationMobileComponent } from './components/vmap-mobile/vmap-tools-geolocation-mobile/vmap-tools-geolocation-mobile.component';
import { VmapToolsSelectionMobileComponent } from './components/vmap-mobile/vmap-tools-selection-mobile/vmap-tools-selection-mobile.component';
import { VmapToolsMenuMobileComponent } from './components/vmap-mobile/vmap-tools-menu-mobile/vmap-tools-menu-mobile.component';
import { VmapFullscreenFormMobileComponent } from './components/vmap-mobile/vmap-fullscreen-form-mobile/vmap-fullscreen-form-mobile.component';
import { VmapToolsTabsComponent } from './components/vmap-tools/components/vmap-tools-tabs/vmap-tools-tabs.component';
import { VmapToolsRequestorComponent } from './components/vmap-tools/components/vmap-tools-requestor/vmap-tools-requestor.component';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { VmapToolsSimpleHtmlComponent } from './components/vmap-tools/components/vmap-tools-simple-html/vmap-tools-simple-html.component';

/**
 * Chargement des fichiers de traduction
 * @param http
 * @returns
 */
export function HttpLoaderFactory(http: HttpClient) {
  return new MultiTranslateHttpLoader(http, environment.appStruct['translate']);
}

@NgModule({
  declarations: [
    CartoComponent,
    VmapControlsComponent,
    VmapControlsMapInfoComponent,
    VmapControlsOverviewComponent,
    VmapControlsLeftButtonsComponent,
    VmapControlsVitisButtonComponent,
    VmapControlsLayersButtonComponent,
    VmapControlsMapsButtonComponent,
    VmapLayersMapsPanelMobileComponent,
    VmapMapsPanelComponent,
    VmapLayersPanelComponent,
    VmapLayersListComponent,
    VmapLayersTreeComponent,
    VmapLayersLegendComponent,
    VmapLayersSettingsComponent,
    VmapToolsComponent,
    VmapToolsMeasureComponent,
    VmapToolsSearchComponent,
    VmapToolsSearchMobileComponent,
    VmapToolsSelectComponent,
    VmapToolsSelectRightPanelComponent,
    VmapToolsFormComponent,
    VmapToolsGeometryFormComponent,
    VmapToolsRequestorModalComponent,
    VmapToolsPrintComponent,
    VmapToolsAddRightPanelComponent,
    VmapToolsModifyRightPanelComponent,
    VmpToolsFormSearchComponent,
    VmapToolsTabsComponent,
    VmapToolsRequestorComponent,
    VmapToolsRefreshComponent,
    VmapToolsMapListComponent,
    VmapToolsCompareMapComponent,
    VmapToolsFilterComponent,
    VmapToolsGeolocationMobileComponent,
    VmapToolsSelectionMobileComponent,
    VmapToolsMenuMobileComponent,
    VmapFullscreenFormMobileComponent,
    VmapToolsSimpleHtmlComponent
  ],
  exports: [
    VmapToolsPrintComponent,
    VmapToolsMeasureComponent,
    VmapToolsSelectComponent,
    VmapToolsSearchComponent,
    VmapToolsTabsComponent,
    VmapToolsRequestorComponent,
    VmapToolsRefreshComponent,
    VmapToolsCompareMapComponent
  ],
  imports: [
    SharedModule,
    FormreaderModule,
    CommonModule,
    HttpClientModule,
    DataGridModule,
    TranslateModule.forChild({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      }
    }),
    routing,
    FormsModule,
    MapCoreModule,
    LayerSymbologyModule,
    ReactiveFormsModule,
    NgbModule,
  ]
})
export class VmapCartoModule { }
