import { Component, OnInit, Input, ViewChild, AfterViewInit, Output, EventEmitter, OnDestroy } from '@angular/core';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-vmap-fullscreen-form-mobile',
  templateUrl: './vmap-fullscreen-form-mobile.component.html',
  styleUrls: ['./vmap-fullscreen-form-mobile.component.scss']
})
export class VmapFullscreenFormMobileComponent implements OnInit, AfterViewInit, OnDestroy {

  @ViewChild(FormreaderComponent) oFormReader: FormreaderComponent;

  @Input() open: boolean;
  @Input() sDefinition;
  @Input() sFormName;
  @Input() sDatas;

  @Output() formChanges = new EventEmitter();

  public definition;
  private datas;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    private vitisClone: VitisClone
  ) { }

  ngOnInit() {
  }

  ngAfterViewInit(): void {
    setTimeout(() => {
      this.definition = this.vitisClone.clone(this.sDefinition);
      this.datas = this.vitisClone.clone(this.sDatas);
      this.oFormReader.initForm(this.definition, this.sFormName, this.datas);

      this.subscriptions.push(this.oFormReader.oFormValues.valueChanges.subscribe(form => {
        this.formChanges.next(form);
      }));
    });
  }

  /**
   * Fonction de destruction
   */
   ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panel de formulaire pour les mobiles
   */
  public closePanel() {
    this.open = false;
  }
}
