import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { Router } from '@angular/router';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';

@Component({
  selector: 'app-vmap-layers-maps-panel-mobile',
  templateUrl: './vmap-layers-maps-panel-mobile.component.html',
  styleUrls: ['./vmap-layers-maps-panel-mobile.component.scss']
})
export class VmapLayersMapsPanelMobileComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;
  @Input() public isWidget: boolean;
  @Input() public haveLayersPanel: boolean;

  public bPanelOpen: boolean;
  public selectedTab: number;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    private vmapToolsService: VmapToolsService,
    private router: Router
  ) { }

  get mapId() {
    return this.olMap ? this.olMap.get('map_id') : null;
  }

  ngOnInit() {

    // Écoute l'ouverture des menus
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'control-layers-maps-mobile');

      // Menu ouvert par défaut
      if (this.bPanelOpen) {
        if (this.isWidget) {
          if (this.haveLayersPanel) {
            this.selectedTab = 1;
          } else {
            this.selectedTab = 2;
          }
        } else {
          this.selectedTab = 0;
        }
      }
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.bPanelOpen = false;
    this.vmapToolsService.setLeftPanel(0, '');
  }

  /**
   * Sélection d'un ongle
   * @param index
   */
  public selectTab(index) {
    this.selectedTab = index;
  }

  /**
   * Load une carte
   */
  public loadMap(map_id) {

    // Change le map_id dans l'URL
    this.router.navigate(['/dashboad/map'], { queryParams: { map_id: map_id } });

    // Ferme l'outil
    this.closePanel();
  }
}
