import { Component, OnInit, Input, ViewChild } from '@angular/core';
import { VitisClone, VitisRequest, vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { Geometry } from 'ol/geom';
import { Feature } from 'ol';
import { TranslateService } from '@ngx-translate/core';
import { environment } from 'src/environments/environment';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VmapToolsFormComponent } from 'src/app/modules/vmap/carto/components/vmap-tools/components/vmap-tools-select-right-panel/vmap-tools-form/vmap-tools-form.component';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';

@Component({
  selector: 'app-vmap-tools-add-right-panel',
  templateUrl: './vmap-tools-add-right-panel.component.html',
  styleUrls: ['./vmap-tools-add-right-panel.component.scss']
})
export class VmapToolsAddRightPanelComponent implements OnInit {

  @ViewChild(VmapToolsFormComponent) oFormComponent: VmapToolsFormComponent;

  @Input() map;
  @Input() selection;

  public sFormName = 'insert';
  public sDefinition: any;
  public sDatasForm: any = {};

  currDatasForm: any = {};

  public geomEventValue: string = '';

  private feature: Feature<Geometry>;
  private oFormReader: FormreaderComponent;

  constructor(
    private vitisClone: VitisClone,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private mapSelectService: VmapSelectService,
    private mapToolsService: VmapToolsService,
    private translate: TranslateService,
    private mapUtils: MapCoreUtilsService,
    private formreaderSrvc: FormreaderService,
  ) { }

  ngOnInit(): void {
    if (this.selection) {
      const def = this.vitisCheck.isString(this.selection.published_form_json_def) ?
        JSON.parse(this.selection.published_form_json_def) : this.selection.published_form_json_def;

      this.sDefinition = this.vitisClone.clone(def);
    }
  }

  /**
   * Permet de récupérer l'update de la geométrie de la feature
   * @param feature
   */
  public subscribeFeature(feature: Feature<Geometry>) {
    this.feature = feature;
  }

  /**
   * Envoie une requête POST d'ajout d'un object dans la couche choisie
   */
  public sendInsertForm(): void {
    let data: any;
    const oFormValues = this.oFormComponent.getFormValues();
    if (this.selection.is_geometry_layer) {
      data = { ...oFormValues, [this.selection.geomfield]: this.mapUtils.getEWKTFromGeomAndProj(this.feature.getGeometry(), this.map, '2154') };
    } else {
      data = { ...oFormValues };
    }
    this.oFormComponent.oFormReader['addObjectLayerForm'] = (resData: any) => {
      return new Promise<void>((resolve, reject) => {
        this.formreaderSrvc.manageLargesFiles(resData).then((managedData: any) => {
          this.vitisRequest.ajaxRequestPromise({
            method: 'POST',
            url: `${environment.properties.app_api_url}/vmap/layers/${this.selection.layer_id}/query`,
            data: this.formreaderSrvc.transformToDataForm(managedData),
          }).then((resp: any) => {
            if (resp && resp.data && resp.data.data) {
              this.refreshLayer(this.selection.layer_id);
              this.cancelInsertForm();
              // Add Toast
              vitisAlert.fireToast({
                icon: 'success',
                title: this.translate.instant('COMPONENT_MAP_ENABLE_UTILS_ADD_SUCCESS_TOAST')
              });
              resolve();
            }
          }).catch(e => {
            console.error(e);
            this.translate.get(['ERROR_INSERT_LAYER_OBJECT', 'ERROR_INSERT_LAYER_OBJECT_DETAIL']).subscribe(async (translation) => {
              vitisAlert.alert({
                type: 'error',
                title: translation['ERROR_INSERT_LAYER_OBJECT'],
                text: translation['ERROR_INSERT_LAYER_OBJECT_DETAIL'],
              });
            });
            reject();
          });
        });
      });
    };

    this.sDefinition[this.sFormName].event = 'addObjectLayerForm()';
    this.oFormComponent.oFormReader.submitForm(this.sDefinition, this.sFormName, data);
  }

  /**
   * Ferme le formulaire d'ajout d'un object dans la couche
   */
  public cancelInsertForm(): void {
    this.mapToolsService.getCurrentOpenedTool().next('');
    this.mapToolsService.closeAllPanels();
  }

  /**
   * Récupère les changement du formulaire d'ajout
   * @param form
   */
  public getFormChanges(form: any): void {
    this.currDatasForm = this.vitisClone.clone(form);
  }

  /**
   * Rafraîchit la couche
   * @param iLayerId
   */
  private refreshLayer(iLayerId: number): void {
    this.mapToolsService.refreshLayer(this.map, iLayerId, true);
  }

  /**
   * Recharge le javascript lorsque le formreader est modifié
   * @param oFormReader
   */
  public updateFormReader(oFormReader: FormreaderComponent) {
    // Charge le JS du formulaire
    if (this.vitisCheck.isDefAndNotNull(this.selection.published_form_js_def)) {
      if (!this.vitisCheck.isSameObject(this.oFormReader, oFormReader)) {
        const oScope = {
          map: this.map,
          oLayer: this.selection,
        };
        this.mapToolsService.loadJs(this.selection.published_form_js_def, this.selection.layer_id, oFormReader, oScope);
        this.oFormReader = this.vitisClone.clone(oFormReader);
      }
    }
  }
}
