import { Component, OnInit, Input, EventEmitter, ViewChild, Optional, Inject, ElementRef } from '@angular/core';
import { Feature, Map } from 'ol';
import { MapToolDef } from '../../map-tools.model';
import { DataGridComponent } from 'src/app/vitis/data-grid/data-grid.component';
import { DataGridOptions } from 'src/app/vitis/data-grid/data-grid.model';
import { WorkspaceListOptions } from 'src/app/vitis/workspacelist/workspacelist.model';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { vitisAlert, VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { DashboardComponent } from 'src/app/vitis/core/components/dashboard/dashboard.component';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';
import { Subscription } from 'rxjs';
import { EventsKey } from 'ol/events';
import { unByKey } from 'ol/Observable';
import { MapCorePopupService, Popup } from 'src/app/vitis/map-core/services/map-core-popup/map-core-popup.service';
import { Geometry } from 'ol/geom';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { NgbActiveModal, NgbModal } from '@ng-bootstrap/ng-bootstrap';
import exportFromJSON from 'export-from-json';

declare var $: any;
@Component({
  selector: 'app-vmap-tools-requestor',
  templateUrl: './vmap-tools-requestor.component.html',
  styleUrls: ['./vmap-tools-requestor.component.scss']
})
export class VmapToolsRequestorComponent implements OnInit {

  @Input() olMap: Map;
  @Input() oTool: MapToolDef;

  @Input() oTabsEmitter: EventEmitter<Array<any>>;
  @Input() oSelectedTabEmitter: EventEmitter<number>;

  @ViewChild(DataGridComponent, { static: false }) oDataGrid: DataGridComponent;
  @ViewChild('exportModal') public exportModalContent: ElementRef;
  private exportModal: NgbActiveModal;

  public iSelectedTab: number;
  public aTabs: Array<any> = [];

  public aLayers: Array<any> = [];
  public currentLayer: string;

  private layer: any;
  private clickedCell_: any;
  private bOpen: boolean;

  public aFilterColumns = [];
  public sFilterGeom: string;
  public olFilterGeom: Geometry;
  public bFilterGeomEnabled: boolean = true;

  // Filtre en cours
  public oFilter: any = {
    type: 'custom',
    nodes: []
  };

  // Types de filtre disponible
  public oFilterTypes: any = {
    custom: 'VITIS_WORKSPACELIST_FILTER_CUSTOM',
    form: 'VITIS_WORKSPACELIST_FILTER_FORM'
  };

  // Largeur de page
  public pageLength: Number = environment.properties.rows_per_page;

  // Options de la grille de données.
  public oDataGridOptions: DataGridOptions = {
    datatableOptions: {},
    datagridOptions: WorkspaceListOptions
  };

  public dataTableInstance: any;

  private locationPopup: Popup;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];
  private mapSubscriptions: EventsKey[] = [];

  // Formats d'exportation de la liste.
  public exportFormats: any = {
    'csv': {
      replace_label: true,
      no_data_value: ''
    },
    'xls': {
      replace_label: true,
      no_data_value: ''
    },
    'json': {
      replace_label: false,
      no_data_value: null
    },
    'xml': {
      replace_label: false,
      no_data_value: ''
    }
  };
  public exportTypes: any = [{
    label: 'REQUESTOR_EXTRACT_TYPE_SELECTION',
    value: 'selected_rows',
  }, {
    label: 'REQUESTOR_EXTRACT_TYPE_CURRENT_PAGE',
    value: 'current_page',
  }, {
    label: 'REQUESTOR_EXTRACT_TYPE_ALL_DATA',
    value: 'all_rows',
  }];
  public exportForm: FormGroup;

  constructor(
    private mapManagerService: MapCoreManagerService,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private auth: AuthService,
    private translate: TranslateService,
    public vmapToolsService: VmapToolsService,
    public mapCoreUtilsService: MapCoreUtilsService,
    public vmapSelectService: VmapSelectService,
    private mapPopup: MapCorePopupService,
    private mapCoreDrawService: MapCoreDrawService,
    private formBuilder: FormBuilder,
    private modalService: NgbModal,
    @Optional() @Inject(DashboardComponent) private dashboardComponent: DashboardComponent,
  ) { }

  ngOnInit(): void {
    // Rafraîchit les données du requêteur en cas de changement de carte
    this.mapSubscriptions.push(this.olMap.on('change', () => {
      this.initRequestor();
      this.initDrawLayer();
    }));

    // Nombre d'éléments à afficher sur la grille
    this.pageLength = Number(environment.properties.rows_per_page);

    // Subscribe aux évènements de la barre d'onglets de l'outil
    this.subscriptions.push(this.oTabsEmitter.subscribe((next: Array<any>) => {
      this.aTabs = next;
      this.iSelectedTab = 0;
    }));

    this.subscriptions.push(this.oSelectedTabEmitter.subscribe((next: number) => {
      this.iSelectedTab = next;

      if (this.aTabs.length > 0) {

        // Vide le filtre
        this.oFilter = {
          type: 'custom',
          nodes: []
        };

        // Couche en cours
        this.currentLayer = this.aTabs[this.iSelectedTab].label;

        // Ferme la popup de localisation
        this.closeLocationPopup_();

        // Initialise la grille et charge la donnée
        this.initLayerDataGrid_();
      }
    }));

    // Observe l'état des bandeaux
    this.subscriptions.push(this.vmapToolsService.getLeftPanelEmiter().subscribe(() => {
      this.autoAjustTables();
    }));
    this.subscriptions.push(this.vmapToolsService.getRightPanelEmiter().subscribe(() => {
      this.autoAjustTables();
    }));
    this.subscriptions.push(this.vmapToolsService.getBottomPanelEmiter().subscribe(() => {
      this.autoAjustTables();
    }));
    if (this.dashboardComponent) {
      this.subscriptions.push(this.dashboardComponent.sidenavOpenChange.subscribe(() => {
        this.autoAjustTables();
      }));
    }
    this.subscriptions.push(this.vmapToolsService.getRefreshMapEmitter().subscribe(() => {
      if (this.bOpen) {
        this.refreshGrid();
      }
    }));

    // Ouverture du requêteur après une requête géométrique
    this.subscriptions.push(this.vmapSelectService.getRequestorSelectionObs().subscribe((selectionOptions) => {
      this.openFromGeomSelection_(selectionOptions);
    }));

    // Ouverture de l'outil
    this.subscriptions.push(this.oTool.onOpen.subscribe(() => {
      this.bOpen = true;
      if (!this.vitisCheck.isDefAndNotNull(this.iSelectedTab)) {
        // Lance le chargement de la première tab
        this.oSelectedTabEmitter.next(0);
      } else {
        this.refreshGrid();
      }
      this.displayGeomSelectionFilter();
    }));

    // Fermeture de l'outil
    this.subscriptions.push(this.oTool.onClose.subscribe(() => {
      this.bOpen = false;
      // Ferme la popup de localisation
      this.closeLocationPopup_();
      // Supprime le feature de filtre
      this.displayGeomSelectionFilter(true);
    }));

    this.initRequestor();
    this.initDrawLayer();

    // formulaire d'exportation de la liste.
    this.exportForm = this.formBuilder.group({
      export_file_name: ['export', Validators.required],
      export_file_format: ['csv', Validators.required],
      export_rows_number: ['20'],
      export_type: ['selected_rows'],
    });
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
    this.mapSubscriptions.forEach(mapSubscription => unByKey(mapSubscription));
    this.displayGeomSelectionFilter(true);
  }

  /**
   * Applique le filtre à la grille
   */
  public applyFilter(): void {
    // Nettoyage du filtre
    this.oFilter.nodes = this.oFilter.nodes.map(e => {
      if (e.compareOperator === '~' && e.column.searchType === 'number') {
        e.compareOperator = '=';
      }
      return e;
    });

    // Filtre formaté pour la requête API
    this.oFilter.value = this.getAPIFilterFromNodes_(this.oFilter.nodes);

    // Rafraichit la vue
    if (this.dataTableInstance && this.dataTableInstance.ajax) {
      this.dataTableInstance.ajax.reload();
    }
  }

  /**
   * Lit le tableau de noeuds et retourne un filtre compatible avec l'API Vitis
   * @param  {Array<any>} aFilterNodes
   * @returns any
   */
  public getAPIFilterFromNodes_(aFilterNodes: Array<any>): any {
    let oFilter: any = {};

    if (this.vitisCheck.isArray(aFilterNodes)) {
      if (aFilterNodes.length > 0) {
        oFilter = {
          relation: 'AND',
          operators: []
        };

        for (let i = 0; i < aFilterNodes.length; i++) {
          const oNode = aFilterNodes[i];

          if (this.vitisCheck.isDefAndNotNull(oNode.column)) {
            const oNodeFilter: any = {};

            if (this.vitisCheck.isDefAndNotNull(oNode.column.field) && this.vitisCheck.isDefAndNotNull(oNode.compareOperator) && this.vitisCheck.isDefAndNotNull(oNode.value)) {
              oNodeFilter.column = oNode.column.field;
              oNodeFilter.compare_operator = oNode.compareOperator;
              oNodeFilter.value = oNode.value;

              if (oNode.compareOperator === '~') {
                oNodeFilter.compare_operator = 'LIKE';
                oNodeFilter.compare_operator_options = {
                  'case_insensitive': true
                };
                oNodeFilter.value = '%' + oNodeFilter.value + '%';
              }

              // modification de la colonne à utiliser pour filtrer
              if (this.vitisCheck.isDefAndNotNull(oNode.column.searchColumn)) {
                oNodeFilter.column = oNode.column.searchColumn;
              }

              oFilter.operators.push(oNodeFilter);
            }
          }
        }
      }
    }

    return oFilter;
  }

  /**
   * Bouton rafraîchir
   *
   * On n'utilise pas celui de datagrid car tous les boutons sont surchargés
   */
  public refreshGrid() {
    if (this.dataTableInstance) {
      this.dataTableInstance.ajax.reload(null, false);
    }
  }

  /**
   * Active ou desactive le filtre géométrique
   */
  public togglegeomfilter() {
    this.bFilterGeomEnabled = !this.bFilterGeomEnabled;
    this.refreshGrid();
  }

  /**
   * Affichage de la fenêtre modale d'exportation de la liste.
   */
  public displayExportModal() {
    this.exportModal = this.modalService.open(this.exportModalContent);
  }

  /**
   * Ferme de la fenêtre modale d'exportation de la liste.
   */
  public hideExportModal() {
    if (this.exportModal) {
      this.exportModal.close();
    }
  }

  /**
   * Lance la fenêtre d'export
   */
  public exportGrid() {

    // Récupération des valeurs du formulaire ou valeur par défaut
    const sFormat: string = (this.exportForm.value.export_file_format) ? this.exportForm.value.export_file_format : 'csv';
    const sFileName: string = (this.exportForm.value.export_file_name) ? this.exportForm.value.export_file_name : 'export';
    const sExportType: string = (this.exportForm.value.export_type) ? this.exportForm.value.export_type : 'selected_rows';

    this.getDataForExport(sExportType, this.exportFormats[sFormat]).then((aData: any[]) => {
      switch (sFormat) {
        case 'csv':
        case 'json':
        case 'xml':
        case 'xls':
          const aConfig: any = {
            data: aData,
            fileName: sFileName,
            exportType: sFormat,
          };
          // transforme le json et télécharge le résultat
          exportFromJSON(aConfig);
          break;
        default:
          console.error('format not available : ' + sFormat);
          break;
      }
    }).catch(() => {
      console.error('error occured while exporting data');
    }).finally(() => {
      this.hideExportModal();
    });
  }

  /**
   * Rafraîchit les données du requêteur
   */
  private initRequestor() {

    // Récupération des couches requêtables
    const requestableLayers = this.getRequestableLayers_(this.olMap);
    this.aLayers = requestableLayers;

    if (this.aLayers.length > 0) {
      this.currentLayer = this.aLayers[0].name;
      this.layer = this.aLayers[0];
    }

    // Réinit variables de fonctionnement
    delete this.sFilterGeom;
    delete this.olFilterGeom;
    this.bFilterGeomEnabled = true;

    // Initialisation des tabs
    setTimeout(() => {
      this.initTabsFromLayers_();
      this.iSelectedTab = null;
    });
  }

  /**
   * Initialise les onglets de l'outil à partir des couches requêtables
   */
  private initTabsFromLayers_() {
    this.aTabs = [];

    for (const layer of this.aLayers) {
      this.aTabs.push({
        label: layer.name,
        value: layer.table,
        layer_id: layer.layer_id,
        selected: false,
      });
    }

    this.oTabsEmitter.next(this.aTabs);
  }

  /**
   * Liste des couches requêtables
   * @param olMap
   * @returns
   */
  private getRequestableLayers_(olMap: Map): Array<any> {
    const aRequestableLayers = [];
    const aLayers = this.mapManagerService.getLayersArray(olMap);

    for (let i = 0; i < aLayers.length; i++) {
      const oLayer = aLayers[i];
      if (oLayer.is_requestor_layer === true && oLayer.have_select_right === true) {
        if (this.vitisCheck.isDefAndNotNull(oLayer.json_columns)) {
          try {
            const aJsonColumns = JSON.parse(oLayer.json_columns);

            // Si au moins une des colonnes possède requestor === true
            if (this.vitisCheck.isDefAndNotNull(aJsonColumns.find((column: any) => {
              return column.requestor === true;
            }))) {
              aRequestableLayers.push(oLayer);
            }
          } catch (error) {
            console.error('json_columns not valid', oLayer.json_columns, oLayer);
          }
        }
      }
    }

    return aRequestableLayers.sort((a, b) => a.name > b.name && 1 || -1);
  }

  /**
   * Permet d'initialiser les colonnes de la grille de données
   */
  private async initLayerDataGrid_(): Promise<void> {
    if (this.aLayers && this.aLayers.length) {
      let columns = [];
      this.layer = this.aLayers.find(layer => layer.name === this.currentLayer);

      if (this.vitisCheck.isDefAndNotNull(this.layer.json_columns)) {

        // Colonnes de la grille
        columns = JSON.parse(this.layer.json_columns).filter(e => e && e.column).map(column => {
          return {
            align: 'left',
            field: column.column,
            title: column.name,
            translateTitle: column.name,
            resizeable: true,
            searchType: column.type,
            searchable: true,
            sortable: true,
            visible: column.requestor
          };
        });

        // Colonnes pour le filtre
        this.aFilterColumns = columns.filter(e => e && e.field && e.visible == true);
      } else {
        return null;
      }

      this.oDataGridOptions.datagridOptions = {
        module: 'vmap',
        ressource: 'vmap/layers/:id/query',
        id_field: this.layer.layer_id,
        display: true,
        columns,
        refresh: false,
        export: false,
      };

      this.initDataGrid_();
    }
  }

  /**
   * Calcule la hauteur de la table
   */
  private calcDataTableHeight(): number {
    if (this.oTool.panelSize === 1) {
      return 265;
    } else {
      return 715;
    }
  }

  /**
   * Réajuste les dimensions de la table
   */
  private autoAjustTables(): void {
    if (this.aLayers.length > 0) {
      setTimeout(() => {
        if (this.dataTableInstance &&
          this.dataTableInstance.columns) {
          this.dataTableInstance.columns.adjust();
        }
      }, 500);

      if (this.oDataGrid &&
        this.oDataGrid.tableId &&
        $('#' + this.oDataGrid.tableId).length) {
        const height = this.calcDataTableHeight();
        $('#' + this.oDataGrid.tableId + '_wrapper').find('.dataTables_scrollBody').css({
          height: `${height}px`
        });
      }
    }
  }

  /**
   * Permet d'initialiser la grille de données
   */
  private initDataGrid_(): void {
    const self = this;

    // Ordre
    const order = [2, 'desc'];

    // Row Id
    const rowId = this.oDataGridOptions.datagridOptions.id_field;

    // Paramètres de la grille de données.
    this.oDataGridOptions.datatableOptions = {
      select: {
        style: 'multi',
        selector: 'tr'
      },
      destroy: true,
      processing: true,
      serverSide: true,
      searching: false,
      lengthMenu: this.getLengthMenu_(),  // Tableau choix du nombre d'éléments par page
      pageLength: this.getPageLength_(),  // Nombre d'éléments par page
      order: order, // Ordre
      scrollX: true,
      scrollY: true,
      rowId: rowId,
      selected: 'selected',
      ajax: (data: any, callback: any, settings: any) => {
        this.getResourceData_(data).then((oResponse: any) => {
          this.oDataGrid.viewContainer.clear(); // Vide viewContainer
          callback(oResponse); // Remplace les données
        }).catch(() => {
          callback({
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          });
        });
      }
    };

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions) &&
      this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.columns)) {

      setTimeout(() => {
        // Colonnes de la liste (définies dans le mode).
        this.oDataGridOptions.datatableOptions.columns = this.getParsedColumns_();

        // Création de la liste.
        this.dataTableInstance = this.oDataGrid.initTable(this.oDataGridOptions);

        // Mise en place des événements des boutons
        this.dataTableInstance.on('click', '.datatable-btn-display', (event: any) => {
          const row = this.dataTableInstance.row($(event.currentTarget).parents('tr'));
          const data = row.data();

          // Géométrie
          let geometry = null;
          if (this.vitisCheck.isDefAndNotNull(this.layer) &&
            this.vitisCheck.isDefAndNotNull(this.layer.geomfield) &&
            this.vitisCheck.isDefAndNotNull(data[this.layer.geomfield])) {
            geometry = this.mapCoreUtilsService.getGeomFromEWKT(data[this.layer.geomfield], this.olMap);
          }

          // Vérification si la cible de l'événement comporte la classe 'centrer'
          if ($(event.currentTarget).hasClass('centrer')) {
            this.locateGeompetry_(data, geometry);
          }

          // Vérification si la cible de l'événement comporte la classe 'fiche'
          if ($(event.currentTarget).hasClass('fiche')) {
            this.openDisplayForm_(data, geometry);
          }
        });

        this.dataTableInstance.on('draw.dt', () => {
          // Réajuste les dimensions de la table
          this.autoAjustTables();
        });

        // Click event (permet de récupérer la cellule cliquée)
        this.dataTableInstance.on('click', 'td', function () {
          self.clickedCell_ = self.dataTableInstance.cell(this);
        });
      });
    }
  }

  /**
   * Retourne le tableau contenant le nombre de lignes affichables
   */
  private getLengthMenu_(): Array<Number> {
    const lengthMenu = [5, 10, 20, 50, 100];

    if (this.oDataGridOptions &&
      this.oDataGridOptions.datagridOptions &&
      this.oDataGridOptions.datagridOptions.max_rows) {

      const iMaxRows = this.oDataGridOptions.datagridOptions.max_rows;

      for (let i = lengthMenu.length - 1; i >= 0; i--) {
        if (lengthMenu[i] > iMaxRows) {
          lengthMenu.splice(i, 1);
        }
      }
    }

    return lengthMenu;
  }

  /**
   * Retourne le nombre d'éléments à afficher
   */
  private getPageLength_(): Number {
    let pageLength = this.pageLength;

    // Concordance avec le tableau de choix
    const aLengthMenu = this.getLengthMenu_();
    if (aLengthMenu.indexOf(pageLength) === -1) {
      if (pageLength > aLengthMenu[aLengthMenu.length - 1]) {
        pageLength = aLengthMenu[aLengthMenu.length - 1];
      } else {
        pageLength = 20;
      }
    }

    return pageLength;
  }

  /**
   * Récupère les colonnes définies dans le mode au format BootstrapTable
   * @returns Array
   */
  private getParsedColumns_(): Array<any> {
    const aColumns: any = [{
      targets: 0,
      data: null,
      defaultContent: '',
      orderable: false,
      className: 'select-checkbox',
      width: '40px'
    }];

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      // Colonne boutons
      if (this.oDataGridOptions.datagridOptions.display === true) {
        let defaultContent = '<div style="display: inline-flex; text-align: center;">';
        const sCenter = this.translate.instant('REQUESTOR_CENTER_ON_OBJECT');
        const sDescription = this.translate.instant('REQUESTOR_OBJECT_DESCRIPTION');

        if (this.oDataGridOptions.datagridOptions.display === true) {
          if (this.layer.is_geometry_layer) {
            defaultContent += '<div title="' + sCenter + '"><span class="datatable-btn datatable-btn-display icon-vitis-search centrer"></span></div>';
          }
          defaultContent += '<div title="' + sDescription + '"><span class="datatable-btn datatable-btn-display fas fa-file fiche"></span></div>';
        }

        defaultContent += '</div>';

        aColumns.push({
          targets: -1,
          data: null,
          orderable: false,
          defaultContent,
          width: '90px'
        });
      }

      // Colonnes du mode
      if (this.vitisCheck.isArray(this.oDataGridOptions.datagridOptions.columns)) {
        const aUserPrivileges = this.auth.getPrivileges();
        for (let i = 0; i < this.oDataGridOptions.datagridOptions.columns.length; i++) {
          const column = this.oDataGridOptions.datagridOptions.columns[i];
          // Privilège obligatoire pour afficher la colonne.
          if (this.vitisCheck.isDefAndNotNull(column.privilege)) {
            if (aUserPrivileges.indexOf(column.privilege) == -1) {
              continue;
            }
          }

          aColumns.push({
            title: column.translateTitle,
            data: column.field,
            name: column.field,
            visible: column.visible === false ? false : true,
            render: (data: any, type: any, row: any, meta: any) => {
              if (column.template) {
                data = '<field-render column_number="' + i + '">' + encodeURIComponent(JSON.stringify(row)) + '</field-render>';
              }
              return data;
            },
            orderable: column.sortable,
            // Alignement du contenu de la colonne.
            className: 'table-align-' + (column.align ? column.align : 'left')
          });
        }
      }
    }

    return aColumns;
  }

  /**
   * Génére l'url pour récupérer les données
   * @returns {string}
   */
  protected getRessourceUrl(): string {
    const sRessourceUrl = this.oDataGridOptions.datagridOptions.ressource;
    return environment.properties.app_api_url + '/' + sRessourceUrl.replace(':id', this.layer.layer_id);
  }

  /**
   * Récupère les donéées au format DataTables
   * @param  {any} params
   * @returns Promise
   */
  private getResourceData_(params: any): Promise<any> {
    return new Promise((resolve, reject) => {
      this.oDataGridOptions.loadingData_ = true;

      // Affiche la géométrie d'intersection si besoin
      this.displayGeomSelectionFilter();

      // Lance la requête
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: this.getRessourceUrl(),
        ajaxLoader: false,
        params: this.getRequestParams_(params),
        success: (resp: any) => {
          this.oDataGridOptions.loadingData_ = false;

          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (this.vitisCheck.isDefAndNotNull(resp.data.status === 1)) {
              if (this.vitisCheck.isArray(resp.data.data)) {
                oReturn.data = resp.data.data;
              }
              if (this.vitisCheck.isDefAndNotNull(resp.data.total_row_number)) {
                oReturn.recordsTotal = resp.data.total_row_number;
                oReturn.recordsFiltered = resp.data.total_row_number;
              }
            }
          }

          resolve(oReturn);
        },
        error: (err: any) => {
          console.error('error while loading values :', err);

          this.oDataGridOptions.loadingData_ = false;
          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          resolve(oReturn);
        }
      });
    });
  }

  /**
   * Parse les params envoyés par DataTables pour créer des paramètres interprétables par l'API Vitis
   * @param  {any} params
   * @returns any
   */
  private getRequestParams_(params: any): any {
    const requestParams: any = {};

    // Order
    if (this.vitisCheck.isDefAndNotNull(params.order)) {
      if (this.vitisCheck.isDefAndNotNull(params.order[0])) {
        if (this.vitisCheck.isString(params.order[0].dir)) {
          requestParams.sort_order = params.order[0].dir.toUpperCase();
        }
        if (this.vitisCheck.isNumber(params.order[0].column)) {
          const iColumnId = params.order[0].column;
          if (this.vitisCheck.isDefAndNotNull(params.columns[iColumnId])) {
            if (this.vitisCheck.isString(params.columns[iColumnId].data)) {
              requestParams.order_by = params.columns[iColumnId].data;
            }
          }
        }
      }
    }

    // Limit
    if (this.vitisCheck.isNumber(params.length)) {
      requestParams.limit = params.length;
    }

    // Offset
    if (this.vitisCheck.isNumber(params.start)) {
      requestParams.offset = params.start;
    }

    // Filter
    if (this.oFilter.nodes.length > 0) {
      requestParams.filter = this.oFilter.value;
    }

    // Filtre géométrique
    if (this.bFilterGeomEnabled && this.vitisCheck.isDefAndNotNull(this.sFilterGeom)) {
      requestParams.intersect_geom = this.sFilterGeom;
      requestParams.result_srid = this.olMap.getView().getProjection().getCode().substring(5);
      requestParams.intersect_buffer = this.vmapSelectService.getRealScaleBuffer_(this.layer, this.sFilterGeom, this.olMap);
    }

    // Attributs
    if (this.vitisCheck.isDefAndNotNull(params.columns)) {
      // détecte la présence d'attribut supplementaire requis par la liste
      const extra_attributs = (this.oDataGridOptions.datagridOptions.extra_attributs) ? this.oDataGridOptions.datagridOptions.extra_attributs : [];
      requestParams.attributs = params.columns.filter(
        (aColumn: any) => this.vitisCheck.isString(aColumn.data)
      ).map(
        (aColumn: any) => aColumn.data
      ).concat(extra_attributs).join('|');
    }

    return requestParams;
  }

  /**
   * Affiche la géométrie sur la carte et zoom dessus
   * @param data
   */
  private locateGeompetry_(data: any, geometry: Geometry) {

    // Texte à afficher
    let popupText = null;
    if (this.vitisCheck.isDefAndNotNull(this.layer) &&
      this.vitisCheck.isDefAndNotNull(this.layer.select_field) &&
      this.vitisCheck.isDefAndNotNull(data[this.layer.select_field])) {
      popupText = data[this.layer.select_field];
    }

    // Ferme la popup de localisation
    this.closeLocationPopup_();

    // Nouvelle popup
    this.locationPopup = this.mapPopup.displayPopup(popupText, new Feature({ geometry: geometry }), this.olMap, { hideCloseBtn: true });

    // Centrer la carte sur la géométrie
    this.mapCoreUtilsService.centerMapOnGeom(this.olMap, geometry, this.vmapToolsService.getCenterPadding());
  }

  /**
   * Ouvre le formulaire de consultation
   * @param data
   */
  private openDisplayForm_(data: any, geometry: Geometry) {

    // Ferme la popup de localisation
    this.closeLocationPopup_();

    // Id de l'enregistrement
    let id = null;
    if (this.vitisCheck.isDefAndNotNull(this.layer) &&
      this.vitisCheck.isDefAndNotNull(this.layer.idfield) &&
      this.vitisCheck.isDefAndNotNull(data[this.layer.idfield])) {
      id = data[this.layer.idfield];
    } else {
      console.error('cannot get selection id', data, this.layer);
      return null;
    }

    this.vmapSelectService.selectById(id, this.layer, this.olMap).then((aData) => {

      // Set les nouvelles sélections
      const aSelections = [];
      if (aData && aData[0]) {
        for (let i = 0; i < aData.length; i++) {
          aSelections.push({
            data: aData[i],
            oLayer: this.layer,
            geometry: geometry,
          });
        }
      }

      // Affiche les popups
      if (this.vitisCheck.isDefAndNotNull(geometry) && this.layer.is_geometry_layer) {
        this.vmapSelectService.displayPopups(this.olMap, aSelections);
      }

      // Sélectionne le mode (modify/insert) avec la sélection qui va avec
      this.vmapSelectService.closeSelections();
      this.vmapSelectService.setSelectMode('modify');
      this.vmapSelectService.setSelection(aSelections);

      // Centrer la carte sur la géométrie
      if (this.vitisCheck.isDefAndNotNull(geometry) && this.layer.is_geometry_layer) {
        this.mapCoreUtilsService.centerMapOnGeom(this.olMap, geometry, this.vmapToolsService.getCenterPadding());
      }
    });
  }

  /**
   * Ferme la popup de localisation
   */
  private closeLocationPopup_() {
    // Ferme la précédente popup
    if (this.locationPopup) {
      this.locationPopup.close();
      this.locationPopup = null;
    }
  }

  /**
   * Retourne la tab en fonction du layer_id passé en paramètre
   * @param iLayerId
   * @returns
   */
  private getTabByLayerId_(iLayerId: string | number) {
    return this.aTabs.find(tab => tab && tab.layer_id == iLayerId);
  }

  /**
   * Ouvre le requêteur après une requête géométrique
   */
  private openFromGeomSelection_(options: any) {
    if (options.layer &&
      options.geom) {

      if (options.layer.layer_id) {
        const tab = this.getTabByLayerId_(options.layer.layer_id);
        const tabIndex = this.aTabs.indexOf(tab);

        if (tab && tabIndex !== -1) {
          this.iSelectedTab = tabIndex;
        }
      }

      if (!this.vitisCheck.isDefAndNotNull(this.iSelectedTab)) {
        this.iSelectedTab = 0;
      }

      this.bFilterGeomEnabled = true;
      this.sFilterGeom = options.geom;
      this.olFilterGeom = options.olGeometry;
      this.vmapToolsService.setBottomPanel(1, 'requestor');
      this.oTool.open();
      this.oSelectedTabEmitter.next(this.iSelectedTab);
    }
  }

  /**
   * Initialise le vector Layer pour ajouter les sélections à la carte
   */
  private initDrawLayer(): void {
    this.olMap.set('requestor_draw_layer', this.mapCoreDrawService.initDrawLayer(this.olMap, null));
  }

  /**
   * Affiche la géométrie de this.olFilterGeom si bFilterGeomEnabled vaut true
   */
  private displayGeomSelectionFilter(bForceDelete = false): void {

    const olDrawLayer = this.olMap.get('requestor_draw_layer');
    olDrawLayer.getSource().clear();

    if (this.bFilterGeomEnabled && this.vitisCheck.isDefAndNotNull(this.olFilterGeom) && bForceDelete !== true) {
      olDrawLayer.getSource().addFeature(new Feature({ geometry: this.olFilterGeom }));
    }
  }

  /**
   * Récupère la donnée pour l'export
   * @param sExportType
   * @param aConfig
   * @returns
   */
  private getDataForExport(sExportType: string, aConfig: any = {}): Promise<any[]> {
    return new Promise((resolve, reject) => {

      let iNbRows: number;
      switch (sExportType) {
        case 'current_page':
          iNbRows = this.dataTableInstance.data().count();
          this.getDataGridDataForExport(iNbRows, aConfig).then(aData => {
            resolve(aData);
          });
          break;
        case 'all_rows':
          vitisAlert.alert({
            text: this.translate.instant('REQUESTOR_EXTRACT_TYPE_ALL_DATA_WARNING'),
            type: 'warning',
            showCancelButton: true,
            confirmButtonText: this.translate.instant('REQUESTOR_EXTRACT_TYPE_ALL_DATA_WARNING_OK'),
            cancelButtonText: this.translate.instant('REQUESTOR_EXTRACT_TYPE_ALL_DATA_WARNING_CANCEL'),
          }).then((result) => {
            if (result.isDismissed === false) {
              iNbRows = this.dataTableInstance.page.info().recordsTotal;
              this.getDataGridDataForExport(iNbRows, aConfig).then(aData => {
                resolve(aData);
              });
            }
          });
          break;
        case 'selected_rows':
          iNbRows = this.dataTableInstance.data().count();
          const aSelectedRows = this.dataTableInstance.rows({ selected: true }).data();
          this.getDataGridDataForExport(iNbRows, { ...aConfig, ...{ aSelectedRows } }).then(aData => {
            resolve(aData);
          });
          break;
        default:
          break;
      }
    });
  }

  /**
   * récupère les lignes visible dans data-grid, remplace les clé et transforme le tout en array type json
   * @param iNbRows
   * @param aConfig
   * @returns
   */
  private getDataGridDataForExport(iNbRows: number, aConfig: any = {}): Promise<any[]> {
    return new Promise((resolve, reject) => {
      const oDataOption = this.dataTableInstance.ajax.params();
      oDataOption.length = iNbRows;

      if (iNbRows) {
        this.oDataGridOptions.datatableOptions.ajax(oDataOption, (oReturn: any) => {
          const aData: any = oReturn.data;
          if (this.vitisCheck.isDefAndNotNull(aData)) {
            const aFinalData = this.parseColumnsForExport(aData, aConfig);
            resolve(aFinalData);
          } else {
            reject();
          }
        }, null);
      }
    });
  }

  /**
   * Traite le contenu de la donnée en enlevant les colonnes non nécessaires
   * @param aData
   * @param aConfig
   * @returns
   */
  private parseColumnsForExport(aData: Array<any>, aConfig: any = {}) {
    const aColumns: any[] = this.oDataGridOptions.datatableOptions.columns;
    const aFinalData: any[] = [];
    for (let i = 0; i < aData.length; i++) {
      const aTmpItem: any = {};
      let isSelected = false;

      for (const oColumn of aColumns) {
        // élimine les colonnes useless au début du tableau
        // boite à cocher bouton d'ouverture de formulaire
        // et teste si la colonne est exportable
        if ('title' in oColumn && 'data' in oColumn) {

          // décide si la signe fait partie de celles sélectionnées
          if (this.vitisCheck.isDefAndNotNull(aConfig.aSelectedRows)) {
            isSelected = this.isSelectedRow(aData[i], aConfig.aSelectedRows);
          }

          // si la colonne à une correspondance dans les données
          // on récupère la donnée en remplaçant la clé par le label de la colonne
          const sKey: string = (aConfig.replace_label) ? oColumn.title : oColumn.data;
          if (this.vitisCheck.isDefAndNotNull(aData[i][oColumn.data])) {
            aTmpItem[sKey] = aData[i][oColumn.data];
          } else {
            aTmpItem[sKey] = aConfig.no_data_value;
          }
        }
      }

      if (this.vitisCheck.isDefAndNotNull(aConfig.aSelectedRows)) {
        if (isSelected) {
          aFinalData.push(aTmpItem);
        }
      } else {
        aFinalData.push(aTmpItem);
      }
    }
    return aFinalData;
  }

  /**
   * Retourne true/false si la ligne fait partie des sélectionnées
   * @param oRow
   * @param aSelectedRows
   * @returns
   */
  private isSelectedRow(oRow: any, aSelectedRows: any[]): boolean {
    for (let i = 0; i < aSelectedRows.length; i++) {
      const oSelectedRow = aSelectedRows[i];

      let isSelected = true;
      for (const key in oRow) {
        if (Object.prototype.hasOwnProperty.call(oRow, key)) {
          if (oSelectedRow[key] !== oRow[key]) {
            isSelected = false;
          }
        }
      }

      if (isSelected) {
        return true;
      }
    }

    return false;
  }
}
