import { Component, ViewChild, AfterViewInit, Input, OnChanges, Output, EventEmitter, OnDestroy } from '@angular/core';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-vmap-tools-location-form',
  templateUrl: './vmap-tools-form.component.html',
  styleUrls: ['./vmap-tools-form.component.scss']
})
export class VmapToolsFormComponent implements AfterViewInit, OnChanges, OnDestroy {

  @ViewChild(FormreaderComponent) oFormReader: FormreaderComponent;

  @Input() sDefinition;
  @Input() sFormName;
  @Input() sDatas;

  @Output() getFormChanges = new EventEmitter();

  private datas: any;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(private vitisClone: VitisClone) { }

  ngOnChanges(changes): void {
    if (this.oFormReader) {
      if (changes) {
        if (changes.sFormName && changes.sFormName.currentValue) {
          this.oFormReader.setFormDefinitionName(changes.sFormName.currentValue);
        }
        if (changes.sDefinition && changes.sDefinition.currentValue && changes.sDatas && changes.sDatas.currentValue) {
          this.oFormReader.initForm(changes.sDefinition.currentValue, this.sFormName, this.vitisClone.clone(changes.sDatas.currentValue));
        }
        if (changes.sDatas && changes.sDatas.currentValue) {
          this.datas = this.vitisClone.clone(changes.sDatas.currentValue);
          this.oFormReader.setFormValues(this.vitisClone.clone(changes.sDatas.currentValue));
        }
        if (this.getFormChanges) {
          this.subscriptions.push(this.oFormReader.oFormValues.valueChanges.subscribe(form => {
            this.getFormChanges.next(form);
          }));
        }
      }
    }
  }

  ngAfterViewInit(): void {
    setTimeout(() => {
      this.datas = this.vitisClone.clone(this.sDatas);
      this.oFormReader.initForm(this.sDefinition, this.sFormName, this.vitisClone.clone(this.sDatas));

      if (this.getFormChanges) {
        this.subscriptions.push(this.oFormReader.oFormValues.valueChanges.subscribe(form => {
          this.getFormChanges.next(form);
        }));
      }
    });
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }
}
