import { Component, OnInit, Input, Output, EventEmitter, OnDestroy } from '@angular/core';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { VitisClone, VitisRequest, vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { BehaviorSubject, Subject } from 'rxjs';
import VectorSource from 'ol/source/Vector';
import { Geometry, LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon } from 'ol/geom';
import VectorLayer from 'ol/layer/Vector';
import { environment } from 'src/environments/environment';
import { Feature } from 'ol';
import { VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { Fill, Stroke, Style } from 'ol/style';
import CircleStyle from 'ol/style/Circle';
import { Snap } from 'ol/interaction';
import { fromExtent } from 'ol/geom/Polygon';
import { transform } from 'ol/proj';
import { TranslateService } from '@ngx-translate/core';
import { unByKey } from 'ol/Observable';

import * as modif_cord_json from './../forms/modif_coord.json';
import * as snap_form from './../forms/snap_form.json';

@Component({
  selector: 'app-vmap-tools-geometry-form',
  templateUrl: './vmap-tools-geometry-form.component.html',
  styleUrls: ['./vmap-tools-geometry-form.component.scss']
})
export class VmapToolsGeometryFormComponent implements OnInit, OnDestroy {

  @Input() map;
  @Input() oLayer;
  @Input() currFeature;
  @Input() geomEventValue: string;
  @Output() geomEventValueChange: EventEmitter<string> = new EventEmitter<string>();

  @Output() subscribeFeature = new EventEmitter();

  public sFormName = 'insert';
  public sDefinition: any;
  public sDatasForm: any = {};

  private geomType;
  public isMultiGeom = false;
  public geomEvent = new BehaviorSubject<string | any>('');
  public isGeometryLayer = false;

  private feature: Feature<Geometry>;
  private vectorSource: VectorSource<Geometry>;
  private vectorLayer: VectorLayer<VectorSource<Geometry>>;
  private vectorLayerSnap: VectorLayer<VectorSource<Geometry>>;

  private layersSnapping: Array<number>;
  private featuresSnap: Array<Feature<Geometry>>;
  private loadSnap: boolean = false;
  public isLoadingSnap: boolean;
  private xhrSnap;
  private currXhrExist = new Subject<boolean>();
  private limitFeaturesSnap = 2000;

  private interactionModify;
  private interactionSelect;
  private interactionSnap;
  private interactionRemove;
  private interactionAdd;

  private geomEventSubscribe;
  private selectModeSubscribe;
  public mode;
  private rightPanelEmitterSubscribe;

  public listGeomtypesId = 'listGeomtypesId_' + Date.now();

  private listenerKeyInteractionModify;
  private listenerKeyInteractionAddFeature;
  private listenerKeyInteractionRemoveFeature;

  constructor(
    private vitisClone: VitisClone,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private mapCoreDrawService: MapCoreDrawService,
    private mapSelectService: VmapSelectService,
    private mapUtils: MapCoreUtilsService,
    private mapToolsService: VmapToolsService,
    private translate: TranslateService
  ) { }

  ngOnInit() {
    if (this.currFeature) {
      this.feature = this.currFeature.clone();
      setTimeout(() => {
        this.geomEvent.next('modify_object');
      });
    }
    if (this.oLayer) {
      this.mode = this.mapSelectService.getSelectMode();
      if (this.vitisCheck.isDefAndNotNull(this.oLayer['ms_geomtype']) &&
        this.vitisCheck.isDefAndNotNull(this.oLayer['ms_geomtype']['label'])) {
        let geomType = '';
        if (this.oLayer.is_multi_edit === true) {
          this.isMultiGeom = true;
          geomType += 'MULTI';
        }
        if (this.oLayer['ms_geomtype']['label'] === 'GEOMETRY') { // La géométrie de la couche est de type GEOMETRY
          this.isGeometryLayer = true;
          geomType += 'POINT';
        } else { // La géométrie de la couche est de type POINT, LIGNE OU POLYGONE
          geomType += this.oLayer['ms_geomtype']['label'];
        }
        this.geomType = geomType;
      } else { // La couche ne possède pas de géométrie
        this.translate.get('ERROR_INSERT_LAYER_OBJECT').subscribe(async (translation) => {
          vitisAlert.alert({
            text: translation,
            showCancelButton: true
          });
        });
      }
      setTimeout(() => {
        this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
      });

      this.selectModeSubscribe = this.mapSelectService.getSelectModeObs().subscribe((mode: string) => {
        this.mode = mode;
      });

      this.geomEventSubscribe = this.geomEvent.subscribe(geomvalue => {
        if (this.geomEventValue != geomvalue) {
          this.geomEventValue = geomvalue;
          if (geomvalue) {
            this.geomEventValueChange.emit(this.geomEventValue);
            switch (geomvalue) {
              case 'modify_object':
                this.deleteCurrentInteraction(['select', 'modify', 'add', 'remove']);
                this.initDrawing();
                this.modifyGeometry();
                break;
              case 'add_object':
                this.deleteCurrentInteraction(['select', 'modify', 'remove']);
                this.initDrawing();
                this.addGeometry();
                break;
              case 'del_object':
                this.deleteCurrentInteraction(['select', 'add']);
                this.removeGeometry();
                break;
              case 'snap':
                this.deleteCurrentInteraction(['modify', 'remove', 'add']);
                this.snap();
                break;
              case 'modify_coord_object':
                this.deleteCurrentInteraction(['select', 'modify', 'add', 'remove']);
                this.modifyCoordGeometry();
                break;
            }
          }
        }
      });

      const this_ = this;
      this.map.on('moveend', function () {
        if (this_.loadSnap && this_.vectorLayerSnap) {
          this_.loadSnappingDatas();
        }
      });
    }
    this.rightPanelEmitterSubscribe = this.mapToolsService.getRightPanelEmiter().subscribe(formSize => {
      if (formSize === 0) {
        this.deleteAllInteraction();
      }
    });
  }

  ngOnDestroy() {
    this.deleteAllInteraction();
  }

  /**
   * Initialise le vector Layer et ajoute la feature si elle existe
   */
  private initDrawing(): void {
    this.vectorSource = new VectorSource();
    this.vectorLayer = this.mapCoreDrawService.initDrawLayer(this.map, this.vectorSource);
    if (this.feature) {
      this.vectorSource.addFeature(this.feature);
    } else {
      this.getInitFeature();
    }
  }

  /**
   * Initialise une nouvelle feature grace au type de la layer
   */
  private getInitFeature(): void {
    let geometry;
    switch (this.geomType) {
      case 'POLYGON':
        geometry = new Polygon([]);
        break;
      case 'MULTIPOLYGON':
        geometry = new MultiPolygon([]);
        break;
      case 'POINT':
        geometry = new Point([]);
        break;
      case 'MULTIPOINT':
        geometry = new MultiPoint([]);
        break;
      case 'LINESTRING':
        geometry = new LineString([]);
        break;
      case 'MULTILINESTRING':
        geometry = new MultiLineString([]);
    }
    this.feature = new Feature({ geometry });
    this.subscribeFeature.next(this.feature);
  }

  /**
   * Retourne si une/des feature(s) sont présente sur le vector source de la map
   * @returns
   */
  public featureExist(): boolean {
    if (this.vectorSource) {
      if (this.vectorSource.getFeatures().length > 0) {
        return true;
      }
    }
    return false;
  }

  /**
   * Event appelé lors du clique sur 'Modifier la géométrie'
   */
  private modifyGeometry(): void {
    const { select, interaction, snap } = this.mapCoreDrawService.addDrawInteraction('modify', this.map, this.vectorLayer, { selectFeature: this.feature, snap: this.interactionSnap, getInteractions: true });
    this.interactionModify = interaction;
    this.interactionSelect = select;
    this.interactionSnap = snap;

    this.listenerKeyInteractionModify = this.interactionModify.on('modifyend', (evt: any) => {
      const feature = evt.features.getArray()[0];
      this.feature = new Feature({ geometry: feature.getGeometry() });
      this.subscribeFeature.next(this.feature);
    });
  }

  /**
   * Event appelé lors du clique sur 'Ajouter un object'
   */
  private addGeometry(): void {
    if (this.geomType) {
      const interactionFormat = this.isMultiGeom ? this.geomType.toLowerCase().substr(5) : this.geomType.toLowerCase();
      const { interaction, snap } = this.mapCoreDrawService.addDrawInteraction(interactionFormat, this.map, this.vectorLayer, { snap: this.interactionSnap, getInteractions: true });
      this.interactionAdd = interaction;
      this.interactionSnap = snap;
      this.listenerKeyInteractionAddFeature = this.vectorSource.on('addfeature', (evt: any) => {
        const geometryAdded: Polygon | Point | LineString = evt.feature.getGeometry();
        if (this.isMultiGeom) {
          const multiGeom: any = this.feature.getGeometry();
          if (multiGeom) {
            switch (interactionFormat) {
              case 'polygon':
                multiGeom.appendPolygon(geometryAdded);
                break;
              case 'point':
                multiGeom.appendPoint(geometryAdded);
                break;
              case 'linestring':
                multiGeom.appendLineString(geometryAdded);
                break;
            }
            this.feature = new Feature({ geometry: multiGeom });
            this.subscribeFeature.next(this.feature);
          }
        } else {
          const features = this.vectorSource.getFeatures();
          if (features && features.length > 1) {
            const currFeature = features[0];
            this.vectorSource.removeFeature(currFeature);
          }
          this.feature = new Feature({ geometry: geometryAdded });
          this.subscribeFeature.next(this.feature);
        }
      });
    }
  }

  /**
   * Event appelé lors du clique sur 'Supprimer un object'
   */
  private removeGeometry(): void {
    const interactionFormat = this.geomType.toLowerCase().substr(5);
    let geometries = [];
    const currGeom: any = this.feature.getGeometry();
    if (currGeom) {
      switch (interactionFormat) {
        case 'polygon':
          geometries = currGeom.getPolygons();
          break;
        case 'point':
          geometries = currGeom.getPoints();
          break;
        case 'linestring':
          geometries = currGeom.getLineStrings();
          break;
      }
    }
    this.vectorSource = new VectorSource();
    this.vectorLayer = this.mapCoreDrawService.initDrawLayer(this.map, this.vectorSource);
    if (geometries.length > 0) {
      this.vectorSource.addFeatures(geometries.map(geom => new Feature({ geometry: geom })));
    }
    this.interactionRemove = this.mapCoreDrawService.addDrawInteraction('remove', this.map, this.vectorLayer, { snap: this.interactionSnap });

    this.listenerKeyInteractionRemoveFeature = this.vectorSource.on('removefeature', (evt: any) => {
      const features = this.vectorLayer.getSource().getFeatures();
      const geomtype = this.feature.getGeometry().getType();
      const interactionFormatFeature = geomtype.toLowerCase().substr(5);
      let geometry;
      switch (interactionFormatFeature) {
        case 'polygon':
          geometry = new MultiPolygon([]);
          features.forEach(feature => geometry.appendPolygon(feature.getGeometry()));
          break;
        case 'point':
          geometry = new MultiPoint([]);
          features.forEach(feature => geometry.appendPoint(feature.getGeometry()));
          break;
        case 'linestring':
          geometry = new MultiLineString([]);
          features.forEach(feature => geometry.appendLineString(feature.getGeometry()));
          break;
      }
      if (features.length) {
        this.feature = new Feature({ geometry });
      } else {
        this.feature = null;
      }
      this.subscribeFeature.next(this.feature);
    });
  }

  /**
   * Event appelé lors du clique sur 'Accrochage universel'
   */
  private snap(): void {
    const sDatas = {
      type: 1,
      pixelTolerance: 15,
      limit: this.limitFeaturesSnap
    };

    if (this.interactionSnap) {
      sDatas.pixelTolerance = this.interactionSnap.pixelTolerance_;
      if (this.interactionSnap.vertex_) {
        sDatas.type = 2;
        if (this.interactionSnap.edge_) {
          sDatas.type = 3;
        }
      }
    }
    if (this.vectorLayerSnap) {
      const style: any = this.vectorLayerSnap.getStyle();
      if (style.getFill() || style.getStroke() || style.getImage()) {
        sDatas['visibility'] = true;
      }
    }
    const definition = this.vitisClone.clone(snap_form['default']);


    if (this.interactionSnap) {
      sDatas.pixelTolerance = this.interactionSnap.pixelTolerance_;
    }

    const layersFields = {
      'fields': [
      ]
    };
    const templateLayer = {
      'type': 'checkbox',
      'value': false,
      'nb_cols': 12
    };
    this.mapSelectService.getSelectLayers().forEach(layer => {
      const layerRow = this.vitisClone.clone({ ...this.vitisClone.clone(templateLayer), 'name': layer.layer_id, 'label': layer.name });
      if (this.layersSnapping && this.layersSnapping.length) {
        sDatas[layer.layer_id] = this.layersSnapping.includes(layer.layer_id);
      }
      layersFields.fields.push(this.vitisClone.clone(layerRow));
    });
    definition.update.rows.unshift(this.vitisClone.clone(layersFields));
    this.loadForm({
      sDefinition: this.vitisClone.clone(definition),
      sFormName: 'update',
      sDatasForm: this.vitisClone.clone(sDatas)
    });
  }

  /**
   * Event appelé lors de la validation du formulaire 'Accrochage universel'
   * @param event
   */
  private createSnap(event): any {
    this.limitFeaturesSnap = event.limit;
    this.layersSnapping = Object.keys(event).filter(key => Number.isInteger(+key) && event[key]).map(layer_id => +layer_id);

    if (this.interactionSnap) {
      this.map.removeInteraction(this.interactionSnap);
      this.interactionSnap = null;
      if (this.vectorLayerSnap) {
        this.vectorLayerSnap.setStyle(null);
        this.map.removeLayer(this.vectorLayerSnap);
        this.vectorLayerSnap = null;
      }
    }

    const vectorSourceSnap = new VectorSource();

    const selectStyle = new Style({
      fill: new Fill({
        color: 'rgba(255, 255, 255, 0.2)',
      }),
      stroke: new Stroke({
        color: '#337ab7',
        width: 2,
      }),
      image: new CircleStyle({
        radius: 3,
        fill: new Fill({
          color: '#337ab7',
        }),
      }),
    });

    this.vectorLayerSnap = new VectorLayer({
      source: vectorSourceSnap,
      style: event.visibility ? selectStyle : new Style(),
      zIndex: 1000,
    });
    this.map.addLayer(this.vectorLayerSnap);

    this.interactionSnap = new Snap({
      source: this.vectorLayerSnap.getSource(),
      pixelTolerance: event.pixelTolerance,
      vertex: event.type == 3 || event.type == 2,
      edge: event.type == 3
    });

    this.loadSnap = this.layersSnapping.length > 0;
    if (this.loadSnap === true) {
      this.loadSnappingDatas();
    } else {
      this.isLoadingSnap = null;
    }
  }

  /**
   * Event appelé lors du clique sur 'Modifier les coordonnées'
   */
  private modifyCoordGeometry() {
    const featureClone = this.feature.clone();
    const geometry: any = featureClone.getGeometry();
    const sDatas = {
      format: 'WKT',
      projection: 'EPSG:4326',
      geomtype: this.geomType,
      type: geometry.getType()
    };
    if (geometry.getCoordinates().length > 0) {
      const EWKT = this.mapUtils.getEWKTFromGeomAndProj(geometry, this.map, '4326');
      const WKT = this.mapUtils.getWKTFromEWKT(EWKT);
      sDatas['wkt'] = WKT.geom;
      sDatas['projection'] = WKT.proj;
    }
    const sDefinition = modif_cord_json['default'];
    sDefinition.update.initEvent = 'initEventModifCord()';
    this.loadForm({
      sDefinition: this.vitisClone.clone(sDefinition),
      sFormName: 'update',
      sDatasForm: this.vitisClone.clone(sDatas)
    });
  }

  /**
   * Event appelé lors de la validation du formulaire 'Modifier les coordonnées'
   * @param event
   */
  private createModifyCoordGeometry(event) {
    let sEWKTGeom;
    switch (event.format) {
      case 'WKT':
        if (event.wkt) {
          sEWKTGeom = this.mapUtils.getEWKTFromWKT({ proj: event.projection, geom: event.wkt });
        }
        break;
      case 'Geojson':
        if (event.geojson) {
          sEWKTGeom = this.mapUtils.getEWKTFromGeojson(JSON.parse(event.geojson), event.projection);
        }
        break;
      case 'XY':
        if (event.longitude && event.latitude) {
          const coordinates = transform([event.longitude, event.latitude], this.map.getView().getProjection(), event.projection);
          sEWKTGeom = this.mapUtils.getEWKTFromGeojson({
            type: event.type,
            coordinates
          }, event.projection);
        }
        break;
    }
    if (sEWKTGeom) {
      const geometry: any = this.mapUtils.getGeomFromEWKT(sEWKTGeom, this.map);
      this.feature = new Feature({ geometry });
      this.subscribeFeature.next(this.feature);
      this.mapUtils.centerMapOnGeom(this.map, this.feature.getGeometry());
    }
  }

  /**
   * Supprime les interactions passées en paramètre
   * @param interactionsToDel Tableau avec les interactions à supprimer de la carte
   */
  public deleteCurrentInteraction(interactionsToDel: Array<string> = []): void {
    if (this.listenerKeyInteractionModify) {
      unByKey(this.listenerKeyInteractionModify);
    }
    if (this.listenerKeyInteractionAddFeature) {
      unByKey(this.listenerKeyInteractionAddFeature);
    }
    if (this.listenerKeyInteractionRemoveFeature) {
      unByKey(this.listenerKeyInteractionRemoveFeature);
    }
    if (this.vectorSource) {
      this.vectorSource.clear();
    }
    if (interactionsToDel.includes('select') && this.interactionRemove) {
      this.map.removeInteraction(this.interactionSelect);
      this.interactionSelect = null;
      if (this.vectorLayer) {
        this.map.removeLayer(this.vectorLayer);
        this.vectorLayer = null;
      }
    }
    if (interactionsToDel.includes('modify') && this.interactionModify) {
      this.map.removeInteraction(this.interactionModify);
      this.interactionModify = null;
    }
    if (interactionsToDel.includes('add') && this.interactionAdd) {
      this.map.removeInteraction(this.interactionAdd);
      this.interactionAdd = null;
    }
    if (interactionsToDel.includes('remove') && this.interactionRemove) {
      this.map.removeInteraction(this.interactionRemove);
      this.interactionRemove = null;
    }
    if (interactionsToDel.includes('snap') && this.interactionSnap) {
      this.map.removeInteraction(this.interactionSnap);
      this.interactionSnap = null;
      if (this.vectorLayerSnap) {
        this.vectorLayerSnap.setStyle(null);
        this.map.removeLayer(this.vectorLayerSnap);
        this.vectorLayerSnap = null;
      }
    }
  }

  /**
   * Récupère les zones d'accrochage de la visualisation en cous (carte extent)
   */
  public loadSnappingDatas() {
    this.isLoadingSnap = true;
    if (this.vectorLayerSnap) {
      // Supprime les features actuelles dans le vectorlayer du snap
      const features = this.vectorLayerSnap.getSource().getFeatures();
      if (features.length > 0) {
        features.forEach(feature => this.vectorLayerSnap.getSource().removeFeature(feature));
      }
    }

    if (this.xhrSnap) {
      this.currXhrExist.next(true);
    }

    const extent = this.map.getView().calculateExtent(this.map.getSize());
    const polygonGeom = fromExtent(extent);
    const sEWKTGeometry = this.mapUtils.getEWKTFromGeom(polygonGeom, this.map);

    // Buffer d'intersection
    const intersect_buffer = this.mapSelectService.getRealScaleBuffer_(this.oLayer, sEWKTGeometry, this.map);

    // Geométrie à intersecter
    const intersect_geom = sEWKTGeometry;

    // Projection en cours
    const result_srid = this.map.getView().getProjection().getCode().substring(5);

    // Paramètres
    const oParams = {
      limit: this.limitFeaturesSnap,
      groupBy: true,
      idList: this.layersSnapping,
      d: Date.now(),
      get_geom: true,
      intersect_buffer,
      intersect_geom,
      result_srid
    };

    const this_ = this;
    this.xhrSnap = this.vitisRequest.ajaxRequest({
      method: 'GET',
      url: `${environment.properties.app_api_url}/vmap/layers/query/layer`,
      params: oParams,
      ajaxLoader: false,
      success: (resp) => {
        let geoms = [];
        if (resp && resp.data && resp.data.data) {
          resp = resp.data.data;
          Object.keys(resp).forEach(layerName => {
            const geomfield = this_.mapSelectService.getSelectLayers().find(layer => layer.name === layerName).geomfield;
            geoms = [...geoms, ...resp[layerName].data.map(data => data[geomfield])];
          });
        }
        // Ajoute les nouvelles features dans le vectorlayer du snap
        this_.featuresSnap = geoms.map(geom => new Feature({ geometry: this_.mapUtils.getGeomFromEWKT(geom, this_.map) }));
        this_.vectorLayerSnap.getSource().addFeatures(this_.featuresSnap);
        this_.isLoadingSnap = false;
        return;
      },
      error: (resp) => {
        this_.isLoadingSnap = false;
      },
      abord: this.currXhrExist
    });
  }

  /**
   * Charge le formulaire à afficher pour les zones d'accrochage ou pour la modifications des coordonnées
   * @param formProperties
   */
  private loadForm(formProperties) {
    const { sDefinition, sFormName, sDatasForm } = formProperties;
    this.sDefinition = this.vitisClone.clone(sDefinition);
    this.sFormName = this.vitisClone.clone(sFormName);
    this.sDatasForm = this.vitisClone.clone(sDatasForm);
  }

  /**
   * Ferme le formulaire pour les zones d'accochages ou pour la modifications des coordonnées
   */
  public sendCloseForm() {
    this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
  }

  /**
   * Valide le formulaire pour les zones d'accrochage ou pour la modifications des coordonnées
   * @param event
   */
  public sendUpdateForm(event) {
    if (this.geomEventValue === 'snap') {
      this.createSnap(event);
    } else if (this.geomEventValue === 'modify_coord_object') {
      this.createModifyCoordGeometry(event);
    }
    this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
  }

  /**
   * Supprime toute les interactions de la cartes + la feature en cours et les subcribe
   */
  private deleteAllInteraction() {
    if (this.feature) {
      this.feature = null;
    }
    this.deleteCurrentInteraction(['snap', 'select', 'modify', 'add', 'remove']);
    if (this.geomEventSubscribe) {
      this.geomEventSubscribe.unsubscribe();
    }
    if (this.selectModeSubscribe) {
      this.selectModeSubscribe.unsubscribe();
    }
    if (this.rightPanelEmitterSubscribe) {
      this.rightPanelEmitterSubscribe.unsubscribe();
    }
  }

  /**
   * Ouvre la liste des différents géométrie qu'on peut ajouter
   * si la couche est de type Géométrie
   */
  public openDropdownGeomtypesList() {
    document.getElementById(this.listGeomtypesId).classList.toggle('show');
  }

  /**
   * Change le type de géométrie à ajouter si c'est une couche de type Géométrie
   * @param geomType point | line | polygon
   */
  public changeGeomType(geomType) {
    if (this.feature) {
      this.feature = null;
    }
    this.deleteCurrentInteraction(['add', 'snap', 'select', 'modify', 'remove']);
    if (this.isMultiGeom) {
      geomType = `MULTI${geomType}`;
    }
    this.geomType = geomType;
    document.getElementById(this.listGeomtypesId).classList.remove('show');
    this.initDrawing();
    this.addGeometry();
  }

  /**
   * Assigne le type de geométrie en fonction du type présent dans la table
   */
  private getGeometryTableField() {
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: `${environment.properties.app_api_url}/vmap/layers/query/fields`,
      params: {
        database: this.oLayer.database,
        schema: this.oLayer.schema,
        table: this.oLayer.table ? this.oLayer.table : this.oLayer.pg_table
      }
    }).then(resp => {
      if (resp && resp.data && resp.data.length > 0) {
        const field = resp.data.find(e => e.name === this.oLayer.geomfield);
        this.geomType = field ? field.geometry_type : null;

        if (this.geomType === 'GEOMETRY') {
          this.isGeometryLayer = true;
          this.geomType = 'POINT';
        }

        if (this.geomType.toLowerCase().includes('multi')) { this.isMultiGeom = true; }
        this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
      }
    }).catch(e => {
      console.error(e);
      this.translate.get('ERROR_INSERT_LAYER_OBJECT').subscribe(async (translation) => {
        vitisAlert.alert({
          text: translation,
          showCancelButton: true
        });
      });
    });
  }

}
