import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { DomSanitizer, SafeHtml } from '@angular/platform-browser';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { MapToolDef } from 'src/modules/vmap/components/carto/components/vmap-tools/map-tools.model';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';

@Component({
  selector: 'app-vmap-tools-simple-html',
  templateUrl: './vmap-tools-simple-html.component.html',
  styleUrls: ['./vmap-tools-simple-html.component.scss']
})
export class VmapToolsSimpleHtmlComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() oTool: MapToolDef;

  // Contenu HTML de l'outil
  public sHTMLContent: SafeHtml;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  constructor(
    private vitisCheck: VitisCheck,
    private vmapToolsService: VmapToolsService,
    private domSanitizer: DomSanitizer,
  ) { }

  ngOnInit(): void {
    this.subscriptions.push(this.oTool.onOpen.subscribe(() => {
      this.initTool();
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Interprète l'HTML et lance le JS
   */
  public initTool() {

    // Contenu HTML
    if (this.vitisCheck.isString(this.oTool.htmlContent) && this.oTool.htmlContent.length > 0) {
      this.sHTMLContent = this.domSanitizer.bypassSecurityTrustHtml(this.oTool.htmlContent);
    }

    // Contenu JS
    if (this.vitisCheck.isString(this.oTool.jsContent) && this.oTool.jsContent.length > 0) {
      setTimeout(() => { // Attend que l'HTML soit chargé
        this.vmapToolsService.loadJs(this.oTool.jsContent, null, null, {
          olMap: this.olMap,
          oTool: this.oTool,
        });
      });
    }
  }
}
