import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { VitisCheck, VitisRequest, VitisClone, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { LayerSymbologyService } from '../../services/layer-symbology/layer-symbology.service';
import { ExpressionModel } from 'src/modules/vmap/components/models/json-symbology/json-symbology-class.model';

declare var $: any;

@Component({
  selector: 'app-layer-symbology-class-avanced-input',
  templateUrl: './layer-symbology-class-avanced-input.component.html',
  styleUrls: ['./layer-symbology-class-avanced-input.component.scss']
})
export class LayerSymbologyClassAvancedInputComponent implements OnInit {

  get attribut() {
    return this.service.attributName;
  }

  get columns() {
    return this.service.columnsAvancedInput['_value'];
  }

  public value: string | any = '';
  public valueSave: string | any = '';
  public label: string = '';
  public multipleclass: number;

  public typeEditor: string = 'avanced';

  public textareaAttrId = 'textareaAttr_' + Date.now();

  public operators = ['=', '!=', '<', '<=', '>', '>='];

  public optionsDatalist = {};

  public isOpenDropDown: boolean = false;

  @Output() changeVal = new EventEmitter<any>();

  constructor(
    public service: LayerSymbologyService,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private vitisClone: VitisClone
  ) { }

  ngOnInit(): void {
    const self = this;
    $('#modal-avanced-text-input').on('show.bs.modal', function (e) {
      self.value = e.relatedTarget.value;
      if (self.isArray(self.value)) {
        self.value.forEach(expression => {
          self.optionsDatalist[expression.table] = self.getOptionsTable(expression.table);
        });
      }
      self.valueSave = self.vitisClone.clone(self.value);
      if (e.relatedTarget.label) {
        self.label = e.relatedTarget.label;
      }
      if (self.vitisCheck.isDefAndNotNull(e.relatedTarget.multipleclass)) {
        self.multipleclass = e.relatedTarget.multipleclass;
        if (self.value === '') {
          self.value = [new ExpressionModel({ operator: self.operators[0] })];
        }

        if (typeof self.value == 'object') {
          self.typeEditor = 'simple';
        } else {
          self.typeEditor = 'avanced';
        }
      } else {
        // Si il n'y a pas de multiclass on set obligatoirement le mode avancé
        self.typeEditor = 'avanced';
        self.multipleclass = -1;
        if (!self.vitisCheck.isDefAndNotNull(self.value)) {
          self.value = '';
        }
      }
    });
  }

  /**
   * Retourne true si le champ est de type string
   * @param val
   * @returns
   */
  isString(val): boolean { return typeof val === 'string'; }

  /**
   * Retourne true si le champ est de type objet
   * @param val
   * @returns
   */
  isArray(val): boolean { return typeof val === 'object'; }

  /**
   * Rajoute "[nom_de_la_colonne]" à l'endroit ou est placé le cursor sinon en fin
   * @param column colonne présent dans la table rataché à la layerpg
   */
  public addColumn(column): void {
    const start = $('#' + this.textareaAttrId).prop('selectionStart');
    const end = $('#' + this.textareaAttrId).prop('selectionEnd');
    this.value = this.value.slice(0, start) + `[${column}]` + this.value.slice(end, this.value.length);
  }

  /**
   * Annule les modifications
   */
  public cancelModifications(): void {
    if (typeof this.value === 'object') {
      this.changeVal.next({ value: this.valueSave, attribut: this.attribut });
    }
    $('#modal-avanced-text-input').modal('hide');
  }

  /**
   * Renvoie la valeur présent dans le champ de l'éditeur avancé/simple dans le champs parent
   */
  public updateAttribut(): void {
    if (typeof this.value == 'object') {
      for (const expression of this.value) {
        if (!expression.table || !expression.operator || !expression.value) {
          vitisAlert.alert({
            'type': 'warning',
            'text': 'Veuillez remplir l\'expression au complet pour pouvoir appliquer les changements.',
          });
          return;
        } else if (['<', '>'].some(e => expression.operator.includes(e))) {
          if (!+expression.value.match(/^\d*$/g)) {
            vitisAlert.alert({
              'type': 'warning',
              'text': 'Veuillez ne mettre que des nombres en valeur si l\'opérateur est \' > \', \' >= \', \' < \', ou \' <= \'.',
            });
            return;
          }
        }
      }
    }
    this.changeVal.next({ value: this.value, attribut: this.attribut });
    $('#modal-avanced-text-input').modal('hide');
  }

  /**
   * Change l'éditeur actuel contre son opposé simple =/ avancé
   */
  public changeEditor = (): void => {
    if (this.typeEditor == 'avanced') {
      vitisAlert.alert({
        'type': 'warning',
        'text': 'Si vous continuez, vous perdez vos modifications en cours, êtes vous sûr de vouloir passer en mode éditeur simple ?',
        'showCancelButton': true
      }).then((result) => {
        if (result.isConfirmed) {
          this.typeEditor = 'simple';
          this.value = typeof this.valueSave == 'object' ? this.valueSave : [new ExpressionModel({ operator: this.operators[0] })];
        }
      });
    } else {
      this.typeEditor = 'avanced';
      this.value = this.service.expressionTaductionArrToString(this.value);
      // Evite la valeur ' [null] = null ' pour le cas où on ouvre les 3 points d'expression et on passe directement dans l'editeur avancé
      if (this.value == ' [null] = null ') {
        this.value = '';
      }
    }
  }

  /**
   * Retourne les colonnes disponibles
   * @param table
   * @returns
   */
  public getOptionsTable = (table): Array<any> => {
    if (!this.optionsDatalist[table]) {
      this.vitisRequest.ajaxRequestPromise({
        url: `${environment.properties.app_api_url}/vmap/formcolumnvalues`,
        method: 'GET',
        params: { layer_id: this.service.bddDatas.layer_id, column_distinct: table },
        displayErrorMessage: false
      }).then((resp: any) => {
        if (resp && resp.data && resp.data.data) {
          this.optionsDatalist[table] = resp.data.data.map(e => e[table]);
        }
      });
    }
    return this.optionsDatalist[table];
  }

  /**
   * Retourne les colonnes disponibles
   * @param table
   */
  public chooseExpressionTable = (table): void => {
    this.getOptionsTable(table);
  }

  /**
   * Ouvre le menu ajouter une expression
   * @param index
   */
  public openAddExpression = (index): void => {
    document.getElementById(`myDropdownOperator_${index}`).classList.toggle('show');
    this.isOpenDropDown = !this.isOpenDropDown;
  }

  /**
   * Ajoute une ligne expression
   * @param sql_operator
   * @param index
   */
  public addExpression = (sql_operator, index) => {
    this.value[index].sql_operator = sql_operator;
    this.value = [
      ...this.value,
      new ExpressionModel({ operator: this.operators[0] })
    ];
    document.getElementById(`myDropdownOperator_${index}`).classList.remove('show');
  }

  /**
   * Supprime une ligne expression
   * @param indexDel
   */
  public deleteExpression = (indexDel) => {
    if (indexDel != 0) {
      this.value[indexDel - 1].sql_operator = null;
    }
    this.value = [...this.value.slice(0, indexDel), ...this.value.slice(Number(indexDel) + 1)];
  }

}
