import { computeMsgId } from '@angular/compiler';
import { Component, Input, OnInit } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { Map } from 'ol';
import { JsonSymbologyExpression } from 'src/app/modules/vmap/models/json-symbology/json-symbology-expression.model';
import { VitisRequest, vitisAlert, VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';
import { environment } from 'src/environments/environment';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { lineSimple } from 'src/modules/vmap/components/models/definition-sym-json';
import { JsonSymbologyClass } from 'src/modules/vmap/components/models/json-symbology/json-symbology-class.model';
import { lineSymbol, polygonSimple, polygonSymbolCenter, polygonSymbolRepeat, unflattenDefinitionSymJson } from '../models/definition-sym-json';
import { LayerSymbologyService } from './services/layer-symbology/layer-symbology.service';

@Component({
  selector: 'app-layer-symbology',
  templateUrl: './layer-symbology.component.html',
  styleUrls: ['./layer-symbology.component.scss']
})
export class LayerSymbologyComponent implements OnInit {

  @Input() public olMap: Map;
  @Input() public oLayer: VmapLayer;
  @Input() public sClass: string;

  public bParamsOpen = true; // Statut d'ouverture du composant paramètre de la couche
  public bParamsClassOpen = false; // Statut d'ouverture du composant paramètre de la classe
  public bLabelClassOpen = false; // Statut d'ouverture du composant label de la classe
  public bSymbologyClassOpen = false; // Statut d'ouverture du composant symbologie de la classe

  public oDefinitionSymJson: any = {}; // Object layer.definition_sym_json
  public oDefinitionSymJsonModified: any = {}; // Copie de l'object layer.definition_sym_json, en cours de modifications
  public isModaleAvancedEditorOpen: boolean = false; // Statut d'ouverture de la modale avec l'éditeur de texte simple/avancé

  public ClassType; // Model de la classe en cours : singleclass (JsonSymbologyClass) | expression (JsonSymbologyExpression)

  public editSymbology: 'layer' | 'class' = 'layer'; // Édition en cours de la page : layer | class
  public class: any = { // Class précharger quand on édite une classe
    index: null,
    content: null,
  };

  private isValidFields: any = {}; // Nom du champs: boolean

  public isLayerClassExist = false;

  public routeDatasource = {
    layer: {
      postgres: 'pglayers',
      file: 'filelayers'
    },
    publication: {
      postgres: 'pgpublications',
      file: 'filepublications'
    }
  };

  get columnsAvancedInput() {
    return this.service.columnsAvancedInput['_value'];
  }

  private msGeomTypes = ['point', 'line', 'polygon'];
  public currGeomType = 'point';
  private def_sym_json_geometries = {};


  public styleTypes = {
    'singleclass': 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_SINGLECLASS',
    'expression': 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_EXPRESSION',
    // 'range': 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_RANGE',
    // 'cluster': 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_CLUSTER',
  };

  public oSubType = this.service.getLabelsList(this.currGeomType);
  public oIcons = this.service.getIcons();
  public isSymbolscaledenom: boolean = false;

  constructor(
    private vmapToolsService: VmapToolsService,
    private service: LayerSymbologyService,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    private formReaderService: FormreaderService,
    private translate: TranslateService
  ) { }

  ngOnInit(): void {
    if (this.oLayer['layertype'] === 'postgres' ||
    this.oLayer['publicationtype'] === 'postgres') {
      this.service.getSymbols();
      this.service.getFonts();
      this.service.bddDatas = { // Initialise les données de bdd pour récupérer les colonnes liées à la couche
        layer_id: this.oLayer['layer_id'],
        database: this.oLayer['database'],
        schema: this.oLayer['schema'],
        table: this.oLayer['table'] || this.oLayer['pg_table'],
        datasource_id: this.oLayer['datasource_id']
      };
      this.service.getColumns().catch(() => {
        console.error('Cannot get layer columns');
        vitisAlert.alert({
          text: 'Impossible de récupérer les colonnes de la couche',
          type: 'error',
        });
      });
      this.isLayerClassExist = true;
    }
    if (!this.oLayer.definition_sym_json) {
      this.oLayer.definition_sym_json = {};
    }
    if (this.isGeometryGeomtype()) { // Si type de géométrie est égal à géométrie
      this.msGeomTypes.forEach(geomType => {
        this.def_sym_json_geometries[geomType] = this.vitisCheck.isDefAndNotNull(this.oLayer.definition_sym_json[geomType]) ?
          this.vitisClone.clone(this.oLayer.definition_sym_json[geomType]) : this.vitisClone.clone(this.oLayer.definition_sym_json);
      });
      this.oLayer.definition_sym_json = this.vitisClone.clone(this.def_sym_json_geometries);
      this.updateoLayerGeomType(this.currGeomType);
    } else { // autre type de géométrie
      this.currGeomType = this.oLayer.ms_geomtype.label.toLowerCase();
      this.oLayer.definition_sym_json = this.unflattenDefSymJson(this.vitisClone.clone(this.oLayer.definition_sym_json));

      this.initDefinitionSymJson(this.oLayer.definition_sym_json);
    }
  }

  /**
   * Initialisation de la définition symbologique de la couche
   * @param oLayerSymDef
   */
  private initDefinitionSymJson(oLayerSymDef) {
    this.initClassType(oLayerSymDef.type);
    this.oDefinitionSymJson = new this.ClassType(this.vitisClone.clone(oLayerSymDef));
    this.oDefinitionSymJsonModified = new this.ClassType(this.oDefinitionSymJson);
    // Met à jour la prévisualisation
    setTimeout(() => {
      this.testModifications();
    });
  }

  /**
   * Appelé au changement de type de géométrie
   * @param type
   */
  public updateoLayerGeomType(geomType): void {
    this.currGeomType = geomType;
    this.oLayer.definition_sym_json[geomType] = this.unflattenDefSymJson(this.vitisClone.clone(this.oLayer.definition_sym_json[geomType]));
    this.oLayer.definition_sym_json[geomType].geomType = geomType;
    this.initDefinitionSymJson(this.oLayer.definition_sym_json[geomType]);
  }

  /**
   * Initialise le type de classe
   * @param type
   */
  private initClassType(type?): void {
    switch (type) {
      case 'singleclass':
        this.ClassType = JsonSymbologyClass;
        break;
      case 'expression':
        this.ClassType = JsonSymbologyExpression;
        break;
      default:
        this.ClassType = JsonSymbologyClass;
        break;
    }
  }

  /**
   * Appelé au changement de type
   * @param type
   */
  public updateType(type): void {
    this.initClassType(type);
    this.oDefinitionSymJsonModified = new this.ClassType(this.oDefinitionSymJsonModified);
    if (this.isGeometryGeomtype()) {
      this.oDefinitionSymJsonModified.geomType = this.currGeomType;
      this.def_sym_json_geometries = {
        ...this.def_sym_json_geometries,
        [this.currGeomType]: this.oDefinitionSymJsonModified
      };
    }
  }

  /**
   * Ferme le menu
   */
  public closePanel() {
    this.vmapToolsService.setLeftPanel(1, 'control-layers');
  }

  /**
   * Met à jour le champs depuis l'éditeur de texte avancé
   * @param e  = { attribut, value } : attribut: chemin depuis l'object odefsymjson ; value: nouvelle valeur
   */
  public changeVal(e: any): void {
    const childs = e.attribut.match(/(\w*\d*)/ig).filter(child => child);
    let oDefClone = this.editSymbology === 'class' ? this.class.content : this.oDefinitionSymJsonModified; // Permet de mettre à jour avec la référence de la vararible
    for (let i = 0; i < childs.length; i++) {
      if (i !== childs.length - 1) {
        oDefClone = oDefClone[childs[i]];
      }
    }
    oDefClone[childs[childs.length - 1]] = e.value;
    if (this.editSymbology === 'class') {
      this.class.content = { ...this.class.content };
    } else if (this.editSymbology === 'layer') {
      this.oDefinitionSymJsonModified = { ...this.oDefinitionSymJsonModified }; // Réatribution du changement car l'objet a été modifié
    }
  }

  /**
   * Utiliser pour supprimer un element dans un tableau de l'objet odefSymJson
   * @param e chemin + index sous forme de string
   */
  public deleteVal(event): void {
    const { deleteClass, e, indexClass } = event;

    if (deleteClass && this.vitisCheck.isDefAndNotNull(indexClass)) { // SUPPRESSION D'UNE CLASSE
      this.oDefinitionSymJsonModified.classes = [...this.oDefinitionSymJsonModified.classes.slice(0, indexClass),
      ...this.oDefinitionSymJsonModified.classes.slice(Number(indexClass) + 1)];
    } else if (e) {
      const paths = e.match(/(\w*\d*)/ig).filter(child => child);

      if (this.vitisCheck.isDefAndNotNull(indexClass)) { // SUPPRESSION D'UN ELEMENT DANS UN TABLEAU -> MULTIPLE CLASSE
        if (paths.length == 5 && this.vitisCheck.isObject(this.class.content[paths[2]][paths[3]][paths[4]])) {
          // SUPPRESION D'UN ELEMENT DANS UN TABLEAU -> .otherFields
          delete this.class.content[paths[2]][paths[3]][paths[4]];
        } else {
          const property = paths[2];
          const indexToDelete = paths[3];
          const path = this.class.content[property];
          if (path) {
            this.class.content[property] = [...path.slice(0, indexToDelete), ...path.slice(Number(indexToDelete) + 1)];
          }
        }
      } else { // SUPPRESION D'UN ELEMENT DANS UN TABLEAU -> SINGLE CLASSE
        if (paths.length == 3 && this.vitisCheck.isObject(this.oDefinitionSymJsonModified[paths[0]][paths[1]][paths[2]])) { // SUPPRESION D'UN ELEMENT DANS UN TABLEAU -> .otherFields
          delete this.oDefinitionSymJsonModified[paths[0]][paths[1]][paths[2]];
        } else {
          const property = paths[0];
          const indexToDelete = paths[1];
          const path = this.oDefinitionSymJsonModified[property];
          this.oDefinitionSymJsonModified[property] = [...path.slice(0, indexToDelete), ...path.slice(Number(indexToDelete) + 1)];
        }
      }
    }
  }

  /**
   * Annule les modifications et reset les valeurs modifiées
   */
  public cancelModifications = (): void => {
    if (this.isGeometryGeomtype()) {
      this.initClassType(this.oLayer.definition_sym_json[this.currGeomType].type);
    } else {
      this.initClassType(this.oLayer.definition_sym_json.type);
    }

    this.oDefinitionSymJsonModified = new this.ClassType(this.oDefinitionSymJson);

    // Si on est en mode classe alors remonte sur l'édition de la couche
    if (this.editSymbology === 'class') {
      this.cancelClassModifications();
    } else {
      // lance le test
      this.testModifications();
    }
  }

  /**
   * Annule les modifications et reset les valeurs modifiées
   */
  public cancelClassModifications = (): void => {
    this.editSymbology = 'layer';
    this.class = { index: null, content: null };

    // lance le test
    this.testModifications();
  }

  /**
   * Envoie la requête de modification de odefinition_sym_json OU modifie la classe éditer
   */
  public updateModifications = (): void => {

    if (this.editSymbology === 'class') {
      this.oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;
    }

    if (this.formValidation(this.oDefinitionSymJsonModified, true)) {
      let oDefSymJson: any;
      let flatDefSymJson: any;
      if (this.isGeometryGeomtype()) { // Si type de géométrie est égal à géométrie
        oDefSymJson = {
          ...this.def_sym_json_geometries,
          [this.currGeomType]: this.oDefinitionSymJsonModified
        };
        flatDefSymJson = {
          ...this.def_sym_json_geometries,
          [this.currGeomType]: this.flattenDefSymJson(this.vitisClone.clone(this.oDefinitionSymJsonModified))
        };
      } else { // autre type de géométrie
        oDefSymJson = this.oDefinitionSymJsonModified;
        flatDefSymJson = this.flattenDefSymJson(this.vitisClone.clone(this.oDefinitionSymJsonModified));
      }
      let route = '';
      if (this.vitisCheck.isDefAndNotNull(this.oLayer.layer_id)) {
        route = `${this.routeDatasource.layer[this.oLayer['layertype']]}/${this.oLayer.layer_id}`;
      } else if (this.vitisCheck.isDefAndNotNull(this.oLayer.publication_id)) {
        route = `${this.routeDatasource.publication[this.oLayer['publicationtype']]}/${this.oLayer.publication_id}`;
      }
      this.vitisRequest.ajaxRequestPromise({
        url: `${environment.properties.app_api_url}/vmap/datasourcetreeview/${route}`,
        method: 'PUT',
        data: this.formReaderService.transformToDataForm({ definition_sym_json: flatDefSymJson }, true)
      }).then((resp: any) => {
        if (resp && resp.data && resp.data.status === 1) {
          if (this.isGeometryGeomtype()) { // Si type de géométrie est égal à géométrie
            this.oDefinitionSymJson = new this.ClassType(oDefSymJson[this.currGeomType]);
            this.oDefinitionSymJsonModified = new this.ClassType(oDefSymJson[this.currGeomType]);
            this.def_sym_json_geometries = this.vitisClone.clone(oDefSymJson);
            this.oLayer.definition_sym_json = this.vitisClone.clone(oDefSymJson);
          } else {
            this.oDefinitionSymJson = new this.ClassType(oDefSymJson);
            // Actualise l'object modifié pour le binding des héritage
            this.oDefinitionSymJsonModified = new this.ClassType(oDefSymJson);
          }
          // update ok
          this.translate.get('FORM_PROPERTIES_EDIT_OK').subscribe(async (translation) => {
            vitisAlert.alert({
              type: 'success',
              text: translation
            });
          });
          // lance le test
          this.testModifications();
        } else {
          // update ko
        }
      });
    }
  }

  /**
   * Envoie la requête de modification de odefinition_sym_json OU modifie la classe éditer
   */
  public updateClassModifications = (): void => {
    this.oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;
    this.editSymbology = 'layer';
    this.updateModifications();
  }

  /**
   * Lance le test sur les modifications en cours
   */
  public testModifications() {
    if (this.editSymbology === 'class') {
      this.oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;
    }
    const flatDefSymJson = this.flattenDefSymJson(this.vitisClone.clone(this.oDefinitionSymJsonModified));
    this.service.runTest(flatDefSymJson);
  }

  /**
   * Lance le test sur les modifications en cours
   */
  public testClassModifications() {

    const oDefinitionSymJsonModified = this.vitisClone.clone(this.oDefinitionSymJsonModified);
    oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;

    if (this.formValidation(oDefinitionSymJsonModified, true)) {
      const flatDefSymJson = this.flattenDefSymJson(this.vitisClone.clone(oDefinitionSymJsonModified));
      this.service.runTest(flatDefSymJson);
    }
  }

  /**
   * Ouvre l'éditeur pour ajouter une classe à la couche
   */
  public addClass = (): void => {
    this.class = {
      newClass: true,
      index: this.oDefinitionSymJsonModified.classes.length,
      content: new JsonSymbologyClass({ id: this.oDefinitionSymJsonModified.classes.length, styles: [this.service.setDefaultValue(this.currGeomType)] }, this.oDefinitionSymJsonModified.type)
    };
    this.editSymbology = 'class';
  }

  /**
   * Function qui prend les paramètres classe pour la modification
   * @param e { indexClass (index de la classe qu'on veut modifier), class (l'object class qu'on veut éditer)  }
   */
  public updateEdit(e): void {
    this.class = {
      index: e.indexClass,
      content: e.class
    };
    this.editSymbology = 'class';
  }

  /**
   * Vérifie si chaque champs qui possède [nom_de_la colonne], nom_de_la colonne fait bien parti de la table
   * @param form object JsonSymbologySingleClass modifié
   * @param formIsValide sert pour la recursivité
   * @returns boolean, true si le formulaire est valide false sinon
   */
  private formValidation(form: any, formIsValide: boolean): boolean {
    for (const key in form) {
      if (typeof form[key] === ('object')) {
        formIsValide = this.formValidation(form[key], formIsValide);
      } else if (typeof form[key] === 'string') {
        const isColumnExist = form[key].match(/[^[\]]+(?=])/g);
        if (isColumnExist) {
          for (let i = 0; i < isColumnExist.length; i++) {
            if (!this.columnsAvancedInput.find(column => column.column_name === isColumnExist[i])) {
              formIsValide = false;
              vitisAlert.alert({
                type: 'warning',
                text: `Veuillez entrer un nom de table valide. La table [${isColumnExist[i]}] n'existe pas.`
              });
            }
          }
        }

        if (this.vitisCheck.isDefAndNotNull(this.isValidFields[key]) && this.isValidFields[key] === false) {
          formIsValide = false;
          vitisAlert.alert({
            type: 'warning',
            text: this.translate.instant('COMPONENT_MAP_SYMBOLOGY_ERROR_INVALID_FORM')
          });
        }
      }
    }
    return formIsValide;
  }

  /**
   * Récupère le bon titre que l'on soit dans la création ou dans l'édition d'une classe
   * @returns
   */
  public getClassTitle = (): string => {
    let classTitle = '';
    this.translate.get(['COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS', 'COMPONENT_MAP_CREATE_SYMBOLOGY_CLASS'])
      .subscribe((translations: string) => {
        classTitle = this.oDefinitionSymJsonModified.classes[this.class.index] ?
          `${translations['COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS']} : ${this.oDefinitionSymJsonModified.classes[this.class.index].legendName}` :
          translations['COMPONENT_MAP_CREATE_SYMBOLOGY_CLASS'];
      });
    return classTitle;
  }

  /**
   * Clonne une classe
   * @param e
   */
  public cloneClass(e: any) {
    const indexClass = e.indexClass;
    if (this.vitisCheck.isDefAndNotNull(indexClass) &&
      this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJsonModified.classes[indexClass])) {
      const newClass = this.vitisClone.clone(this.oDefinitionSymJsonModified.classes[indexClass]);
      this.oDefinitionSymJsonModified.classes.splice(indexClass + 1, 0, newClass);
    }
  }

  /**
   * Remonce une classe
   * @param e
   */
  public moveUpClass(e: any) {
    const indexClass = e.indexClass;
    if (this.vitisCheck.isDefAndNotNull(indexClass) &&
      this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJsonModified.classes[indexClass])) {
      this.moveClass(this.oDefinitionSymJsonModified.classes, indexClass, indexClass - 1);
    }
  }

  /**
   * Descend une classe
   * @param e
   */
  public moveDownClass(e: any) {
    const indexClass = e.indexClass;
    if (this.vitisCheck.isDefAndNotNull(indexClass) &&
      this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJsonModified.classes[indexClass])) {
      this.moveClass(this.oDefinitionSymJsonModified.classes, indexClass, indexClass + 1);
    }
  }

  /**
   * Déplace une élement dans le tableau de classes
   * @param arr
   * @param old_index
   * @param new_index
   * @returns
   */
  private moveClass(arr: Array<any>, old_index: number, new_index: number) {
    if (new_index >= 0 && new_index < arr.length) {
      arr.splice(new_index, 0, arr.splice(old_index, 1)[0]);
    }
    return arr;
  }

  /**
   * Modifie la validité d'un champs
   * @param e
   */
  public isValidField(e: any) {
    Object.assign(this.isValidFields, e);
  }

  /**
   * Retuorne true si la symbologie est de type geometry
   * @returns
   */
  private isGeometryGeomtype() {
    return this.oLayer['layertype'] === 'postgres' && this.oLayer['ms_geomtype']['label'] === 'GEOMETRY';
  }

  /**
   * Fonction appelée au changement de type
   * @param e
   */
  public changeType = (e) => {
    this.oDefinitionSymJsonModified.type = e;
    this.updateType(e);
  }

  /**
   * Set valeurs in otherField
   * @param event
   */
  public setJsonValue(event: any) {
    this.oDefinitionSymJsonModified = this.service.setOtherFieldValue(event, this.oDefinitionSymJsonModified);
  }

  /**
   * Vérifie si symbolscaledenom à une valeur quand il y a une modification
   * @param event
   */
  public updateSymbolScaleDeNom(event: any) {
    this.oDefinitionSymJsonModified.symbolscaledenom = event;
    this.isSymbolscaledenom = event == '' ? false : true;
  }

  /**
   * Passe d'une structure à plat à une structure utilisable dans le studio
   * @param ooDefinitionSymJson
   */
  private unflattenDefSymJson(oDefinitionSymJson) {

    oDefinitionSymJson = this.unflattenItem(oDefinitionSymJson, 'general');

    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.classes)) {
      for (let i = 0; i < oDefinitionSymJson.classes.length; i++) {
        oDefinitionSymJson.classes[i] = this.unflattenItem(oDefinitionSymJson.classes[i], 'classes');
        oDefinitionSymJson.classes[i] = this.unflattenClass(oDefinitionSymJson.classes[i]);
      }
    } else {
      oDefinitionSymJson = this.unflattenClass(oDefinitionSymJson);
    }

    return oDefinitionSymJson;
  }

  /**
   * Passe d'une structure à plat à une structure utilisable dans le studio
   * @param oDefinitionSymJson
   */
  private unflattenClass(oDefinitionSymJson: any) {
    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.label)) {
      oDefinitionSymJson.label = this.unflattenItem(oDefinitionSymJson.label, 'labels');
    }
    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.styles)) {
      for (let i = 0; i < oDefinitionSymJson.styles.length; i++) {
        oDefinitionSymJson.styles[i] = this.unflattenItem(oDefinitionSymJson.styles[i], 'styles');
      }
    }
    return oDefinitionSymJson;
  }

  /**
   * Passe d'une structure à plat à une structure utilisable dans le studio
   * @param oDefinitionSymJson
   * @param levelInObject
   * @returns
   */
  private unflattenItem(oDefinitionSymJson: any, levelInObject: string) {
    let keys: Array<string>;
    let localUnflattenDefinitionSymJson = unflattenDefinitionSymJson[levelInObject];
    // ajout un niveau avec le type de géométrie si besoin
    if (this.vitisCheck.isDefAndNotNull(localUnflattenDefinitionSymJson[this.currGeomType])) {
      localUnflattenDefinitionSymJson = localUnflattenDefinitionSymJson[this.currGeomType];
    }

    if (this.currGeomType == 'polygon') {
      if (Object.keys(oDefinitionSymJson).findIndex((key: string) => key === 'polygonSymbolRepeat') != -1) {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...polygonSymbolRepeat };
      } else if (Object.keys(oDefinitionSymJson).findIndex((key: string) => key === 'polygonSymbolCenter') != -1) {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...polygonSymbolCenter };
      } else {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...polygonSimple };
      }
    }
    if (this.currGeomType == 'line') {
      if (Object.keys(oDefinitionSymJson).findIndex((key: string) => key === 'lineSymbol') != -1) {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...lineSymbol };
      } else {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...lineSimple };
      }
    }

    for (const key in oDefinitionSymJson) {
      if (Object.prototype.hasOwnProperty.call(oDefinitionSymJson, key)) {
        keys = [];
        if (!this.vitisCheck.isObject(oDefinitionSymJson[key])) {
          const element = oDefinitionSymJson[key];

          if (this.vitisCheck.isDefAndNotNull(localUnflattenDefinitionSymJson[key])) {
            if (this.vitisCheck.isDefAndNotNull(element) && element != '') {
              if (!this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.otherFields)) {
                oDefinitionSymJson.otherFields = {};
              }
              keys = localUnflattenDefinitionSymJson[key].split('.');

              if (keys.length == 2) {
                if (!this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.otherFields[keys[0]])) {
                  oDefinitionSymJson.otherFields[keys[0]] = {};
                }
                oDefinitionSymJson.otherFields[keys[0]][keys[1]] = this.vitisClone.clone(element);
              }
            }

            if (keys.length != 1) {
              delete oDefinitionSymJson[key];
            }
          }
        }
      }
    }
    return oDefinitionSymJson;
  }

  /**
   * Met à plat la définition de la symbologie en json pour la sauvegarde en bdd
   * @param oDefinitionSymJson
   */
  private flattenDefSymJson(oDefinitionSymJson: any) {
    oDefinitionSymJson = this.flattenOtherFields(oDefinitionSymJson);

    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.classes)) {
      for (let i = 0; i < oDefinitionSymJson.classes.length; i++) {
        oDefinitionSymJson.classes[i] = this.flattenOtherFields(oDefinitionSymJson.classes[i]);
        oDefinitionSymJson.classes[i] = this.flattenClass(oDefinitionSymJson.classes[i]);
      }
    } else {
      return this.flattenClass(oDefinitionSymJson);
    }

    return oDefinitionSymJson;
  }

  /**
   * met à plat une classe
   * @param oDefinitionSymJson
   */
  private flattenClass(oDefinitionSymJson: any) {
    oDefinitionSymJson.label = this.flattenOtherFields(oDefinitionSymJson.label);
    for (let i = 0; i < oDefinitionSymJson.styles.length; i++) {
      oDefinitionSymJson.styles[i] = this.flattenOtherFields(oDefinitionSymJson.styles[i]);
    }
    return oDefinitionSymJson;
  }

  /**
   * met à plat OtherFields
   * @param oDefinitionSymJson
   */
  private flattenOtherFields(oDefinitionSymJson: any) {
    if (Object.prototype.hasOwnProperty.call(oDefinitionSymJson, 'otherFields')) {
      for (const key in oDefinitionSymJson.otherFields) {
        if (Object.prototype.hasOwnProperty.call(oDefinitionSymJson.otherFields, key)) {
          const subArray = oDefinitionSymJson.otherFields[key];
          for (const subKey in subArray) {
            if (Object.prototype.hasOwnProperty.call(subArray, subKey)) {
              oDefinitionSymJson[subKey] = subArray[subKey];
            }
          }
        }
      }
      delete oDefinitionSymJson.otherFields;
    }

    return oDefinitionSymJson;
  }
}
