import { ActivatedRoute } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { environment } from 'src/environments/environment';
import { Component, OnInit, ViewChild } from '@angular/core';
import { controlEnabledForm } from './controls-enabled.form';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { availableControls } from 'src/app/vitis/map-core/services/map-models.model';
import { VitisRequest, vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

@Component({
  selector: 'app-controls-enabled',
  templateUrl: './controls-enabled.component.html',
  styleUrls: ['./controls-enabled.component.scss']
})
export class ControlsEnabledComponent implements OnInit {

  @ViewChild(FormreaderComponent, { static: false }) oFormReader: FormreaderComponent;
  private iMapId: number;

  public oControlsFormOptions: any;
  constructor(
    private route: ActivatedRoute,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService
  ) { }

  ngOnInit(): void {
    this.getControlsList().then(options => {
      controlEnabledForm.datasources.datasource_controls.options = options;
    });
    this.route.data.subscribe(oMode => {
      this.route.params.subscribe(oParams => {
        if (this.vitisCheck.isDefAndNotNull(oParams) && this.vitisCheck.isDefAndNotNull(oParams.id)) {
          this.iMapId = oParams.id;
        } else {
          vitisAlert.alert({
            type: 'error',
            title: 'error'
          });
        }
      });
    });
  }

  /**
   * Retourne la liste des informations à mettre dans le select
   * @returns
   */
  public getControlsList(): Promise<any> {
    return new Promise((resolve, reject) => {
      const optionsControls = [];
      availableControls.forEach((control, index) => {
        this.translate.get(control).subscribe(async (translation) => {
          await optionsControls.push({ value: control, label: translation ? translation : control });
          if (index + 1 === availableControls.length) {
            resolve(optionsControls.reduce((acc, curr) => {
              return [...acc, curr.label + '|' + curr.value];
            }, []));
          }
        });
      });
    });
  }

  /**
   * Appelé après génération de la vue
   */
  ngAfterViewInit() {
    this.initForm();
  }

  /**
   * Initialise le formulaire
   */
  public initForm() {
    const aEnabledControls: string[] = [];
    // Récupération des contrôles activés
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: environment.properties.app_api_url + '/vmap/maps/' + this.iMapId
    }).then((resp) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          const oMapTools: any = JSON.parse(resp.data.data.maptools);
          if (oMapTools) {
            const oControls: any = oMapTools.controls;
            if (oControls && Object.keys(oControls).length) {
              Object.keys(oControls).forEach(key => {
                if (oControls[key] === true) {
                  // Ajoute les contrôles activées
                  aEnabledControls.push(key);
                }
              });
            }
            this.oFormReader.initForm(controlEnabledForm, 'update', { 'controls_availables': aEnabledControls });
          } else {
            this.oFormReader.initForm(controlEnabledForm, 'update', { 'controls_availables': [] });
          }
        } else {
          // Message d'erreur.
          this.translate.get('COMPONENT_MAP_ENABLE_UTILS_ERROR_GET_TOAST').subscribe((translation) => {
            vitisAlert.alert({
              type: 'error',
              title: translation
            });
          });
        }
      }
    });
  }
}
