import { ActivatedRoute } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { environment } from 'src/environments/environment';
import { Component, OnInit, ViewChild } from '@angular/core';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VitisRequest, vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { nativeMapToolsDef } from 'src/modules/vmap/components/carto/components/vmap-tools/map-tools-buttons';
import Map from 'ol/Map';
import View from 'ol/View';
import TileLayer from 'ol/layer/Tile';
import XYZ from 'ol/source/XYZ';
import OSM from 'ol/source/OSM';
import { fromLonLat } from 'ol/proj';


@Component({
  selector: 'app-tools-enabled',
  templateUrl: './tools-enabled.component.html',
  styleUrls: ['./tools-enabled.component.scss']
})
export class ToolsEnabledComponent implements OnInit {

  @ViewChild(FormreaderComponent, { static: false }) oFormReader: FormreaderComponent;

  // Définition des outils dans la carto
  public oMapTools: any;
  // Définitionn de tous les outils
  public oMapToolsDef: any;
  public bCollapsed: boolean = false;
  public bAddingIcon: boolean = false;
  // Permet de connaître les outils disponibles
  public oMapToolsAvailable: Array<String> = [];
  public sSrcImg: string = 'conf/images/carte_toulouse.png';
  public sDragOverTool: string;

  public sMapDivId = 'ToolsEnabledComponent-map-' + Date.now();

  // Id de la carte
  private iMapId: number;
  // Est mis à jour quand un élément est prit pour être glissé
  private sSourceDragItemName: string;
  // Est mis à jour quand un élément est déposé
  private sDestinationDragItemName: string;
  // Est mis à jour à l'élément dragenter pour connaître l'emplacement où déposer
  private sPositionDrag: string;
  // Est mis à jour au quand un élément est prit pour être glissé, permet de connaître la position de départ
  private sStartPositionDrag: string;

  public oToolsFormOptions: any;
  public oMap: Map;
  constructor(
    private route: ActivatedRoute,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService,
  ) { }

  ngOnInit(): void {
    this.route.data.subscribe(oMode => {
      this.route.params.subscribe(oParams => {
        if (this.vitisCheck.isDefAndNotNull(oParams) && this.vitisCheck.isDefAndNotNull(oParams.id)) {
          this.iMapId = oParams.id;
        } else {
          vitisAlert.alert({
            type: 'error',
            title: 'error'
          });
        }
      });
    });
    this.initTools();
  }

  /**
   * Appelé après génération de la vue
   */
  ngAfterViewInit() {
    // Initialisation de la map centrée sur la France
    this.oMap = new Map({
      view: new View({
        center: [300000, 6000000],
        zoom: 5,
      }),
      layers: [
        new TileLayer({
          source: new OSM(),
        }),
      ],
      target: this.sMapDivId
    });
  }

  /**
   * Quand une icône d'un outil est prise pour être glissée
   * @param event
   * @param sPosition Position dans les élément 'upright', 'downright', 'bottom' ou 'trash'
   */
  public onDragStart(event: DragEvent, sPosition: string) {
    this.sStartPositionDrag = sPosition;
    event['target']['classList'].forEach((sClassName: string) => {
      if (sClassName.indexOf('-item') !== -1) {
        this.sSourceDragItemName = sClassName.replace('-item', '');
      }
    });
  }

  /**
   * Lorsqu'une icône d'un outil est dépôsée
   * @param event
   */
  public onDragEnd(event: DragEvent) {
    this.sDragOverTool = null;
    this.sStartPositionDrag = null;
  }

  /**
   * Event quand on lache le drop
   * @param event
   * @param sPosition
   * @param sToolname
   */
  public onDrop(event: DragEvent, sPosition: string, sToolname: string) {
    this.sPositionDrag = sPosition;
    this.sDestinationDragItemName = sToolname;

    // Si on dépose sur la poubelle
    if (sPosition == 'trash' && this.sStartPositionDrag != '') {
      this.removeToolFromMap(this.sSourceDragItemName);
    } else if (Object.keys(this.oMapTools).indexOf(this.sStartPositionDrag) !== -1 && Object.keys(this.oMapTools).indexOf(this.sPositionDrag) !== -1) {
      // Changement de position de l'outil
      this.toolPositionChange();
    } else if (Object.keys(this.oMapTools).indexOf(this.sStartPositionDrag) === -1 && Object.keys(this.oMapTools).indexOf(this.sPositionDrag) !== -1) {
      // Ajout d'un nouvel outil
      this.toolAddFromAvailable();
    }

    // lache l'outil en dragover
    this.sDragOverTool = null;
    this.sStartPositionDrag = null;
  }

  /**
   * Event appelé au survol
   * @param event
   * @param sPosition
   * @returns
   */
  public onDragOver(event: DragEvent, sPosition: string) {

    // Permet de ne pas gérer si on pousse un outil depuis la fenêtre d'ajout vers la poubelle
    if (this.sStartPositionDrag === 'addMenu' && sPosition === 'trash') {
      return;
    }
    this.sDragOverTool = sPosition;
    event.preventDefault();
  }

  /**
   * Event appelé lorsque l'outil sort de la zone de drop
   * @param event
   * @param sPosition
   */
  public onDragLeave(event: DragEvent) {
    this.sDragOverTool = null;
  }

  /**
   * Ajoute un outil depuis une barre (une autre ou la même)
   */
  private toolPositionChange() {
    if (this.sDestinationDragItemName.substr(0, 7) === 'endlist') {
      this.oMapTools[this.sStartPositionDrag].splice(this.oMapTools[this.sStartPositionDrag].indexOf(this.sSourceDragItemName), 1);
      this.oMapTools[this.sPositionDrag].push(this.sSourceDragItemName);
    } else if (this.oMapTools[this.sPositionDrag].indexOf(this.sSourceDragItemName) === -1) {
      this.oMapTools[this.sStartPositionDrag].splice(this.oMapTools[this.sStartPositionDrag].indexOf(this.sSourceDragItemName), 1);
      this.oMapTools[this.sPositionDrag].splice(this.oMapTools[this.sPositionDrag].indexOf(this.sDestinationDragItemName), 0, this.sSourceDragItemName);
    } else if (this.oMapTools[this.sPositionDrag].indexOf(this.sSourceDragItemName) !== -1) {
      if (this.sDestinationDragItemName == 'add-tool') {
        this.oMapTools[this.sPositionDrag].splice(this.oMapTools[this.sPositionDrag].indexOf(this.sSourceDragItemName), 1);
        this.oMapTools[this.sPositionDrag].push(this.sSourceDragItemName);
      } else {
        this.oMapTools[this.sPositionDrag].splice(this.oMapTools[this.sPositionDrag].indexOf(this.sSourceDragItemName), 1);
        this.oMapTools[this.sPositionDrag].splice(this.oMapTools[this.sPositionDrag].indexOf(this.sDestinationDragItemName), 0, this.sSourceDragItemName);
      }
    }
  }

  /**
   * Ajoute un outil depuis ceux disponibles
   */
  private toolAddFromAvailable() {
    if (this.sDestinationDragItemName == 'add-tool' && this.oMapTools[this.sPositionDrag].indexOf(this.sSourceDragItemName) === -1) {
      this.oMapTools[this.sPositionDrag].push(this.sSourceDragItemName);
      this.oMapToolsAvailable.splice(this.oMapToolsAvailable.indexOf(this.sSourceDragItemName), 1);
    } else {
      this.oMapTools[this.sPositionDrag].splice(this.oMapTools[this.sPositionDrag].indexOf(this.sDestinationDragItemName), 0, this.sSourceDragItemName);
      this.oMapToolsAvailable.splice(this.oMapToolsAvailable.indexOf(this.sSourceDragItemName), 1);
    }
  }

  /**
   * Récupère les outils de la carte sous forme de JSON
   */
  private initTools() {

    // Chargement outils disponibles
    this.getNativeToolsDef().then(oMapToolsDef => {
      this.oMapToolsDef = oMapToolsDef;

      // Récupération des outils activés
      this.getMapTools().then(oMapTools => {
        if (oMapTools) {
          // Récupération des outils en base
          const oTools: any = oMapTools.tools;
          if (oTools && Object.entries(oTools).length > 0) {
            this.oMapTools = oTools;
          }
        }

        if (!this.oMapTools || Object.entries(this.oMapTools).length == 0) {
          this.oMapTools = {
            'upright': [],
            'downright': [],
            'bottom': []
          };
        }

        this.initToolsAvailable();
      });
    });
  }

  /**
   * Récupère les outils de la carte depuis la base
   * @returns
   */
  private getMapTools(): Promise<any> {
    return new Promise((resolve, reject) => {
      // Récupération des outils activés
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: environment.properties.app_api_url + '/vmap/maps/' + this.iMapId
      }).then((resp) => {
        if (resp.data) {
          if (resp.data.status === 1) {
            try {
              const oMapTools: any = JSON.parse(resp.data.data.maptools);
              resolve(oMapTools);
            } catch (e) { }
          } else {
            // Message d'erreur.
            this.translate.get('COMPONENT_MAP_ENABLE_UTILS_ERROR_GET_TOAST').subscribe((translation) => {
              vitisAlert.alert({
                type: 'error',
                title: translation
              });
            });
          }
        }
      });
    });
  }

  /**
   * Récupère la définition des outils
   * @returns
   */
  private getNativeToolsDef(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.getToolsmap().then(oCustomTool => {
        if (this.vitisCheck.isObject(oCustomTool)) {
          resolve({ ...nativeMapToolsDef, ...oCustomTool });
        } else {
          resolve({ ...nativeMapToolsDef });
        }
      }).catch(() => {
        resolve({ ...nativeMapToolsDef });
      });
    });
  }

  /**
   * Initialise les outils disponibles
   */
  private initToolsAvailable() {
    // Pour chaque postion dans la carte
    if (this.oMapTools) {
      Object.keys(this.oMapToolsDef).forEach((sToolName: string) => {
        let bIsItemAvailable: boolean = true;
        Object.keys(this.oMapTools).forEach((sPosition: string) => {
          const aMapTools: Array<string> = this.oMapTools[sPosition];
          if (aMapTools.indexOf(sToolName) !== -1) {
            bIsItemAvailable = false;
          }
        });
        if (bIsItemAvailable && this.oMapToolsAvailable.indexOf(sToolName) === -1) {
          this.oMapToolsAvailable.push(sToolName);
        }
      });
    }
  }

  /**
   * Supprime un outil de la carte
   * @param sTool
   */
  private removeToolFromMap(sTool: string) {
    if (this.oMapTools[this.sStartPositionDrag]) {
      this.oMapToolsAvailable.push(sTool);
      this.oMapTools[this.sStartPositionDrag].splice(this.oMapTools[this.sStartPositionDrag].indexOf(sTool), 1);
    }
  }

  /**
   * Requête ajax pour récupérer les outils sur la table toolmap
   * @returns
   */
  private getToolsmap(): Promise<any> {
    return new Promise((resolve, reject) => {
      // Charge les modules complémentaires depuis la base
      this.vitisRequest.ajaxRequestPromise({
        url: `${environment.properties.app_api_url}/vmap/toolmap`,
        method: 'GET',
      }).then((resp: any) => {
        const oCustomTool = {};
        if (resp.data && this.vitisCheck.isArray(resp.data.data)) {
          for (let i = 0; i < resp.data.data.length; i++) {
            const oTool = resp.data.data[i];
            if (oTool &&
              oTool.toolmap_id &&
              oTool.name &&
              oTool.icon &&
              oTool.title &&
              oTool.type &&
              oTool.panel_position &&
              oTool.panel_size &&
              oTool.component &&
              oTool.html_content &&
              oTool.js_content) {
              oCustomTool[oTool.name] = {
                name: oTool.name,
                icon: oTool.icon,
                title: oTool.title,
                type: oTool.type,
                panelPosition: oTool.panel_position,
                panelSize: oTool.panel_size,
                component: oTool.component,
                htmlContent: oTool.html_content,
                jsContent: oTool.js_content,
              };
            }
          }
        }
        resolve(oCustomTool);
      });
    });
  }
}
