import { JsonSymbologyStyle } from 'src/modules/vmap/components/models/json-symbology/json-symbology-style.model';
import { JsonSymbologyLabel } from './json-symbology-label.model';


export class JsonSymbologyClassOpacity { // Opacité
    opacity: number;

    constructor(style: any = {}) {
        this.opacity = style && style.opacity ? style.opacity : 1;
    }
}

export class JsonSymbologyClassVisibility { // Visibilité
    minscale: number;
    maxscale: number;

    constructor(style: any = {}) {
        if (style) {
            this.minscale = style.minscale;
            this.maxscale = style.maxscale;
        }
    }
}

export class ClassSubFields {
    opacity?: JsonSymbologyClassOpacity;
    visibility?: JsonSymbologyClassVisibility;
}

export class JsonSymbology {
    type?: string; // singleclass, valueclass, range, simpleexpression, advancedexpression, cluster
    classes?: Array<JsonSymbologyClass>;

    otherFields?: ClassSubFields = {};

    constructor(oDefSym: any = {}) {
        this.otherFields = new ClassSubFields();
        oDefSym.otherFields = oDefSym.otherFields ?? new ClassSubFields(); // Evite le oDefSym.otherFields undefined
        this.otherFields.opacity = new JsonSymbologyClassOpacity(oDefSym.otherFields.opacity ? oDefSym.otherFields.opacity : {});
        this.otherFields.visibility = new JsonSymbologyClassVisibility(oDefSym.otherFields.visibility ? oDefSym.otherFields.visibility : {});
    }
}

export class ExpressionModel {
    operator?: string;
    sql_operator?: string;
    value?: string;
    table?: string;

    constructor(obj: any = {}) {
        this.operator = obj.operator || null;
        this.sql_operator = obj.sql_operator || null;
        this.value = obj.value || null;
        this.table = obj.table || null;
    }
}

export class JsonSymbologyClass extends JsonSymbology {
    id?: number; // Utile si plusieurs classe
    legendName: string; // Nom de la légende
    expression?: string | Array<any>; // expression
    hasLabel: boolean; // Affiche un libellé
    hasStyle: boolean; // Affiche un style carto
    label?: JsonSymbologyLabel; // Définition de la classe
    styles?: Array<any>; // Définition des styles
    opacity?: number; // OPACITY (défaut 1)
    minscale?: number; // MINSCALEDENOM
    maxscale?: number; // MAXSCALEDENOM
    geomType?: string;
    symbolscaledenom?: number;

    constructor(oDefSym: any = {}, typeClass) {
        super(oDefSym);
        this.styles = [];

        if (oDefSym) {
            if (typeClass == 'expression') {
                this.expression = '';
                if (oDefSym.expression) {
                    if (typeof oDefSym.expression == 'object') {
                        this.expression = [];
                        oDefSym.expression.forEach(expression => {
                            this.expression = [...this.expression, new ExpressionModel(expression)];
                        });
                    } else {
                        this.expression = oDefSym.expression;
                    }
                }
                this.id = oDefSym.id;
            } else {
                this.type = oDefSym.type || 'singleclass';
                this.opacity = oDefSym.opacity || 1;

                if (oDefSym.minscale) {
                    this.minscale = oDefSym.minscale;
                }

                if (oDefSym.maxscale) {
                    this.maxscale = oDefSym.maxscale;
                }
            }

            this.legendName = oDefSym.legendName || '';
            this.hasLabel = oDefSym.hasLabel;
            this.hasStyle = oDefSym.hasStyle;
            this.geomType = oDefSym.geomType;
            this.label = new JsonSymbologyLabel(oDefSym.label || {});
            this.symbolscaledenom = oDefSym.symbolscaledenom;

            if (oDefSym.styles) {
                oDefSym.styles.forEach((style, index) => {
                    style.id = index + 1;
                    style.hasStyle = style.hasStyle ? true : false;
                    this.styles.push(new JsonSymbologyStyle(style));
                });
            }
        }
    }
}
