export class JsonSymbologyStylesOpacity { // Opacité
  opacity: number;

  constructor(style: any = {}) {
    if (style) {
      this.opacity = style.opacity;
    }
  }
}

export class JsonSymbologyStylesVisibility { // Visibilité
  minscale: number;
  maxscale: number;

  constructor(style: any = {}) {
    if (style) {
      this.minscale = style.minscale;
      this.maxscale = style.maxscale;
    }
  }
}

export class JsonSymbologyStylesSymbolOutline { // Contour du symbole
  outlineColor: string;
  width: number;

  constructor(style: any = {}) {
    if (style) {
      this.outlineColor = style.outlineColor;
      this.width = style.width;
    }
  }
}

export class JsonSymbologyStylesSymbolOffset { // Décalage du symbole
  offset: string;

  constructor(style: any = {}) {
    if (style) {
      this.offset = style.offset;
    }
  }
}

export class JsonSymbologyStylesSymbolSizeVariation { // Variation de la taille du symbole
  minsize: number;
  maxsize: number;
  minwidth: number;
  maxwidth: number;

  constructor(style: any = {}) {
    if (style) {
      this.minsize = style.minsize;
      this.maxsize = style.maxsize;
      this.minwidth = style.minwidth;
      this.maxwidth = style.maxwidth;
    }
  }
}

export class JsonSymbologyStylesSymbolRepeat { // Symbole répété dans tout le polygone
  symbol?: string;
  rotation?: number;
  fillcolor?: string;
  size?: number;
  outlineColor?: string;
  width?: number;
  gap?: number;
  offset?: number;
  polygonSymbolRepeat: boolean = true;

  constructor(style: any = {}) {
    if (style) {
      this.symbol = style.symbol;
      this.rotation = style.rotation;
      this.fillcolor = style.fillcolor;
      this.size = style.size;
      this.outlineColor = style.outlineColor;
      this.width = style.width;
      this.gap = style.gap;
      this.offset = style.offset;
    }
  }
}

export class JsonSymbologyStylesSymbolCenter { // Symbole au centroïde de chaque polygone
  geomtransform?: string = 'centroid';
  symbol?: string;
  rotation?: number;
  fillcolor?: string;
  size?: number;
  outlineColor?: string;
  width?: number;
  offset?: number;
  polygonSymbolCenter: boolean = true;

  constructor(style: any = {}) {
    if (style) {
      this.symbol = style.symbol;
      this.rotation = style.rotation;
      this.fillcolor = style.fillcolor;
      this.size = style.size;
      this.outlineColor = style.outlineColor;
      this.width = style.width;
      this.offset = style.offset;
    }
  }
}

export class JsonSymbologyStylesLineDash {
  linecap?: string;
  pattern?: string;

  constructor(style: any = {}) {
    if (style) {
      this.linecap = style.linecap;
      this.pattern = style.pattern;
    }
  }
}

export class JsonSymbologyStylesLineSymbol {
  symbol?: string;
  rotation?: number;
  fillcolor?: string;
  size?: number;
  outlineColor?: string;
  width?: number;
  outlineWidth?: number;
  offset?: number;
  gap?: number;
  linejoin?: string;
  initialgap?: string;
  lineSymbol: boolean = true;

  constructor(style: any = {}) {
    if (style) {
      this.symbol = style.symbol;
      this.rotation = style.rotation;
      this.fillcolor = style.fillcolor;
      this.size = style.size;
      this.outlineColor = style.outlineColor;
      this.width = style.width;
      this.outlineWidth = style.outlineWidth;
      this.offset = style.offset;
      this.gap = style.gap;
      this.linejoin = style.linejoin;
      this.initialgap = style.initialgap;
    }
  }
}

export class StyleSubFields {
  opacity?: JsonSymbologyStylesOpacity;
  visibility?: JsonSymbologyStylesVisibility;
  symbolOutline?: JsonSymbologyStylesSymbolOutline;
  symbolOffset?: JsonSymbologyStylesSymbolOffset;
  symbolSizeVariation?: JsonSymbologyStylesSymbolSizeVariation;
  polygonSymbolRepeat?: JsonSymbologyStylesSymbolRepeat;
  polygonSymbolCenter?: JsonSymbologyStylesSymbolCenter;
  lineDash?: JsonSymbologyStylesLineDash;
  lineSymbol?: JsonSymbologyStylesLineSymbol;
}

/**
 * Style
 */
export class JsonSymbologyStyle {
  id: number; // Pour l'ordre des styles
  hasStyle: boolean;
  symbol?: string; // Symbole
  rotation?: string; // ANGLE
  fillcolor?: string; // COLOR
  width?: number;
  size?: string; // SIZE (défaut 10)
  outlineColor?: string;

  otherFields?: StyleSubFields = {};

  constructor(style: any = {}) {

    if (style) {
      this.id = style.id || 1;
      this.hasStyle = style.hasStyle == null ? true : style.hasStyle;
      this.rotation = style.rotation || '0';
      this.symbol = style.symbol || '';
      this.fillcolor = style.fillcolor;
      this.size = style.size || '10';
      this.width = style.width;
      this.outlineColor = style.outlineColor;

      if (style.otherFields) {
        this.otherFields = new StyleSubFields();
        if (style.otherFields.opacity) {
          this.otherFields.opacity = new JsonSymbologyStylesOpacity(style.otherFields.opacity);
        }
        if (style.otherFields.visibility) {
          this.otherFields.visibility = new JsonSymbologyStylesVisibility(style.otherFields.visibility);
        }
        if (style.otherFields.symbolOutline) {
          this.otherFields.symbolOutline = new JsonSymbologyStylesSymbolOutline(style.otherFields.symbolOutline);
        }
        if (style.otherFields.symbolOffset) {
          this.otherFields.symbolOffset = new JsonSymbologyStylesSymbolOffset(style.otherFields.symbolOffset);
        }
        if (style.otherFields.symbolSizeVariation) {
          this.otherFields.symbolSizeVariation = new JsonSymbologyStylesSymbolSizeVariation(style.otherFields.symbolSizeVariation);
        }
        if (style.otherFields.polygonSymbolRepeat) {
          this.otherFields.polygonSymbolRepeat = new JsonSymbologyStylesSymbolRepeat(style.otherFields.polygonSymbolRepeat);
        }
        if (style.otherFields.polygonSymbolCenter) {
          this.otherFields.polygonSymbolCenter = new JsonSymbologyStylesSymbolCenter(style.otherFields.polygonSymbolCenter);
        }
        if (style.otherFields.lineDash) {
          this.otherFields.lineDash = new JsonSymbologyStylesLineDash(style.otherFields.lineDash);
        }
        if (style.otherFields.lineSymbol) {
          this.otherFields.lineSymbol = new JsonSymbologyStylesLineSymbol(style.otherFields.lineSymbol);
        }
      }
    }
  }
}