/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * Remplit le formulaire
   */
  scope.loadProperties = function () {
    let sRessourceURL = env.properties.app_api_url + '/vitis/properties';

    services.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: sRessourceURL,
    }).then((resp) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          // Applique au formulaire
          setTimeout(() => {
            scope.setFormValues(resp.data.data);
          }, 500);
        }
      }
    });
  }

  /**
   * Fonction de mise à jour des properties
   */
  scope.updateProperties = function (oFormValues) {
    return new Promise((resolve, reject) => {
      services.route.data.subscribe((oConfig) => {
        let sRessourceURL = env.properties.app_api_url + '/' + oConfig.options.ressource;

        let oData = {}
        for (const key in oFormValues) {
          if (services.vitisCheck.isDefAndNotNull(oFormValues[key])) {
            oData[key] = oFormValues[key];
          }
        }

        services.vitisRequest.ajaxRequestPromise({
          method: 'PUT',
          url: sRessourceURL,
          data: oData
        }).then((resp) => {
          if (resp.data) {
            if (resp.data.status === 1) {
              this.getProperties().then((oProperties) => {

                // MAJ variable properties
                env.properties = {
                  ...env.properties,
                  ...oProperties
                };

                // toast
                services.translate.get('FORM_PROPERTIES_EDIT_OK').subscribe((translation) => {
                  services.vitisAlert.fireToast({
                    type: 'success',
                    title: translation
                  });
                  resolve();
                });
              }).catch(() => {
                console.error('Could not refresh properties');
              });
            }
          }
        });
      });
    })
  }

  /**
   * Get Properties
   */
  scope.getProperties = function () {
    return new Promise((resolve, reject) => {

      let sRessourceURL = env.properties.app_api_url + '/vitis/properties';
      services.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: sRessourceURL,
      }).then((resp) => {
        if (resp.data) {
          if (resp.data.status === 1) {
            resolve(resp.data.data);
          } else {
            reject();
          }
        } else {
          reject();
        }
      });
    });
  }
}
