/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

    /**
     * Initialise l'outil de gestion de l'étendue
     */
    scope.initOLMapExtent = function () {

        // Dernière fois qu'un carte a été chargée
        scope.lastMapLoad_ = Date.now();

        var lastMapChange = Date.now();
        var oMapFiled = services.oFormReader.getFormField('map_extent');

        /**
         * Change l'étendue de la carte
         * @param {*} sExtent 
         * @param {*} sCoordsys 
         * @param {*} olMap 
         */
        var setOlMapExtent = function (sExtent, sCoordsys, olMap) {
            if (sExtent) {
                var olExtent = sExtent.split('|').map(coord => Number(coord));
                var currMapProjection = olMap.getView().getProjection().getCode();
                var aNewExtent = oMapFiled.transformExtent(olExtent, `EPSG:${sCoordsys}`, currMapProjection);
                oMapFiled.setExtent(aNewExtent);
            }
        }

        /**
         * Calcule l'étendue de la carte, la reprojette et la place sur le champ texte
         */
        var setFormExtentFromMap = function (olMap) {
            var currMapExtent = olMap.getView().calculateExtent();
            var currMapProjection = olMap.getView().getProjection().getCode();
            var formProjection = 'EPSG:' + scope.oFormValues.get('coordsys_id').value;
            var aNewExtent = oMapFiled.transformExtent(currMapExtent, currMapProjection, formProjection);
            lastMapChange = Date.now();
            scope.oFormValues.patchValue({
                extent: aNewExtent.join('|')
            });
        }

        /**
         * Écoute les changements de la carte et met à jour le champ étendue
         */
        var listenMapChanges = function () {
            services.formreaderSrvc.mapChangeEventObservable.subscribe(olMap => {
                if (scope.lastMapLoad_ + 1000 < Date.now()) {
                    setFormExtentFromMap(olMap);
                }
            });
        }

        var firstLoad = true;
        services.formreaderSrvc.mapChangeEventObservable.subscribe(olMap => {
            if (firstLoad) {

                // Set l'étendue du composant carto
                setOlMapExtent(scope.oFormValues.get('extent').value, scope.oFormValues.get('coordsys_id').value, olMap);

                if (scope.mapListened_ !== true) {

                    // Écoute les changements du composant carto
                    setTimeout(() => {
                        listenMapChanges();
                    }, 500);

                    // Met à jour l'extent et la carte par rapport au champ coordsys_id
                    scope.oFormValues.controls['coordsys_id'].valueChanges.subscribe(value => {
                        setFormExtentFromMap(olMap);
                    });

                    // Met à jour l'extent de la carte par rapport au champ extent
                    scope.oFormValues.controls['extent'].valueChanges.subscribe(value => {
                        if (lastMapChange + 1000 < Date.now()) {
                            setOlMapExtent(scope.oFormValues.get('extent').value, scope.oFormValues.get('coordsys_id').value, olMap);
                        }
                    });
                }
            }
            firstLoad = false;
            scope.mapListened_ = true;
        });

        var oDefaultLayerLocation = services.oFormReader.getFormField('default_layer_location_id');
        if (scope.vitisCheck.isDefAndNotNull(oDefaultLayerLocation) &&
            scope.vitisCheck.isDefAndNotNull(oDefaultLayerLocation.oninit)) {
            oDefaultLayerLocation.oninit.subscribe(() => {
                if (scope.sFormDefinitionName === 'update') {
                    scope.vitisRequest.ajaxRequestPromise({
                        url: env.properties.app_api_url + '/vmap/maps/layers/getAll',
                        method: 'GET',
                        params: {
                            filter: {
                                'relation': 'AND',
                                'operators': [{
                                    'column': 'map_id',
                                    'compare_operator': '=',
                                    'value': scope.oParams.id
                                }, {
                                    'column': 'is_location',
                                    'compare_operator': '=',
                                    'value': true
                                }]
                            }
                        }
                    }).then(resp => {
                        if (resp && resp.data && resp.data.data) {
                            const options = resp.data.data.map(layer => `${layer.layer_name}|${layer.layer_id}`);
                            const currentOptionsDatasource = scope.oFormDefinition.datasources.datasource_layers_localisable.options;
                            scope.oFormDefinition.datasources.datasource_layers_localisable.options = [...currentOptionsDatasource, ...options];
                            const currentOptionsSelect = oDefaultLayerLocation.options.choises;
                            oDefaultLayerLocation.options.choises = [...currentOptionsSelect, ...resp.data.data.map(layer => ({
                                "value": layer.layer_id.toString(),
                                "label": layer.layer_name
                            }))];
                        }
                    });
                }
            });
        }

        //Initialise maptools
        if (scope.oFormValues.controls['maptools']) {
            scope.oFormValues.controls['maptools'].setValue(JSON.stringify({
                controls: {
                    zoom_controls: true,
                    zoom_slider: true,
                    map_projection: true,
                    map_title: true,
                    numeric_scale: true,
                    graphic_scale: true,
                    mouse_coordinates: true,
                    map_overview: true
                }
            }));
        }
    }

    scope.checkBeforeInsert = function () {
        if (scope.oFormValues.controls['layers'] && scope.oFormValues.controls['layers'].value) {
            scope.oFormValues.controls['layers'].setValue(scope.oFormValues.value.layers.map(layer => JSON.stringify(layer)));
        }
        scope.oFormValues.markAsPristine();
        return services.SimpleformComponent.sendSimpleForm({ ...scope.oFormValues.value, ...scope.getFormValues(true) });
    }
}