<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour gérer les requêtes de la table layer_map
 *
 */
class DatasourceController extends GenericController {
  /**
   * Récupère toutes les entités pour une route de type GET Module/Objet.
   * @return Response réponse à retourner par l'API
   */
  public function getDatas(): Response {
    $aData = $this->requestToArray();
    // chargement de la connexion à la base de donnée
    $aEntities = $this->getDataWithRequestParams($aData);
    if ($aEntities['status'] == 0) {
      return $this->formatGenericResponseForErrorOnSelect();
    }

    $aSerializerContext = $this->getSerializerContext();

    // Permet de transformer de renvoyer les bonnes valeurs si des properties existe
    $oResponse = array_map(function ($arrValue) {
      foreach ($arrValue as $key => $value) {
        if (gettype($value) === 'string') {
          $arrValue[$key] = $this->oProperties->replacePropertiesInString($value);
        }
      }
      return $arrValue;
    }, $aEntities['data']);

    $aEntities['data'] = $oResponse;

    // transforme le tableau de réponse dans le résultat souhaité
    return $this->formatResponse($aEntities, Response::HTTP_OK, $aSerializerContext);
  }

  /**
   * Créé une entité pour une route de type POST Module/Objet.
   * @return Response réponse à retourner par l'API
   */
  public function postData(): Response {
    $sData = $this->requestToArray();

    if (array_key_exists('directory_path', $sData)) {
      $sDirPath = $sData['directory_path'];
      if (!empty($sDirPath)) {
        $oFileAccessor = $this->getFileAccessor($sDirPath);
        // Check si le chemin directory_path exist
        if (!($oFileAccessor->file_exists($sDirPath))) {
          $sMessage = 'Directory path does not exists';
          return $this->formatGenericError('TREE_VIEW_ERROR_LOADING_FILES', '', $sMessage, Response::HTTP_FORBIDDEN);
        }
      }
    }

    // Sauvegarde en base de donnée
    $oEntity = $this->insertDataInDb(json_encode($sData));

    if ($oEntity == null) {
      return $this->formatGenericResponseForErrorOnInsert();
    }

    if ($oEntity['status'] == 0) {
      return $this->formatGenericResponseForErrorOnInsert();
    }
    return $this->formatResponse($oEntity);
  }
}
