<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\DataAccessor\DatabaseConnection;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour datasource
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="datasource")
 */
class Datasource extends AbstractEntity {
  /**
   * @var Properties
   */
  private $oProperties;
  /**
   * @var int
   * @Column(type="integer", name="datasource_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).datasource_datasource_id_seq")
   */
  private $datasource_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $name;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $description;

  /**
   * @var int
   * @Column(type="integer", nullable=false)
   */
  private $datasourcetype_id;

  /**
   * @var DatasourceType
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\DatasourceType", inversedBy="datasources")
   * @JoinColumn(name="datasourcetype_id", referencedColumnName="datasourcetype_id")
   */
  protected $datasourcetype;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $pg_database;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $pg_server;

  /**
   * @var int
   * @Column(type="integer", nullable=true)
   */
  private $pg_port;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $pg_login;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $pg_password;

  /**
   * @var bool
   * @Column(type="boolean")
   */
  private $pg_public;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $directory_path;

  /**
   * @var string
   * @Column(type="string", length=255, nullable=true)
   */
  private $flux_url;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $flux_type;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $flux_version;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $flux_options;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $flux_login;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $flux_password;

  /**
   * @var ArrayCollection<AbstractLayer>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\AbstractLayer", mappedBy="datasource")
   */
  private $aLayers;

  /**
   * @var ArrayCollection<AbstractPublication>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\AbstractPublication", mappedBy="datasource")
   */
  private $aPublications;

  public function __construct() {
    $this->aLayers = new ArrayCollection();
    $this->aPublications = new ArrayCollection();
  }

  public function __toString(): ?string {
    return $this->getName();
  }

  /**
   * @return int identifiant de la source de données
   */
  public function getDatasourceId(): int {
    return $this->datasource_id;
  }

  /**
   * @return string|null nom de la source de données
   */
  public function getName(): ?string {
    return $this->name;
  }

  /**
   * @param string $name nom de la source de données
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string|null description de la source de données
   */
  public function getDescription(): ?string {
    return $this->description;
  }

  /**
   * @param string $description description de la source de données
   * @return self
   */
  public function setDescription(?string $description): self {
    $this->description = $description;
    return $this;
  }

  /**
   * @return int|null
   */
  public function getDatasourceTypeId(): ?int {
    return $this->datasourcetype_id;
  }

  /**
   * @param int $datasourcetype_id
   * @return self
   */
  public function setDatasourceTypeId(?int $datasourcetype_id): self {
    $this->datasourcetype_id = $datasourcetype_id;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getDatasourceType(): ?string {
    return $this->datasourcetype->getLabel();
  }

  /**
   * @return string|null
   */
  public function getPgDatabase(): ?string {
    return $this->pg_database;
  }

  /**
   * @param string|null $pg_database
   * @return self
   */
  public function setPgDatabase(?string $pg_database): self {
    $this->pg_database = $pg_database;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getPgServer(): ?string {
    return $this->pg_server;
  }

  /**
   * @param string|null $pg_server
   * @return self
   */
  public function setPgServer(?string $pg_server): self {
    $this->pg_server = $pg_server;
    return $this;
  }

  /**
   * @return int|null
   */
  public function getPgPort(): ?int {
    return $this->pg_port;
  }

  /**
   * @param int|null $pg_port
   * @return self
   */
  public function setPgPort(?int $pg_port): self {
    $this->pg_port = $pg_port;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getPgLogin(): ?string {
    return $this->pg_login;
  }

  /**
   * @param string|null $pg_login
   * @return self
   */
  public function setPgLogin(?string $pg_login): self {
    $this->pg_login = $pg_login;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getPgPassword(): ?string {
    return $this->pg_password;
  }

  /**
   * @param string|null $pg_password
   * @return self
   */
  public function setPgPassword(?string $pg_password): self {
    $this->pg_password = $pg_password;
    return $this;
  }

  /**
   * @return bool|null
   */
  public function getPgPublic(): ?bool {
    return $this->pg_public;
  }

  /**
   * @param bool|null $pg_public
   * @return self
   */
  public function setPgPublic(?bool $pg_public): self {
    $this->pg_public = $pg_public;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getDirectoryPath(): ?string {
    return $this->directory_path;
  }

  /**
   * @param string|null $directory_path
   * @return self
   */
  public function setDirectoryPath(?string $directory_path): self {
    $this->directory_path = $directory_path;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getFluxUrl(): ?string {
    return $this->flux_url;
  }

  /**
   * @param string|null $flux_url
   * @return self
   */
  public function setFluxUrl(?string $flux_url): self {
    $this->flux_url = $flux_url;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getFluxOptions(): ?string {
    return $this->flux_options;
  }

  /**
   * @param string|null $flux_options
   * @return self
   */
  public function setFluxOptions(?string $flux_options): self {
    $this->flux_options = $flux_options;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getFluxLogin(): ?string {
    return $this->flux_login;
  }

  /**
   * @param string|null $flux_login
   * @return self
   */
  public function setFluxLogin(?string $flux_login): self {
    $this->flux_login = $flux_login;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getFluxPassword(): ?string {
    return $this->flux_password;
  }

  /**
   * @param string|null $flux_password
   * @return self
   */
  public function setFluxPassword(?string $flux_password): self {
    $this->flux_password = $flux_password;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getFluxVersion(): ?string {
    return $this->flux_version;
  }

  /**
   * @param string|null $flux_version
   * @return self
   */
  public function setFluxVersion(?string $flux_version): self {
    $this->flux_version = $flux_version;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getFluxType(): ?string {
    return $this->flux_type;
  }

  /**
   * @param string|null $flux_type
   * @return self
   */
  public function setFluxType(?string $flux_type): self {
    $this->flux_type = $flux_type;
    return $this;
  }

  /**
   * retourne la liste des couches
   * @return array
   */
  public function getLayers(): array{
    return $this->aLayers->toArray();
  }

  /**
   * retourne la liste des couches
   * @return array
   */
  public function getPublications(): array{
    return $this->aPublications->toArray();
  }

  /**
   * Retourne la configuration pour une connexion postgres
   * @return array|null
   */
  public function getPostgresConnexionData(bool $bAllowAdminLoginPwd = true, ?string $sDbName = null): ?array{
    // si ce n'est pas une base postgres
    if ($this->datasourcetype->getDatasourceTypeId() !== 1) {
      return null;
    }

    $sDefaultLogin = null;
    $sDefaultPwd = null;
    $sDefaultHost = null;
    $sDefaultPort = null;
    $sDefaultDatabase = $sDbName;

    if ($this->pg_public !== true) {
      $sDefaultHost = $this->oProperties->get('db_server');
      $sDefaultPort = $this->oProperties->get('db_port');

      if (empty($sDefaultDatabase) && $sDefaultDatabase !== 'null') {
        $sDefaultDatabase = $this->oProperties->get('db_name');
      }

      if ($bAllowAdminLoginPwd) {
        $sDefaultLogin = $this->oProperties->get('db_superuser');
        $sDefaultPwd = $this->oProperties->get('db_password');
      }
    }

    $aData = [
      'login' => $this->pg_login ?? $sDefaultLogin,
      'password' => $this->pg_password ?? $sDefaultPwd,
      'server' => $this->pg_server ?? $sDefaultHost,
      'port' => $this->pg_port ?? $sDefaultPort,
      'sgbd' => DatabaseConnection::SGBD,
      // par défaut le ssl_mode sera en prefer
    ];

    if (!empty($sDefaultDatabase)) {
      $aData['database'] = $sDefaultDatabase;
    }

    return $aData;
  }

/* ---------------------------------------------------------------- */
/* -------------------------- Interfaces -------------------------- */
/* ---------------------------------------------------------------- */

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @param Properties $oService
   * @return void
   */
  public function setPropertiesService(Properties $oService): void {
    $this->oProperties = $oService;
  }
}
