<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour searchmode
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="searchmode")
 */
class SearchMode extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="searchmode_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).searchmode_searchmode_id_seq")
   */
  private $searchmode_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $label;

  /**
   * @var ArrayCollection<LayerPostgres>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\LayerPostgres", mappedBy="location_search_mode")
   */
  private $aLayersPostgres;

  public function __construct() {
    $this->aLayersPostgres = new ArrayCollection();
  }

  public function __toString(): ?string {
    return $this->getLabel();
  }

  /**
   * @return int identifiant du theme
   */
  public function getSearchModeId(): int {
    return $this->searchmode_id;
  }

  /**
   * @return string|null label du theme
   */
  public function getLabel(): ?string {
    return $this->label;
  }

  /**
   * @param string $label label du theme
   * @return self
   */
  public function setLabel(?string $label): self {
    $this->label = $label;
    return $this;
  }

  /**
   * retourne la liste des couches postgres
   * @return array
   */
  public function getLayersPostgres(): array{
    return $this->aLayersPostgres->toArray();
  }
}
