<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour source
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="source")
 */
class Source extends AbstractEntity {
  /**
   * @var EntityManager
   */
  private $oEntityManager;
  /**
   * @var LoggerManager
   */
  private $oLoggerManager;

  /**
   * @var int
   * @Column(type="integer", name="source_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).source_source_id_seq")
   */
  private $source_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $name;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $url;

  /**
   * @var ArrayCollection<AbstractLayer>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\AbstractLayer", mappedBy="source")
   */
  private $aLayers;

  public function __construct() {
    $this->aLayers = new ArrayCollection();
  }

  public function __toString(): ?string {
    return $this->getName();
  }

  /**
   * @return int identifiant de la source
   */
  public function getSourceId(): int {
    return $this->source_id;
  }

  /**
   * @return string|null nom de la source
   */
  public function getName(): ?string {
    return $this->name;
  }

  /**
   * @param string $name nom de la source
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string|null url de la source
   */
  public function getUrl(): ?string {
    return $this->url;
  }

  /**
   * @param string $url url de la source
   * @return self
   */
  public function setUrl(?string $url): self {
    $this->url = $url;
    return $this;
  }

  /**
   * retourne la liste des couches
   * @return array
   */
  public function getLayers(): array{
    return $this->aLayers->toArray();
  }
}
