import { Injectable } from '@angular/core';
import { Router, CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, Route } from '@angular/router';
import { AuthService } from './auth.service';

@Injectable({
  providedIn: 'root'
})
export class AuthGuardService implements CanActivate {

  constructor(
    public auth: AuthService,
    public router: Router
  ) { }

  /**
   * Redirige l'utilisateur sur la page de login si il n'est pas authentifié
   * @param {ActivatedRouteSnapshot} route
   * @param {RouterStateSnapshot} state
   * @return {boolean} true si l'utilisateur est authentifié, sinon false
   */
  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): boolean {

    const IsWidget: boolean = (route.routeConfig.path == 'widget');
    const AllowLocalStorage: boolean = (!IsWidget);

    if (this.auth.isAuthenticated(AllowLocalStorage)) {
      return true;
    }

    console.error('AuthGuardService: user not authorized');

    if (this.auth.isConnecting) {
      return false;
    }

    if (IsWidget) {
      const url: string = state.url.split('?')[0];
      const child = this.findRouteChildren(route, url);

      if (child) {
        if (child.data.privilege === 'public') {
          return true;
        }
      }

      this.router.navigate(['error', 'widget_expired_token'], { queryParams: route.queryParams });
    } else {
      this.router.navigate(['login'], { queryParams: route.queryParams });
    }
    return false;
  }

  /**
   * cherche l'enfant correspondant à la route complète
   * @param {ActivatedRouteSnapshot} route
   * @param {string} sRoute route complète provenant du statut courant
   * @returns {Route|null}
   */
  private findRouteChildren(route: ActivatedRouteSnapshot, sRoute: string): Route {
    for (const child of route.routeConfig.children) {
      const sChildRoute: string = '/' + route.routeConfig.path + '/' + child.path;
      if (sChildRoute == sRoute) {
        return child;
      }
    }
    return null;
  }
}
