import { Component, NgModule, Compiler, ElementRef, ViewContainerRef, ViewChild, OnInit } from '@angular/core';
import { environment } from 'src/environments/environment';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { CommonModule } from '@angular/common';
import { TranslateModule } from '@ngx-translate/core';
import dataTablesFrench from 'src/assets/lang/datatables-fr.json';
import dataTablesEn from 'src/assets/lang/datatables-en.json';
import { WorkspaceListButton } from '../workspacelist/workspacelist.model';
import { DataGridOptions } from 'src/app/vitis/data-grid/data-grid.model';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { GlobalFilePickerComponent } from 'src/app/vitis/formreader/components/file-picker/global-file-picker/global-file-picker.component';
import { FormBuilder, FormGroup, Validators, AbstractControl } from '@angular/forms';
import exportFromJSON from 'export-from-json';

declare var $: any;

@Component({
  selector: 'app-data-grid',
  templateUrl: './data-grid.component.html',
  styleUrls: ['./data-grid.component.scss']
})
export class DataGridComponent implements OnInit {
  // @ViewChild(FormreaderComponent, { static: false }) oFormReader: FormreaderComponent;

  // Options de la grille de données.
  public oDataGridOptions: DataGridOptions;
  // Instance de la grille de données.
  public dataTableInstance: any;
  // Id des éléments html
  public uniqueId: String = this.getUniqueId_();
  public tableId: String = 'table' + this.uniqueId;
  public toolbarId: String = 'toolbar' + this.uniqueId;

  // Formats d'exportation de la liste.
  public exportFormats: any = {
    'csv': {
      replace_label: true,
      no_data_value: ''
    },
    'xls': {
      replace_label: true,
      no_data_value: ''
    },
    'json': {
      replace_label: false,
      no_data_value: null
    },
    'xml': {
      replace_label: false,
      no_data_value: ''
    }
  };
  public form: FormGroup;

  constructor(
    private vitisCheck: VitisCheck,
    private compiler: Compiler,
    private element: ElementRef,
    public viewContainer: ViewContainerRef,
    private translate: TranslateService,
    private auth: AuthService,
    private vitisRequest: VitisRequest,
    private fb: FormBuilder
  ) { }

  ngOnInit() {
    // formulaire d'exportation de la liste.
    this.form = this.fb.group({
      export_file_name: ['export', Validators.required],
      export_file_format: ['csv', Validators.required],
      export_rows_number: ['20']
    });
  }

  /**
   * Initialise la table
   * @param {DataGridOptions} DataGridOptions
   * @returns object
   */
  public initTable(oDataGridOptions: DataGridOptions): object {
    const self = this;
    this.oDataGridOptions = oDataGridOptions;

    this.addDataTableRenders();

    // Composant filepicker
    if (this.vitisCheck.isArray(this.oDataGridOptions.customTemplateComponents)) {
      this.oDataGridOptions.customTemplateComponents.push(GlobalFilePickerComponent);
    } else {
      this.oDataGridOptions.customTemplateComponents = [
        GlobalFilePickerComponent
      ];
    }
    if (!this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.columns) || this.vitisCheck.isEmptyObject(this.oDataGridOptions.datagridOptions.columns)) {
      this.oDataGridOptions.datagridOptions.columns = oDataGridOptions.datatableOptions.columns;
    }

    for (let i = 0; i < this.oDataGridOptions.datatableOptions.columns.length; i++) {
      // Traductions du libellé des colonnes.
      const oColumn = this.oDataGridOptions.datatableOptions.columns[i];
      if (this.vitisCheck.isDefAndNotNull(oColumn.title)) {
        this.translate.get(oColumn.title).subscribe((sTranslation: string) => {
          oColumn.title = sTranslation;
        });
      }

      // ajout du render pour les types spécifique (image, document, link)
      if (this.vitisCheck.isDefAndNotNull(oColumn.type)) {
        switch (oColumn.type) {
          case 'image':
            oColumn.template = '<app-global-file-picker class="datatable-btn" *ngIf="row[column.data]" [oField]="{ displayOnly: true }" [aUrls]="row[column.data].split(\',\')" [sFilePickerType]="\'image\'"></app-global-file-picker>';
            oColumn.render = $.fn.dataTable.render.template(oColumn, i);
            break;
          case 'document':
            oColumn.template = '<app-global-file-picker class="datatable-btn" *ngIf="row[column.data]" [oField]="{ displayOnly: true }" [aUrls]="row[column.data].split(\',\')" [sFilePickerType]="\'file\'"></app-global-file-picker>';
            oColumn.render = $.fn.dataTable.render.template(oColumn, i);
            break;
          case 'link':
            oColumn.render = $.fn.dataTable.render.link();
            break;
          case 'html':
            break;
          default:
            oColumn.render = $.fn.dataTable.render.text();
            break;
        }
      } else {
        if (!this.vitisCheck.isDefAndNotNull(oColumn.template)) {
          oColumn.render = $.fn.dataTable.render.text();
        }
      }
    }

    // Table
    oDataGridOptions.datatableOptions.language = environment.properties.language === 'fr' ? dataTablesFrench : dataTablesEn;
    this.dataTableInstance = $('#' + this.tableId).DataTable(oDataGridOptions.datatableOptions);

    // Selection de toutes les lignes
    $('th.select-checkbox').on('click', (event) => {
      const target = $(event.target);
      if (target.hasClass('selected')) {
        this.dataTableInstance.rows().deselect();
        target.removeClass('selected');
      } else {
        this.dataTableInstance.rows().select();
        target.addClass('selected');
      }
    });

    // S'il y a des lignes non-sélectionnées décoche le checkbox d'entête
    this.dataTableInstance.on('select deselect draw', () => {
      if (this.dataTableInstance.rows({
        selected: true
      }).count() !== this.dataTableInstance.rows().count()) {
        $('th.select-checkbox').removeClass('selected');
      } else {
        $('th.select-checkbox').addClass('selected');
      }
    });

    // Permet de ne pas selectionner lors du click sur un bouton
    this.dataTableInstance.on('click', '.datatable-btn', () => {
      const selectedRows = this.dataTableInstance.rows({
        selected: true
      });
      setTimeout(() => {
        this.dataTableInstance.rows().deselect();
        selectedRows.select();
      });
    });

    // Boutons display
    const oDatatable = this.dataTableInstance;
    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.displayEvent)) {
      this.dataTableInstance.on('click', '.datatable-btn-display', function () {
        const data = oDatatable.row($(this).parents('tr')).data();
        self.oDataGridOptions.displayEvent(data);
      });
    }
    // Boutons update
    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.updateEvent)) {
      this.dataTableInstance.on('click', '.datatable-btn-edit', function () {
        const data = oDatatable.row($(this).parents('tr')).data();
        self.oDataGridOptions.updateEvent(data);
      });
    }

    // Draw fields event
    this.dataTableInstance.on('draw', () => {
      this.formatFields(this.oDataGridOptions);

      // Colonnes éditables.
      if (oDataGridOptions.datatableOptions.editableColumns) {
        for (let i = 0; i < this.oDataGridOptions.datatableOptions.columns.length; i++) {
          if (oDataGridOptions.datatableOptions.editableColumns.includes(this.oDataGridOptions.datatableOptions.columns[i].name)) {
            this.dataTableInstance.columns(i).nodes().flatten().to$().attr('contenteditable', true);
          }
        }
      }
    });

    // Retourne l'instance de la grille de données.
    return this.dataTableInstance;
  }

  /**
   * Formate les champs d'un tableau selon le template d'une colonne
   * @param {any} oDataGridOptions
   */
  public formatFields(oDataGridOptions: any) {

    if (oDataGridOptions &&
      oDataGridOptions.datagridOptions) {

      const aTemplateComponents = oDataGridOptions.customTemplateComponents;
      const aFieldRefs = this.element.nativeElement.getElementsByTagName('field-render');
      const aComponents = [...aTemplateComponents];

      for (let i = 0; i < aFieldRefs.length; i++) {

        const oFieldRef = aFieldRefs[i];
        const oRow = JSON.parse(decodeURIComponent(oFieldRef.innerText));
        const oColumn = this.oDataGridOptions.datagridOptions.columns[oFieldRef.getAttribute('column_number')];

        // Vide le cache du composant
        this.compiler.clearCache();

        // Création du composant dynamique
        const oComponent = this.createDynamicComponent_(oColumn.template, oRow, oColumn);
        aComponents.push(oComponent);
      }

      // Création d'un module dynamique
      const oModule = this.createDynamicModule_(aComponents);

      try {
        // Compile le module et le component
        this.compiler.compileModuleAndAllComponentsAsync(oModule)
          .then(factories => {
            for (let i = aFieldRefs.length - 1; i >= 0; i--) {

              const oFieldRef = aFieldRefs[i];
              const oParentNode = oFieldRef.parentNode;

              // Coposant dynamiquement crée précédement et instancié
              const componentIndex = aTemplateComponents.length + i;
              const componentFactory = factories.componentFactories[componentIndex];

              // Crée le component et l'ajout à la vue
              const componentRef = this.viewContainer.createComponent(componentFactory);

              // Ajoute à l'élément la nouvelle chaine
              oParentNode.appendChild(componentRef.location.nativeElement);

              // Supprime la chaine de base
              oFieldRef.parentNode.removeChild(oFieldRef);
            }
            this.dataTableInstance.columns.adjust();
          });
      } catch (error) {
        console.error(error);
      }
    }
  }

  /**
   * Initialise l'événement d'un bouton
   * @param {WorkspaceListButton} button
   * @param {oWorkspaceList} any container de l'objet workspacelist
   */
  public initButtonEvent_(button: WorkspaceListButton, oWorkspaceList?: any) {
    // Transforme la chaine de caractères en vrai fonction
    if (this.vitisCheck.isString(button.event)) {
      button.sEvent = button.event;
    }
    if (this.vitisCheck.isString(button.eventOnClose)) {
      button.sEvent = button.eventOnClose;
    }
    if (this.vitisCheck.isString(button.sEvent)) {
      button.event = () => {

        // Nom de la fonction sans les paranthèses
        let sFunction = button.sEvent;
        if (sFunction.indexOf('(') !== -1) {
          sFunction = sFunction.substr(0, sFunction.indexOf('('));
        }

        // Lance la fonction depuis workspacelist
        if (this.vitisCheck.isDefAndNotNull(oWorkspaceList)) {
          if (this.vitisCheck.isFunction(oWorkspaceList[sFunction])) {
            return oWorkspaceList[sFunction].call(oWorkspaceList);
          }
        }

        // Lance la fonction depuis datagrid
        if (this.vitisCheck.isFunction(this[sFunction])) {
          return this[sFunction].call(this);
        }
      };
    }

    // Recursivité
    if (this.vitisCheck.isArray(button.buttons)) {
      for (let i = 0; i < button.buttons.length; i++) {
        const buttonChild = button.buttons[i];
        this.initButtonEvent_(buttonChild, oWorkspaceList);
      }
    }
  }

  /**
   * Initialise l'affichage d'un bouton
   * @param {WorkspaceListButton} button
   * @param {oWorkspaceList} any container de l'objet workspacelist
   */
  public initButtonDisplay_(button: WorkspaceListButton, oWorkspaceList?: any) {
    // Privilège du bouton.
    if (this.vitisCheck.isDefAndNotNull(button.privilege) && (typeof (button.privilege) == 'string')) {
      const aUserPrivileges = this.auth.getPrivileges();
      if (aUserPrivileges.indexOf(button.privilege) != -1) {
        button.display = true;
      } else {
        button.display = false;
      }
    }

    // Recursivité
    if (this.vitisCheck.isArray(button.buttons)) {
      for (let i = 0; i < button.buttons.length; i++) {
        const buttonChild = button.buttons[i];
        this.initButtonDisplay_(buttonChild, oWorkspaceList);
      }
    }
  }

  /**
   * Récupère les lignes selectionnées dans le tableau
   * @returns Array
   */
  public getSelectedRows_(): Array<any> {
    const aRows = this.dataTableInstance.rows({ selected: true }).data();
    return aRows;
  }

  /**
   * Récupère identifiants des lignes selectionnées dans le tableau
   * @returns Array
   */
  public getSelectedIds_(): Array<any> {

    const aIds = [];
    const aRows = this.getSelectedRows_();

    for (let i = 0; i < aRows.length; i++) {
      const oRow = aRows[i];

      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
        if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.id_field)) {
          if (this.vitisCheck.isDefAndNotNull(oRow)) {
            if (this.vitisCheck.isDefAndNotNull(oRow[this.oDataGridOptions.datagridOptions.id_field])) {

              const sId = oRow[this.oDataGridOptions.datagridOptions.id_field];
              aIds.push(sId);
            }
          }
        }
      }
    }
    return aIds;
  }

  /**
   * Trouve un identifiant html unique
   * @returns string
   */
  private getUniqueId_(): string {
    let id = '_' + Math.random().toString(36).substr(2, 9);
    if (document.getElementById(id)) {
      id = this.getUniqueId_();
    }
    return id;
  }

  /**
   * Recherche dans les colonnes du mode celle dont le field correspond à sColumnName
   * @param {String} sColumnName
   * @returns any
   */
  public getColumnByName_(sColumnName: String): any {

    if (this.oDataGridOptions &&
      this.oDataGridOptions.datagridOptions &&
      this.oDataGridOptions.datagridOptions.columns) {

      // Pour chaque colonne
      for (let i = 0; i < this.oDataGridOptions.datagridOptions.columns.length; i++) {
        const column = this.oDataGridOptions.datagridOptions.columns[i];

        if (column.field.trim() === sColumnName.trim()) {
          return column;
        }
      }
    }
  }

  /**
   * Crée un composant Angular dynamiquement
   * @param template
   * @param row
   * @param column
   */
  private createDynamicComponent_(template: string, row: any, column: any) {
    return Component({
      template
    })(class {
      public row: Array<any>;
      public column: Array<any>;
      constructor() {
        this.row = row;
        this.column = column;
      }
    });
  }

  /**
   * Crée un module angular dynamiquement
   * @param aComponents
   */
  private createDynamicModule_(aComponents: any) {
    return NgModule({
      declarations: aComponents,
      imports: [
        CommonModule,
        TranslateModule.forRoot()
      ]
    })(class { });
  }

  /**
   * Affichage de la fenêtre modale d'exportation de la liste.
   */
  public displayExportModal() {
    $('#datagrid_export_modal').modal('show');
  }

  /**
   * Téléchargement du fichier d'exportation de la liste.
   */
  public exportDataGrid() {
    // récupération des valeurs du formulaire ou valeur par défaut
    const iMaxRow: number = (this.form.value.export_rows_number) ? this.form.value.export_rows_number : 0;
    const sFormat: string = (this.form.value.export_file_format) ? this.form.value.export_file_format : 'csv';
    const sFileName: string = (this.form.value.export_file_name) ? this.form.value.export_file_name : 'export';
    this.getDataGridDataForExport(iMaxRow, this.exportFormats[sFormat]).then((aData: any[]) => {
      switch (sFormat) {
        case 'csv':
        case 'json':
        case 'xml':
        case 'xls':
          const aConfig: any = {
            data: aData,
            fileName: sFileName,
            exportType: sFormat,
          };
          // transforme le json et télécharge le résultat
          exportFromJSON(aConfig);
          break;
        default:
          console.error('format not available : ' + sFormat);
          break;
      }
    }).catch(() => {
      console.error('error occured while exporting data');
    }).finally(() => {
      $('#datagrid_export_modal').modal('hide');
      // on recharge les données sinon les templates sautent
      this.dataTableInstance.ajax.reload(null, false);
    });
  }

  /**
   * récupère les lignes visible dans data-grid, remplace les clé et transforme le tout en array type json
   * @param {number} iNbRows nombre de ligne à récupérer si 0 on récupère tout
   * @param {any} aConfig
   * @returns {any} json
   */
  private getDataGridDataForExport(iNbRows: number = 0, aConfig: any = {}): Promise<any[]> {
    return new Promise((resolve, reject) => {
      const aFinalData: any[] = [];
      const aColumns: any[] = this.oDataGridOptions.datatableOptions.columns;

      const oDataOption = this.dataTableInstance.ajax.params();
      oDataOption.length = iNbRows;

      // si 0 on récupère toutes les lignes
      if (iNbRows == 0) {
        delete oDataOption.length;
      }

      this.oDataGridOptions.datatableOptions.ajax(oDataOption, (oReturn) => {
        const aData: any = oReturn.data;
        for (let i = 0; i < aData.length; i++) {
          const aTmpItem: any = {};

          for (const oColumn of aColumns) {
            // élimine les colonnes useless au début du tableau
            // boite à cocher bouton d'ouverture de formulaire
            // et teste si la colonne est exportable
            if ('title' in oColumn && 'data' in oColumn) {
              // si la colonne à une correspondance dans les données
              // on récupère la donnée en remplaçant la clé par le label de la colonne
              const sKey: string = (aConfig.replace_label) ? oColumn.title : oColumn.data;
              if (this.vitisCheck.isDefAndNotNull(aData[i][oColumn.data])) {
                aTmpItem[sKey] = aData[i][oColumn.data];
              } else {
                aTmpItem[sKey] = aConfig.no_data_value;
              }
            }
          }
          aFinalData.push(aTmpItem);
        }
        resolve(aFinalData);
      }, null);
    });
  }

  /**
   * Ajoute les fonctions pour les renders custom
   */
  private addDataTableRenders() {
    const this_ = this;

    // - Template -> utile pour image/document
    $.fn.dataTable.render.template = (oColumn: any, i: number) => {
      return (data: any, type: any, row: any, meta: any) => {
        if (oColumn.template) {
          data = '<field-render column_number="' + i + '">' + encodeURIComponent(JSON.stringify(row)) + '</field-render>';
        }
        return data;
      };
    };

    // - Link
    $.fn.dataTable.render.link = () => {
      return (data, type, row) => {
        if (this_.vitisCheck.isDefAndNotNull(data) && data != '') {
          return '<a class="datatable-btn" href="' + data + '">Lien ?</a>';
        }
        return data;
      };
    };
  }
}