import { EventEmitter } from '@angular/core';
import { NgbModalOptions } from '@ng-bootstrap/ng-bootstrap';

/**
 * Options boutons
 */
export class DragAndDropGridButton {
  event?: any;
  type: string; // button / group / modal
  label: string;
  sEvent?: string;
  name?: string;
  class?: string;
  icon?: string;
  buttons?: Array<DragAndDropGridButton>;
  modal_component?: string;
  modal_options?: NgbModalOptions; // https://ng-bootstrap.github.io/#/components/modal/api#NgbModalOptions
  refreshGridOnClose?: boolean; // recharge la grille lors de la fermeture (default: false)
  eventOnClose?: string; // Événement à lancer à la fermeture
  modal_need_selected_row?: boolean; // Vérifie qu'au moins 1 ligne de la grille est sélectionné (default: false)
  modal_label_no_row_error?: string; // Message d'erreur à afficher si aucune ligne n'a été sélectionné alors que c'était néccessaire
  privilege?: string; // Affichage du bouton en fonction d'un privilère
  display?: boolean; // Affichage du bouton (string = privilège obligatoire).
}

/**
 * Options des colonnes Workspacelist
 */
export class DragAndDropGridColumn {
  title: string; // Titre à afficher
  field: string; // Nom de la colonne
  sortable?: boolean; // Ordre modifiable
  template?: string; // Template à utiliser
  resizeable?: boolean; // Non disponible
  width?: string; // Non disponible
  visible?: boolean; // Colonne visible (mais utilisable) vaut true par défaut
  searchable?: boolean; // Permet de désectiver le filtre personnalisé sur cette colonne
  searchSuggestions?: any; // Suggestions (false ou tableau)
}

/**
 * Options pour composant de type WorkspaceList
 */
export class DragAndDropGridOptions {
  module?: string; // (ex: gtf) permet de charger les composant du module
  ressource?: string; // (ex: gtf/orders) ressource API à attaquer
  ressource_source: string; // Ressource à appeler pour le datagrid associé
  id_field?: string; // (ex: order_id) colonne id de l'API
  index_field: number; // Champ contenant le numéro de l'index
  name_field: string; // Champ contenant le label à afficher
  parent_id_field?: string; // cas de l'utilisation de workspacelist dans une section, permet soit de faire un remplacement dans l'URL avec :id_parent soit de faire un filtre sur l'élément parent
  disable_filter_parent?: boolean; // Permet de désactiver le filtrage via les noeud sur le parent automatiquement
  order_by?: Array<any>; // [[numbero de la colonne], [asc / desc]]
  columns: Array<DragAndDropGridColumn>;
  buttons: Array<DragAndDropGridButton>;
  edition_component?: string; // Utilisation d'un composant autre que simpleform pour l'insertion et l'edition
  height?: number; // Hauteur de la grille
  singleSelect?: boolean; // Désactivation de la selection multiple
  max_rows?: number; // Nombre maximum d'enregistrements à afficher
  reversed?: boolean; // Inverse l'ordre d'affichage
}

/**
 * Options pour rafraichissement automatique
 */
export class DragAndDropGridAutorefreshOptions {
  interval: number; // Intervalle de rafraichissement
  compare_columns?: Array<string>; // Colonnes à comparer
}