import { Component, OnInit, Input, Inject, Host, ElementRef, AfterViewInit } from '@angular/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { FieldsService } from '../../services/fields.service';

/**
 * Type bouton
 *
 * Arguments:
 * type : type de champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * class : classe à utiliser ("btn-ungroup btn-group-sm")
 * buttons : tableau d'objets contenant les définitions des boutons
 * buttons[].type : type de champ (button, submit)
 * buttons[].label : libellé
 * buttons[].class : classe du bouton (btn-primary)
 * buttons[].event : fonction à lancer au click du bouton, doit être contenue dans scope,
 * le formReader ou dans le eventContainer définit dans le formulaire
 *
 * Exemple:
 * {
 *    "type": "button",
 *    "class": "btn-ungroup btn-group-sm",
 *    "nb_cols": 12,
 *    "buttons": [
 *      {
 *        "type": "button",
 *        "label": "log_values",
 *        "class": "btn-primary",
 *        "event": "logValues()"
 *      }
 *    ]
 * }
 */
@Component({
  selector: 'app-button',
  templateUrl: './button.component.html',
  styleUrls: ['./button.component.scss']
})
export class ButtonComponent extends FormreaderField implements OnInit, AfterViewInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;
  @Input() oFormReader: any;
  @Input() tab: any;

  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  ngOnInit() {
  }

  ngAfterViewInit() {
    // Tooltip sur tous les boutons.
    if (this.vitisCheck.isDefAndNotNull(this.oField.buttons)) {
      for (let i = 0; i < this.oField.buttons.length; i++) {
        this.tooltip(this.oFormValues, this.oField.buttons[i], this.element.nativeElement.children[i]);
      }
    }
  }

  /**
   * Appelle un event appelé par un bouton
   * @param  {any} oButton
   */
  public callBtnEvent(oButton: any) {
    this.oFormReader.callBtnEvent(oButton);
  }

  /**
   * Ne retourne pas de valeur
   * @param oFormValues
   * @param oField
   *
   * @returns null
   */
  getValue(oFormValues: any, oField: any) {
    return null;
  }
}
