import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

/**
 * Type radio
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * required? : champ requis
 * default_value? : valeur par défaut
 * options : liste des possibilités
 * options.choises[].label : libellé
 * options.choises[].value : valeur à utiliser
 * nb_cols_label? :  si inline, nombre de colonnes à utiliser (de 1 à 12) pour le label
 * inline? : Label et boutons sur la même ligne
 *
 * Exemple:
 * {
 *   "type": "radio",
 *   "name": "Niveau valide ?",
 *   "label": "Radio",
 *   "nb_cols": 12,
 *   "required": true,
 *   "options": {
 *     "choices": [{
 *         "label": "Oui",
 *         "value": true,
 *       },
 *       {
 *         "label": "Non",
 *         "value": false,
 *       }
 *     ]
 *   }
 * }
 */
@Component({
  selector: 'app-radio',
  templateUrl: './radio.component.html',
  styleUrls: ['./radio.component.scss']
})
export class RadioComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      return this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }

}
