import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Validators } from '@angular/forms';

/**
 * Type url
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * required? : champ requis
 * default_value? : valeur par défaut
 * pattern? : regex à utiliser pour contraindre la saisie
 * maxLength? : Nombre de caractères maximum.
 *
 * Exemple:
 * {
 *     "type": "url",
 *     "name": "url_name",
 *     "label": "Nom",
 *     "required": false,
 *     "nb_cols": 12
 * }
 */
@Component({
  selector: 'app-url',
  templateUrl: './url.component.html',
  styleUrls: ['./url.component.scss']
})
export class UrlComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  public urlRegex = '^https?:\\/\\/.{1,}';

  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      return this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);

    // Validateurs
    this.updateFieldValidator_(this.oField);
  }

  /**
   * Création d'un validateur spécifique
   * @param oField
   */
  private updateFieldValidator_(oField: any) {

    let aValidators = [];

    // Récupère les validators inclus par FormReader
    if (this.vitisCheck.isArray(oField.validators_list)) {
      aValidators = oField.validators_list;
    }

    // Set uniquement si pattern n'est pas déjà défini
    if (!this.vitisCheck.isDefAndNotNull(oField.pattern)) {
      aValidators.push(Validators.pattern(this.urlRegex));
    }

    // Ajoute les validateurs
    this.oFormValues.controls[oField.name].setValidators(aValidators);

    // Messages
    if (!oField.validation_messages) {
      oField.validation_messages = [];
    }
    oField.validation_messages.push({
      type: 'pattern',
      message: this.translate.instant('FORM_READER_ERROR_URL')
    });
  }



}
