import { Component, OnInit, Input } from '@angular/core';
import { Map } from 'ol';
import { ZoomSlider } from 'ol/control';

@Component({
  selector: 'app-core-controls-zoom',
  templateUrl: './core-controls-zoom.component.html',
  styleUrls: ['./core-controls-zoom.component.scss']
})
export class CoreControlsZoomComponent implements OnInit {

  @Input() olMap: Map;
  @Input() showSlider: boolean;
  @Input() isSmallScreen: boolean;


  public showZoomControls: boolean = true;
  public sZoomSliderId = `zoom-slider-${Date.now()}`;

  constructor() { }

  ngOnInit(): void {
    if (this.olMap.get('enabled_controls')) {
      // Si enabled_controls est renseigné, met à jour l'affichage des contrôles de zoom
      this.showZoomControls = this.olMap.get('enabled_controls').zoom_controls;
      this.showSlider = this.olMap.get('enabled_controls').zoom_slider;
    } else {
      // Tout est montré par défaut
      this.showZoomControls = true;
      this.showSlider = false;
    }
    if (this.showSlider === true) {
      this.initZoomSlider_();
    }
  }

  /**
   * Effectue un zoom positif
   */
  public zoomMapIn() {
    this.olMap.getView().animate({
      zoom: this.olMap.getView().getZoom() + 1,
      duration: 250
    });
  }

  /**
   * Effectue un zoom negatif
   */
  public zoomMapOut() {
    this.olMap.getView().animate({
      zoom: this.olMap.getView().getZoom() - 1,
      duration: 250
    });
  }

  /**
   * Initialise le slider de zoom
   */
  private initZoomSlider_() {
    setTimeout(() => {
      const zoomSlider = new ZoomSlider();
      const domSliderElem = document.getElementById(this.sZoomSliderId);
      zoomSlider.setTarget(domSliderElem);
      this.olMap.addControl(zoomSlider);
    });
  }
}
