import { Component, Input, OnInit } from '@angular/core';
import { Map } from 'ol';
import { MapJSON } from 'src/app/vitis/map-core/services/map-models.model';

@Component({
  selector: 'app-core-controls',
  templateUrl: './core-controls.component.html',
  styleUrls: ['./core-controls.component.scss']
})
export class CoreControlsComponent implements OnInit {

  @Input() olMap: Map;
  @Input() templateType: string;

  public enabledControls: any = {
    zoom_controls: false,
    zoom_to_extent: false,
    zoom_slider: false,
    geolocation: false,
    map_projection: false,
    location_controls: false,
  };

  public mapJSON: MapJSON;

  constructor() { }

  ngOnInit(): void {

    this.mapJSON = this.olMap.get('map_json');

    if (this.mapJSON) {
      this.initMapControls_(this.mapJSON);
    }
  }

  /**
   * Initialise les controles de la carte
   * @param  {any} oControls
   */
  private initMapControls_(mapJSON: any) {
    const oControls = mapJSON.controls;

    // Zoom
    if (oControls.zoom_controls) {
      this.enabledControls.zoom_controls = true;
    }

    // zoom_slider
    if (oControls.zoom_slider) {
      this.enabledControls.zoom_slider = true;
    }

    // zoom_to_extent
    if (oControls.zoom_to_extent) {
      this.enabledControls.zoom_to_extent = true;
    }

    // map_projection
    if (oControls.map_projection) {
      this.enabledControls.map_projection = true;
    }

    // location, localisation
    if (oControls.location_controls) {
      this.enabledControls.location_controls = true;
    }
  }
}
