import { Injectable } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';

declare const $: any;

@Injectable({
  providedIn: 'root'
})
export class StatisticsService {

  // Légende des graphiques.
  public aDataLegend: Array<any>;
  public aDataLegendTranslations: any;

  constructor(
    private translate: TranslateService
  ) { }

  /**
   * Retourne la légende.
   * @param  {Array} aData
   * @param  {string} slegend
   * @returns Array
   */
  public getDataLabels(aData: any, slegend: string) {
    const aLabels: Array<any> = [];
    for (let i = 0; i < aData.length; i++) {
      if (aLabels.indexOf(String(aData[i][slegend])) === -1) {
        aLabels.push(String(aData[i][slegend]));
      }
    }
    // Traduction de la légende.
    this.translate.get(aLabels).subscribe((translations) => {
      this.aDataLegendTranslations = translations;
    });
    return aLabels.sort();
  }

  /**
   * Retourne la traduction de la légende.
   * @returns Array
   */
  public getLegendTranslation() {
    const aLegend: Array<any> = [];
    for (let i = 0; i < this.aDataLegend.length; i++) {
      aLegend.push(this.aDataLegendTranslations[this.aDataLegend[i]]);
    }
    return aLegend;
  }

  /**
   * Formatage des données pour l'affichage suivant le type de graphique.
   * @param  {Array} aData
   * @param  {string} sChartType
   * @param  {object} oOptions
   * @returns Array
   */
  public formatDataForChart(aData: any, sChartType: string, oOptions: Object = {}) {
    const aReturn: Array<any> = [];
    // Indéxation des données par label.
    const oLabelData: Object = {};
    for (let i = 0; i < this.aDataLegend.length; i++) {
      if (typeof(oLabelData[this.aDataLegend[i]]) == 'undefined') {
        oLabelData[this.aDataLegend[i]] = [];
      }
        for (let j = 0; j < aData.length; j++) {
          oLabelData[this.aDataLegend[i]].push(aData[j][this.aDataLegend[i]]);
        }
    }
    let oGraphData: Object;
    // Formatage du tableau des données suivant le type de graphique.
    let iTotal: number;
    for (let i = 0; i < this.aDataLegend.length; i++) {
      switch (sChartType) {
        //
        case 'bar':
          oGraphData = {
            data: oLabelData[this.aDataLegend[i]],
            label: this.aDataLegendTranslations[this.aDataLegend[i]],
          };
          if (typeof(oOptions['stack']) != 'undefined') {
            oGraphData['stack'] = oOptions['stack'];
          } // Grouper les labels dans la même barre.
          aReturn.push(oGraphData);
          break;
        //
        case 'doughnut':
          // Total sur la période.
          iTotal = 0;
          for (let j = 0; j < oLabelData[this.aDataLegend[i]].length; j++) {
            iTotal += oLabelData[this.aDataLegend[i]][j];
          }
          aReturn.push(iTotal);
          break;
        }
    }
    return aReturn;
  }
}
