import { Pipe, PipeTransform } from '@angular/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import moment from 'moment';
import { environment } from 'src/environments/environment';

@Pipe({ name: 'userDate' })
export class UserDatePipe implements PipeTransform {
  constructor(
    private vitisCheck: VitisCheck,
    private auth: AuthService
  ) { }

  /**
   * Effectue la transformation
   * @param value
   * @param type
   * @returns
   */
  public transform(value: number, type?: string): string {

    let userDate: string;

    if (this.vitisCheck.isNumber(value)) {
      const oUserInfos = this.auth.getUserInfos();

      // Format de date par défaut
      const sFormatDate = oUserInfos ? oUserInfos['formatdate_id'] : 'DD-MM-YYY';

      switch (type) {
        case 'date':
          userDate = moment.unix(value).format(sFormatDate);
          break;
        case 'from_now':
          // chargement des traduction pour moment
          // peut avoir des conséquences sur le formatage des formats L, LL, LLL, LTS, ...
          moment.locale(environment.properties.language);
          userDate = moment.unix(value).fromNow();
          break;
        case 'hour':
          userDate = moment.unix(value).format('HH:mm:ss');
          break;
        case 'seconds':
          userDate = moment.utc(value * 1000).format('HH:mm:ss');
          break;
        default:
          userDate = moment.unix(value).format(sFormatDate + ' HH:mm:ss');
          break;
      }
      // Supprime l'espace entre le "T" et l'heure (ISO 8601).
      userDate = String(userDate).replace('T ', 'T');
    } else {
      userDate = String(value);
    }

    return userDate;
  }
}