// Définition des types de champs de formulaire.
export const fieldsDef: any = [
  {
    category_label: 'STUDIO_CATEGORY_BUTTON',
    fields: [
      {
        type: 'radio',
        label: 'STUDIO_CATEGORY_FIELD_RADIO',
        icon: 'icon-vitis-radio_button',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default_radio: true,
          required: true,
          radio_options: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
        },
        definition: {
          type: 'radio',
          label: '',
          options: {
            choices: [
              {
                label: 'Oui',
                value: true
              },
              {
                label: 'Non',
                value: false
              }
            ]
          }
        }
      },
      {
        type: 'checkbox',
        label: 'STUDIO_CATEGORY_FIELD_CHECKBOX',
        icon: 'icon-vitis-checkmark',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
        },
        definition: {
          type: 'checkbox'
        }
      },
      {
        type: 'button',
        label: 'STUDIO_CATEGORY_FIELD_BUTTON',
        icon: 'icon-vitis-button-icon',
        obj_attributes: {
          type: true,
          btn_label: true,
          btn_class: true,
          btn_type: true,
          btn_event: true,
          tooltip: true,
          nb_cols: true,
          dynamic_visible: true,
          dynamic_disabled: true,
        },
        definition: {
          type: 'button',
          class: 'btn-ungroup btn-group-sm',
          buttons: [
            {
              type: 'submit',
              name: 'form_submit',
              label: 'FORM_UPDATE',
              class: 'btn-primary'
            }
          ]
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_MAP',
    fields: [
      {
        type: 'map_bing',
        label: 'STUDIO_CATEGORY_FIELD_MAP_BING',
        icon: 'icon-vitis-bing',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          map_params: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
        },
        definition: {
          type: 'map_bing',
          label: '',
          style: {
            height: '250px'
          },
          map_options: {
            proj: 'EPSG:3857',
            type: 'bing',
            output_format: 'ewkt',
            center: {
              coord: [
                300000,
                6000000
              ],
              zoom: 5,
              scale: 25000000,
              extent: [
                -20026376.39,
                -20048966.1,
                20026376.39,
                20048966.1
              ]
            },
            controls: {
              // mouse_position: true,
              zoom_controls: true,
              // scale_line: true,
              map_projection: true,
              location_control: false
            },
            layers: [],
            interactions: {
              multi_geometry: false,
              full_screen: true,
              remove_all: false,
              remove: true,
              modify: true,
              draw_point: true,
              draw_line: true,
              draw_polygon: true,
              SE: true
            },
            draw_color: 'rgba(54,184,255,0.6)',
            contour_color: 'rgba(0,0,0,0.4)',
            contour_size: 2,
            circle_radius: 6,
            features: [],
            coord_accuracy: 8,
            source: {
              culture: 'fr-FR',
              key: '',
              style: 'Aerial'
            },
            map_geocoders: {
              osm: {
                title: 'OpenStreetMap',
                url: 'https://nominatim.openstreetmap.org/search?'
                  + 'format=json&addressdetails=1&limit=[limit]&extratags=1&namedetails=1&polygon_geojson=1&countrycodes=fr&q=[search]',
                data_field: 'data',
                title_field: 'display_name',
                geojson_field: 'geojson',
                summary_fields: [
                  {
                    key: 'address.country',
                    label: 'Pays'
                  },
                  {
                    key: 'address.state',
                    label: 'Région'
                  },
                  {
                    key: 'address.postcode',
                    label: 'Code postal'
                  },
                  {
                    key: 'extratags.population',
                    label: 'Population'
                  }
                ],
                id: 'osm'
              },
              national_adresse: {
                title: 'Base nationale',
                url: 'https://api-adresse.data.gouv.fr/search/?q=[search]&limit=[limit]',
                data_field: 'data.features',
                title_field: 'properties.label',
                geojson_field: 'geometry',
                summary_fields: [
                  {
                    key: 'properties.context',
                    label: 'Département'
                  },
                  {
                    key: 'properties.city',
                    label: 'Ville'
                  },
                  {
                    key: 'properties.postcode',
                    label: 'Code postal'
                  }
                ],
                id: 'national_adresse'
              }
            },
            map_default_geocoders: 'osm'
          }
        }
      },
      {
        type: 'map_osm',
        label: 'STUDIO_CATEGORY_FIELD_MAP_OSM',
        icon: 'icon-vitis-magnifying-glass2',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          map_params: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
        },
        definition: {
          type: 'map_osm',
          label: '',
          style: {
            height: '250px'
          },
          map_options: {
            proj: 'EPSG:3857',
            type: 'OSM',
            output_format: 'ewkt',
            center: {
              coord: [
                300000,
                6000000
              ],
              zoom: 5,
              scale: 25000000,
              extent: [
                -20026376.39,
                -20048966.1,
                20026376.39,
                20048966.1
              ]
            },
            controls: {
              // mouse_position: true,
              zoom_controls: true,
              // scale_line: true,
              map_projection: true,
              location_control: false
            },
            layers: [],
            interactions: {
              multi_geometry: false,
              full_screen: true,
              remove_all: false,
              remove: true,
              modify: true,
              draw_point: true,
              draw_line: true,
              draw_polygon: true,
              SE: true
            },
            draw_color: 'rgba(54,184,255,0.6)',
            contour_color: 'rgba(0,0,0,0.4)',
            contour_size: 2,
            circle_radius: 6,
            features: [],
            coord_accuracy: 8,
            map_geocoders: {
              osm: {
                title: 'OpenStreetMap',
                url: 'https://nominatim.openstreetmap.org/search?'
                  + 'format=json&addressdetails=1&limit=[limit]&extratags=1&namedetails=1&polygon_geojson=1&countrycodes=fr&q=[search]',
                data_field: 'data',
                title_field: 'display_name',
                geojson_field: 'geojson',
                summary_fields: [
                  {
                    key: 'address.country',
                    label: 'Pays'
                  },
                  {
                    key: 'address.state',
                    label: 'Région'
                  },
                  {
                    key: 'address.postcode',
                    label: 'Code postal'
                  },
                  {
                    key: 'extratags.population',
                    label: 'Population'
                  }
                ],
                id: 'osm'
              },
              national_adresse: {
                title: 'Base nationale',
                url: 'https://api-adresse.data.gouv.fr/search/?q=[search]&limit=[limit]',
                data_field: 'data.features',
                title_field: 'properties.label',
                geojson_field: 'geometry',
                summary_fields: [
                  {
                    key: 'properties.context',
                    label: 'Département'
                  },
                  {
                    key: 'properties.city',
                    label: 'Ville'
                  },
                  {
                    key: 'properties.postcode',
                    label: 'Code postal'
                  }
                ],
                id: 'national_adresse'
              }
            },
            map_default_geocoders: 'osm'
          }
        }
      },
      {
        type: 'map_vmap',
        label: 'STUDIO_CATEGORY_FIELD_MAP_VMAP',
        icon: 'icon-vitis-vitis-2',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          map_params: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
        },
        definition: {
          type: 'map_vmap',
          label: '',
          style: {
            height: '250px'
          },
          map_options: {
            proj: 'EPSG:3857',
            type: 'vmap',
            output_format: 'ewkt',
            center: {
              coord: [
                300000,
                6000000
              ],
              zoom: 5,
              scale: 25000000,
              extent: [
                -20026376.39,
                -20048966.1,
                20026376.39,
                20048966.1
              ]
            },
            controls: {
              // mouse_position: true,
              zoom_controls: true,
              // scale_line: true,
              map_projection: true,
              location_control: false
            },
            layers: [],
            interactions: {
              multi_geometry: false,
              full_screen: true,
              remove_all: false,
              remove: true,
              modify: true,
              draw_point: true,
              draw_line: true,
              draw_polygon: true,
              SE: true
            },
            draw_color: 'rgba(54,184,255,0.6)',
            contour_color: 'rgba(0,0,0,0.4)',
            contour_size: 2,
            circle_radius: 6,
            features: [],
            coord_accuracy: 8,
            tree: {},
            map_geocoders: {
              osm: {
                title: 'OpenStreetMap',
                url: 'https://nominatim.openstreetmap.org/search?'
                  + 'format=json&addressdetails=1&limit=[limit]&extratags=1&namedetails=1&polygon_geojson=1&countrycodes=fr&q=[search]',
                data_field: 'data',
                title_field: 'display_name',
                geojson_field: 'geojson',
                summary_fields: [
                  {
                    key: 'address.country',
                    label: 'Pays'
                  },
                  {
                    key: 'address.state',
                    label: 'Région'
                  },
                  {
                    key: 'address.postcode',
                    label: 'Code postal'
                  },
                  {
                    key: 'extratags.population',
                    label: 'Population'
                  }
                ],
                id: 'osm'
              },
              national_adresse: {
                title: 'Base nationale',
                url: 'https://api-adresse.data.gouv.fr/search/?q=[search]&limit=[limit]',
                data_field: 'data.features',
                title_field: 'properties.label',
                geojson_field: 'geometry',
                summary_fields: [
                  {
                    key: 'properties.context',
                    label: 'Département'
                  },
                  {
                    key: 'properties.city',
                    label: 'Ville'
                  },
                  {
                    key: 'properties.postcode',
                    label: 'Code postal'
                  }
                ],
                id: 'national_adresse'
              }
            },
            map_default_geocoders: 'osm'
          }
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_DATE',
    fields: [
      {
        type: 'date',
        label: 'STUDIO_CATEGORY_FIELD_DATE',
        icon: 'icon-vitis-calendar-check-o',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default_date: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'date'
        }
      },
      {
        type: 'datetime',
        label: 'STUDIO_CATEGORY_FIELD_DATETIME',
        icon: 'icon-vitis-calendar-check-o',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default_date: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'datetime'
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_DATA',
    fields: [
      {
        type: 'upload',
        label: 'STUDIO_CATEGORY_FIELD_UPLOAD',
        icon: 'icon-vitis-file',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          formats: true,
          required: true,
          multiple: false,
          display_only: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true
        },
        definition: {
          type: 'upload',
          label: '',
          options: {}
        }
      },
      {
        type: 'image_wsdata',
        label: 'STUDIO_CATEGORY_FIELD_IMAGE_UPLOAD',
        icon: 'icon-vitis-file',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          required: true,
          multiple: true,
          display_only: true,
          tooltip: true,
          nb_cols: true,
          dynamic_visible: true,
          dynamic_disabled: true
        },
        definition: {
          type: 'image_wsdata',
          label: '',
          options: {}
        }
      },
      {
        type: 'ui_grid',
        label: 'STUDIO_CATEGORY_FIELD_UI_GRID',
        icon: 'icon-vitis-grid',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          grid_title: true,
          ui_grid: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true
        },
        disabled: true,
        definition: {
          type: 'ui_grid',
          label: '',
          datasource: {
          }
        }
      },
      { // data_grid testé uniquement avec l'option vmap_grid: true
        type: 'data_grid',
        label: 'STUDIO_CATEGORY_FIELD_UI_GRID',
        icon: 'icon-vitis-bo_grid',
        obj_attributes: {
          type: true,
          label: true,
          vmap_grid: false,
          tooltip: true,
          nb_cols: true,
          dynamic_visible: true
        },
        definition: {
          type: 'data_grid',
          label: '',
          webservice_grid: {
            params: {
            }
          },
          options: {
          }
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_INTERFACE',
    fields: [
      {
        type: 'hr',
        label: 'STUDIO_CATEGORY_FIELD_HR',
        icon: 'icon-vitis-lines',
        obj_attributes: {
          type: true,
          dynamic_visible: true
        },
        definition: {
          type: 'hr'
        }
      },
      {
        type: 'title',
        label: 'STUDIO_CATEGORY_FIELD_TITLE',
        icon: 'icon-vitis-label_fill',
        obj_attributes: {
          type: true,
          label: true,
          tooltip: true,
          dynamic_visible: true
        },
        definition: {
          type: 'title'
        }
      },
      {
        type: 'subtitle',
        label: 'STUDIO_CATEGORY_FIELD_SUBTITLE',
        icon: 'icon-vitis-subtitle',
        obj_attributes: {
          type: true,
          label: true,
          tooltip: true,
          dynamic_visible: true
        },
        definition: {
          type: 'subtitle'
        }
      },
      {
        type: 'text_body',
        label: 'STUDIO_CATEGORY_FIELD_TEXT_BODY',
        icon: 'icon-vitis-text',
        obj_attributes: {
          type: true,
          label: true,
          nb_cols: true,
          nb_rows: true,
          dynamic_visible: true
        },
        definition: {
          type: 'text_body'
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_LABEL',
    fields: [
      {
        type: 'label',
        label: 'STUDIO_CATEGORY_FIELD_LABEL',
        icon: 'icon-vitis-label',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
        },
        definition: {
          type: 'label',
          label: ''
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_LIST',
    fields: [
      {
        type: 'list',
        label: 'STUDIO_CATEGORY_FIELD_LIST',
        icon: 'icon-vitis-list',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          datasource: true,
          size: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'list',
          datasource: {}
        }
      },
      {
        type: 'select',
        label: 'STUDIO_CATEGORY_FIELD_SELECT',
        icon: 'icon-vitis-list_fill',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          datasource: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'select',
          datasource: {}
        }
      },
      {
        type: 'searchable_select',
        label: 'STUDIO_CATEGORY_FIELD_SEARCHABLE_SELECT',
        icon: 'icon-vitis-search',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          datasource: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'searchable_select',
          datasource: {}
        }
      },
      {
        type: 'double_select',
        label: 'STUDIO_CATEGORY_FIELD_DOUBLE_SELECT',
        icon: 'icon-vitis-double_select',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          required: true,
          datasource: true,
          nb_rows: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
        },
        definition: {
          type: 'double_select',
          datasource: {}
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_NUMBER',
    fields: [
      {
        type: 'integer',
        label: 'STUDIO_CATEGORY_FIELD_INTEGER',
        icon: 'icon-vitis-integer',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default_number: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'integer'
        }
      },
      {
        type: 'float',
        label: 'STUDIO_CATEGORY_FIELD_FLOAT',
        icon: 'icon-vitis-floating',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default_number: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'float'
        }
      },
      {
        type: 'slider',
        label: 'STUDIO_CATEGORY_FIELD_SLIDER',
        icon: 'icon-vitis-merged_brackeys',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          min: true,
          max: true,
          default_number: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
        },
        definition: {
          type: 'slider',
          label: 'radio',
          options: {
            min: 0,
            max: 100,
            precision: 1,
            step: 1
          }
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_TEXT',
    fields: [
      {
        type: 'text',
        label: 'STUDIO_CATEGORY_FIELD_TEXT',
        icon: 'icon-vitis-text',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          pattern: true,
          default: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'text'
        }
      },
      {
        type: 'textarea',
        label: 'STUDIO_CATEGORY_FIELD_TEXTAREA',
        icon: 'icon-vitis-multilines',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          nb_rows: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'textarea'
        }
      },
      {
        type: 'email',
        label: 'STUDIO_CATEGORY_FIELD_EMAIL',
        icon: 'icon-vitis-mail',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          pattern: true,
          default: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'email',
          pattern: '^(([a-zA-Z]|[0-9])|([-]|[_]|[.]))+[@](([a-zA-Z]|[0-9])|([-]|[_]|[.])){2,63}[.](([a-zA-Z0-9]){2,63})+$'
        }
      },
      {
        type: 'password',
        label: 'STUDIO_CATEGORY_FIELD_PASSWORD',
        icon: 'icon-vitis-lock',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          pattern: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'password'
        }
      },
      {
        type: 'url',
        label: 'STUDIO_CATEGORY_FIELD_URL',
        icon: 'icon-vitis-world',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
          dynamic_disabled: true,
          dynamic_event_function: true,
        },
        definition: {
          type: 'url'
        }
      },
      {
        type: 'tinymce',
        label: 'STUDIO_CATEGORY_FIELD_TINYMCE',
        icon: 'icon-vitis-tinyMCE2',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          required: true,
          tinymce_height: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
        },
        definition: {
          type: 'tinymce',
          label: '',
          options: {}
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_DISPLAY',
    fields: [
      {
        type: 'linkurl',
        label: 'STUDIO_CATEGORY_FIELD_LINK_URL',
        icon: 'icon-vitis-world',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          text: true,
          target: true,
          default: true,
          tooltip: true,
          nb_cols: true,
          dynamic_visible: true,
          dynamic_disabled: true
        },
        definition: {
          type: 'linkurl'
        }
      },
      {
        type: 'imageurl',
        label: 'STUDIO_CATEGORY_FIELD_IMAGE_URL',
        icon: 'icon-vitis-imageurl',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          default: true,
          tooltip: true,
          nb_cols: true,
          dynamic_visible: true
        },
        definition: {
          type: 'imageurl'
        }
      },
      {
        type: 'angularurl',
        label: 'STUDIO_CATEGORY_FIELD_ANGULAR_URL',
        icon: 'icon-vitis-world',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          text: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true
        },
        definition: {
          type: 'angularurl'
        }
      }
    ]
  },
  {
    category_label: 'STUDIO_CATEGORY_OTHER',
    fields: [
      {
        type: 'hidden',
        label: 'STUDIO_CATEGORY_FIELD_HIDDEN',
        icon: 'icon-vitis-eye-blocked',
        obj_attributes: {
          type: true,
          name: true,
          default: true,
          dynamic_value: true
        },
        definition: {
          type: 'hidden'
        }
      },
      {
        type: 'color_picker',
        label: 'STUDIO_CATEGORY_FIELD_COLOR_PICKER',
        icon: 'icon-vitis-paint',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          format_color_picker: true,
          default: true,
          required: true,
          tooltip: true,
          nb_cols: true,
          dynamic_value: true,
          dynamic_visible: true,
        },
        definition: {
          type: 'color_picker',
          label: ''
        }
      },
      {
        type: 'dessin',
        label: 'STUDIO_CATEGORY_FIELD_DESSIN',
        icon: 'icon-vitis-dessin',
        obj_attributes: {
          type: true,
          label: true,
          name: true,
          dessin_button_label: true,
          editable_dessin: true,
          required: true,
          tooltip: true,
          dynamic_visible: true,
          dynamic_disabled: true,
        },
        definition: {
          type: 'dessin',
          label: ''
        }
      }
    ]
  }
];