import { Component, OnInit, AfterViewInit, OnDestroy } from '@angular/core';
import { StudioService } from '../../services/studio.service';
import { VitisClone, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

declare const $: any;

@Component({
  selector: 'app-configuration',
  templateUrl: './configuration.component.html',
  styleUrls: ['./configuration.component.scss']
})
export class StudioConfigurationComponent implements OnInit, AfterViewInit, OnDestroy {

  public sSelectedFormType: string;
  public sSelectedFormName: string;
  public aFormTypes = ['Default', 'Published', 'Perso'];
  public aFormNames = [];
  public haveFieldListRessource: boolean = false;

  private resizeInterval_: any;

  constructor(
    private studioService: StudioService,
    private vitisClone: VitisClone,
    private vitisCheck: VitisCheck
  ) { }

  ngOnInit() {

    // Changement de type de formulaire (default, custom, published...)
    this.studioService.getStudioFormType().subscribe((sFormType: string) => {
      this.sSelectedFormType = this.vitisClone.clone(sFormType);
    });

    // Changement de nom de formulaire (display, insert, update...)
    this.studioService.getStudioFormName().subscribe((sFormName: string) => {
      this.sSelectedFormName = this.vitisClone.clone(sFormName);
    });

    // Options du studio
    this.studioService.getStudioOptions().subscribe((oStudioOptions) => {

      // Types non standard
      if (this.vitisCheck.isDefAndNotNull(oStudioOptions.available_form_types)) {
        this.aFormTypes = oStudioOptions.available_form_types;
      }

      // Nom non standard
      if (this.vitisCheck.isDefAndNotNull(oStudioOptions.available_form_names)) {
        this.aFormNames = oStudioOptions.available_form_names;
      }

      if (this.vitisCheck.isDefAndNotNull(oStudioOptions.field_list_ressource)
        && this.vitisCheck.isDefAndNotNull(oStudioOptions.field_list_ressource_result_attr)) {
        this.haveFieldListRessource = true;
      }
    });
  }

  ngAfterViewInit() {

    // Calcule la hauteur de la section "attributs" en fonction de la section "configuration" toutes les secondes
    this.resizeInterval_ = setInterval(() => {
      const iConfigurationSectionHeight = $('.studio-configuration-section').height();
      (document.querySelector('.studio-attributs-section') as HTMLElement).style.height =
        'calc(100% - ' + iConfigurationSectionHeight + 'px - 5px)';
    }, 1000);
  }

  ngOnDestroy() {
    clearInterval(this.resizeInterval_);
  }

  /**
   * Change de type de formulaire
   * @param sFormType
   */
  public changeFormType(sFormType: string) {
    this.studioService.setStudioFormType(sFormType);

    // Sort du mode édition
    this.studioService.setStudioEditedField(null);

    // On recharge la définition du formulaire donc il n'est en modification
    this.studioService.setStudioIsBeingModified(false);
    // Lance le rechargement du formulaire
    setTimeout(() => {
      this.studioService.ajaxLoadStudioForm();
    });
  }

  /**
   * Change de nom de formulaire
   * @param sFormName
   */
  public changeFormName(sFormName: string) {
    this.studioService.setStudioFormName(sFormName);
  }

  /**
   * Publie le formulaire personnalisé
   */
  public publishCustomForm() {
    // Save custom form
    this.studioService.ajaxSaveStudioForm().then(() => {
      // Publish custom form
      this.studioService.ajaxSaveStudioForm('Published').then((oForm: any) => {
        this.studioService.reloadStudioForm(oForm);
      });
    });
  }

  /**
   * Sauvegarde le brouillon
   */
  public saveCustomForm() {
    // Save custom form
    this.studioService.ajaxSaveStudioForm().then((oForm: any) => {
      this.studioService.reloadStudioForm(oForm);
    });
  }

  /**
   * Remplace le brouillon par le formulaire par défaut
   */
  public replaceCustomFormByDefaultForm() {
    this.studioService.ajaxLoadStudioForm('Default');
  }

  /**
   * Remplace le brouillon par le formulaire publié
   */
  public replaceCustomFormByPublishedForm() {
    this.studioService.ajaxLoadStudioForm('Published');
  }

  /**
   * Remplace le brouillon par le formulaire publié
   */
  public quicklyUpdateForm() {
    this.studioService.openListFieldsCanBeAdd();
  }
}
