import { Component, OnInit, Input, OnChanges } from '@angular/core';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';
import { search as jmespathSearch } from 'jmespath';

@Component({
  selector: 'app-data-source-external-api',
  templateUrl: './data-source-external-api.component.html',
  styleUrls: ['./data-source-external-api.component.scss']
})
export class StudioDataSourceExternalApiComponent implements OnInit, OnChanges {

  @Input() public sSourceManagerMode: string;
  @Input() public oSelectedDataSource: any;

  public oExternalApiResult: any = {};
  public aExternalApiResultKeys: any = [];
  public aExternalApiResultValues: any = [];

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService
  ) { }

  ngOnInit() {
  }

  ngOnChanges() {
    if (this.oSelectedDataSource.dataType == 'externalApi') {
      switch (this.sSourceManagerMode) {
        case 'overview':
        case 'edition':
          break;
        case 'creation':
          // Structure de la source.
          this.oSelectedDataSource.type = 'external_API';
          break;
      }
    }
  }
  /**
   * Chargement des données de l'Api externe.
   * @returns void
   */
  public testExternalApiResult = function () {
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: this.oSelectedDataSource.api_url
    }).then((resp: any) => {
      if (resp.status === 200) {
        if (this.vitisCheck.isDefAndNotNull(resp.data)) {

          // Résultat de la requête.
          this.oExternalApiResult = resp.data;

          // Clés et valeurs.
          this.aExternalApiResultKeys = jmespathSearch(this.oExternalApiResult, this.oSelectedDataSource.api_key);
          this.aExternalApiResultValues = jmespathSearch(this.oExternalApiResult, this.oSelectedDataSource.api_value);
        }
      }
    });
  };
}
