import { Component, OnInit, Input, OnChanges } from '@angular/core';
import { environment } from 'src/environments/environment';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-data-source-web-service',
  templateUrl: './data-source-web-service.component.html',
  styleUrls: ['./data-source-web-service.component.scss']
})
export class StudioDataSourceWebServiceComponent implements OnInit, OnChanges {

  @Input() public sSourceManagerMode: string;
  @Input() public oSelectedDataSource: any;

  public aWebServices: any = [];
  public aWebServiceResources: any = [];
  public aWebServiceAttributes: any = [];
  public aWebServiceData: any = [];

  constructor(
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private translate: TranslateService
  ) { }

  ngOnInit() {
  }

  ngOnChanges() {
    if (this.oSelectedDataSource.dataType == 'webService') {
      switch (this.sSourceManagerMode) {
        case 'overview':
        case 'edition':
          this.loadWebServices();
          this.loadWebServiceResources(this.oSelectedDataSource.webservice.name);
          break;
        case 'creation':
          // Structure de la source.
          this.oSelectedDataSource.webservice = {
            name: ''
          };
          this.oSelectedDataSource.ressource = {
            name: ''
          };

          // Chargement de la liste des service web.
          this.loadWebServices();
          break;
      }
    }
  }

  /**
   * Chargement de la liste des web services disponibles.
   * @returns void
   */
  public loadWebServices = function () {

    // Vide l'aperçu du web service.
    this.aWebServiceAttributes.length = 0;
    this.aWebServiceData.length = 0;
    this.aWebServices.length = 0;

    // Lancement de la requête
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: environment.properties.app_api_url + '/vitis/webservices',
      params: {
        order_by: 'name'
      },
      httpMethodOverride: true,
    }).then((resp: any) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {
            this.aWebServices = resp.data.data;
          }
        }
      }
    });
  };

  /**
   * Chargement de la liste des ressources d'un web services.
   * @param  {string} sWebServiceName
   * @returns void
   */
  public loadWebServiceResources = function (sWebServiceName: string) {

    // Vide l'aperçu du web service.
    this.aWebServiceAttributes.length = 0;
    this.aWebServiceData.length = 0;
    this.aWebServiceResources.length = 0;

    // Lancement de la requête
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: environment.properties.app_api_url + '/vitis/webservices/' + sWebServiceName + '/ressources',
      params: {
        attributs: 'name',
        order_by: 'name'
      },
      httpMethodOverride: true,
    }).then((resp: any) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {
            this.aWebServiceResources = resp.data.data;
          }
        }
      }
    });
  };

  /**
   * Mise à jour des paramètres obligatoires.
   * @returns void
   */
  public updateDataSourceParams() {
    this.oSelectedDataSource.ressource_id = this.oSelectedDataSource.webservice.name + '/' + this.oSelectedDataSource.ressource.name;
  }

  /**
   * Teste un web service.
   * @param  {string} sWebServiceName
   * @param  {string} sResourceName
   * @returns void
   */
  public testWebService = function (sWebServiceName: string, sResourceName: string) {

    // Vide les données.
    this.aWebServiceAttributes.length = 0;
    this.aWebServiceData.length = 0;

    // Lancement de la requête
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: environment.properties.app_api_url + '/' + sWebServiceName + '/' + sResourceName,
      params: {
        limit: 100
      },
      httpMethodOverride: true,
    }).then((resp: any) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {

            // Nom des colonnes du web service (aperçu).
            this.aWebServiceData = resp.data.data;
            if (this.vitisCheck.isDefAndNotNull(this.aWebServiceData)) {
              const aWebServiceAttributes = Object.keys(this.aWebServiceData[0]);
              for (let i = 0; i < aWebServiceAttributes.length; i++) {

                // Pas de données tableau ou objet.
                if (!this.vitisCheck.isArray(this.aWebServiceData[0][aWebServiceAttributes[i]]) &&
                  !this.vitisCheck.isObject(this.aWebServiceData[0][aWebServiceAttributes[i]])) {
                  this.aWebServiceAttributes.push(aWebServiceAttributes[i]);
                }
              }
            }
          }
        }
      }
    });
  };
}
