import { Component, OnInit, Input, ViewChild } from '@angular/core';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { WorkspacelistComponent } from '../workspacelist.component';
import { FormreaderComponent } from '../../formreader/formreader.component';
import { WorkspaceListOptions } from '../workspacelist.model';
import moment from 'moment';

declare var $: any;

@Component({
  selector: 'app-formfilter',
  templateUrl: './formfilter.component.html',
  styleUrls: ['./formfilter.component.scss']
})
export class FormfilterComponent implements OnInit {

  @ViewChild(FormreaderComponent, { static: false }) oFormReader: FormreaderComponent;

  @Input() oWorkspaceList: WorkspacelistComponent;
  @Input() oFilterTypes: any;
  @Input() oResults: any;
  @Input() oFilter: any;
  @Input() oMode: any;

  public oForm: any;
  public bFormOpen: boolean = true;
  public modeOptions: WorkspaceListOptions;

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest
  ) { }

  ngOnInit() {

    // Options
    if (this.vitisCheck.isDefAndNotNull(this.oMode)) {
      if (this.vitisCheck.isDefAndNotNull(this.oMode.options)) {
        this.modeOptions = this.oMode.options;
      }
    }

    // Récupère le formulaire
    if (this.vitisCheck.isDefAndNotNull(this.modeOptions)) {
      if (this.vitisCheck.isDefAndNotNull(this.modeOptions.form_path)) {
        this.getJSONForm_(this.oMode).then((oForm) => {
          if (this.vitisCheck.isDefAndNotNull(oForm.search)) {

            // Mémorise le formulaire
            this.oForm = oForm;

            // Affiche le formulaire
            this.initForm(oForm);
          }
        });
      }
    }

    // Affiche le formulaire si des valeurs sont passées
    if (this.vitisCheck.isDefAndNotNull(this.oFilter)) {
      if (this.vitisCheck.isArray(this.oFilter.nodes)) {
        if (this.oFilter.nodes.length > 0) {
          this.bFormOpen = true;
        }
      }
    }
  }

  /**
   * Initialise le formulaire
   * @param  {any} oForm
   */
  public initForm(oForm: any) {

    if (this.vitisCheck.isDefAndNotNull(oForm)) {
      if (this.vitisCheck.isDefAndNotNull(oForm.search)) {

        // Events
        oForm.search.eventContainer = this;
        oForm.search.event = 'applyFilter()';

        // Ajout d'un bouton invisible
        oForm.search.rows.push({
          'fields': [{
            'type': 'button',
            'nb_cols': 12,
            'buttons': [{
              'type': 'submit',
              'name': 'form_submit',
              'label': 'FORM_SEARCH_BUTTON',
              'class': 'd-none'
            }
            ]
          }]
        });

        // Lance le formulaire dans FormReader
        this.oFormReader.initForm(oForm, 'search', this.getFromValuesFromNodes_());
      }
    }
  }

  /**
   * Applique le filtre
   */
  public applyFilter(): void {

    const oValues = this.oFormReader.getFormValues();
    const aNodes = this.getNodesFromValues(oValues);

    if (this.vitisCheck.isDefAndNotNull(aNodes)) {
      this.oFilter.nodes = aNodes;
      this.oWorkspaceList.applyFilter();
    }
  }

  /**
   * Réinitialise le filtre
   */
  public resetFilter(): void {
    this.oFormReader.clearValues();
    this.applyFilter();
  }

  /**
   * Récupère la définition JSON d'un formulaire
   * @param  {any} oMode
   * @returns Promise
   */
  private getJSONForm_(oMode: any): Promise<any> {
    return new Promise((resolve, reject) => {

      const url = oMode.options.form_path;

      // Récupère le formulaire
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url
      }).then((resp) => {
        if (this.vitisCheck.isDefAndNotNull(resp.data)) {
          resolve(resp.data);
        } else {
          console.error('Error getting JSON form');
          reject(false);
        }
      });
    });
  }

  /**
   * Retourne les valeurs du formulaire d'après oFilter.nodes
   * @returns any
   */
  private getFromValuesFromNodes_(): any {

    const oValues: any = {};

    if (this.vitisCheck.isDefAndNotNull(this.oFilter)) {
      if (this.vitisCheck.isArray(this.oFilter.nodes)) {
        for (let i = 0; i < this.oFilter.nodes.length; i++) {
          const node = this.oFilter.nodes[i];
          if (this.vitisCheck.isDefAndNotNull(node.value)) {
            if (this.vitisCheck.isDefAndNotNull(node.column)) {
              if (this.vitisCheck.isDefAndNotNull(node.column.field)) {
                oValues[node.column.field] = node.value;
              }
            }
          }
        }
      }
    }

    return oValues;
  }

  /**
   * Recupère les noeuds depuis les valeurs
   * @param  {any} oValues
   * @returns Array
   */
  private getNodesFromValues(oValues: any): Array<any> {

    let aNodes = [];

    for (const key in oValues) {
      if (this.vitisCheck.isDefAndNotNull(oValues[key]) &&
        oValues[key] !== '') {

        const sValue = oValues[key];
        const fieldType = this.getType_(key);
        let column = this.getColumnByName_(key);

        // Pour les filtres de type formulaire on autorise
        // le fait que la colonne ne soit pas présente
        if (!this.vitisCheck.isDefAndNotNull(column)) {
          column = {
            field: key
          };
        }

        switch (fieldType) {
          case 'date':
            aNodes = this.addDateToFilter_(column, sValue, aNodes);
            break;
          case 'select':
            aNodes.push({
              column,
              compareOperator: '=',
              value: sValue
            });
            break;
          default:
            aNodes.push({
              column,
              compareOperator: '~',
              value: sValue
            });
            break;
        }
      }
    }

    return aNodes;
  }

  /**
   * Recherche dans les colonnes du mode celle dont le field correspond à sColumnName
   * @param  {String} sColumnName
   * @returns any
   */
  private getColumnByName_(sColumnName: String): any {

    if (this.modeOptions) {
      if (this.modeOptions.columns) {

        // Pour chaque colonne
        for (let i = 0; i < this.modeOptions.columns.length; i++) {
          const column = this.modeOptions.columns[i];

          if (column.field.trim() === sColumnName.trim()) {
            return column;
          }
        }
      }
    }
  }

  /**
   * Retourne le type de champ
   * @param sFieldName
   */
  private getType_(sFieldName: any): string {
    const oField = this.oFormReader.getFormField(sFieldName);
    if (oField && oField.type) {
      return oField.type;
    }
  }

  /**
   * Ajoute à aNodes le filtre pour la date et retourne aNodes
   * @param column
   * @param sValue
   * @param aNodes
   */
  private addDateToFilter_(column: any, sValue: any, aNodes: any): Array<any> {

    if (this.vitisCheck.isDefAndNotNull(sValue)) {

      const year = moment(sValue).year();
      const month = moment(sValue).month();
      const day = moment(sValue).date();

      if (this.vitisCheck.isNumber(year) &&
        this.vitisCheck.isNumber(month) &&
        this.vitisCheck.isNumber(day)) {

        // Date du jour
        const dayDate = moment().year(year).month(month).date(day).hour(0).minute(0).second(0);

        // Date de fin du jour
        const nextDayDate = moment().year(year).month(month).date(day).hour(23).minute(59).second(59);

        aNodes.push({
          column,
          compareOperator: '>',
          value: dayDate.format()
        });
        aNodes.push({
          column,
          compareOperator: '<',
          value: nextDayDate.format()
        });
      }
    }

    return aNodes;
  }
}
