import { Component, OnInit, Input } from '@angular/core';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { WorkspacelistComponent } from '../workspacelist.component';
import { Status } from './model/status.model';
import { WorkspaceListOptions } from '../workspacelist.model';

declare var $: any;

@Component({
  selector: 'app-statusfilter',
  templateUrl: './statusfilter.component.html',
  styleUrls: ['./statusfilter.component.scss']
})
export class StatusfilterComponent implements OnInit {

  @Input() oWorkspaceList: WorkspacelistComponent;
  @Input() oFilterTypes: any;
  @Input() oResults: any;
  @Input() oFilter: any;
  @Input() oMode: any;

  public oSearchColumn: any;
  public sSearchColumn: string;
  public sCounterColumn: string;
  private aParams: any = {};
  public aStatus: Array<Status> = [];

  public modeOptions: WorkspaceListOptions;
  public aCompareOperators: Array<string> = ['~', '=', '>', '<'];

  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest
  ) { }

  ngOnInit() {

    // Options
    if (this.vitisCheck.isDefAndNotNull(this.oMode)) {
      if (this.vitisCheck.isDefAndNotNull(this.oMode.options)) {
        this.modeOptions = this.oMode.options;
      }
    }

    // Si config n'existe pas arrete l'initialisation
    if (!this.modeOptions) {
      return;
    }

    // Colonne de l'état
    this.sSearchColumn = this.modeOptions.search.statusFilter.searchColumn;
    // Colonne pour le nombre d'objet par état
    this.sCounterColumn = this.modeOptions.search.statusFilter.request.counter_key;
    // Paramètre pour la requête
    this.aParams = this.modeOptions.search.statusFilter.request.params;

    // Liste des status
    this.modeOptions.search.statusFilter.values.forEach((status) => {
      this.aStatus.push(Object.assign(new Status(), status));
      this.aStatus[this.aStatus.length - 1].animate();
    });

    if (this.oWorkspaceList &&
      this.oWorkspaceList.onRefresh) {
      this.oWorkspaceList.onRefresh.subscribe(() => {
        this.getResourceData_(this.aParams).then((oResponse: any) => {
          // Remise à zéro des états des demandes.
          for (let i = 0; i < this.aStatus.length; i++) {
            this.aStatus[i].iCount = 0;
          }
          //
          oResponse.forEach(filter => {
            this.aStatus.forEach((status: Status) => {
              if (status.value == filter.filter) {
                status.iCount = filter[this.sCounterColumn];
              }
            });
          });
        });
      });
    }

  }

  /**
   * Fonction lancée lors du clique sur un état des demandes
   * @param  {string} searchValue
   * @returns void
   */
  public filterStatus(sSearchValue: string): void {
    // Pour chaque colonne
    this.modeOptions.columns.forEach(column => {
      if (column.field === this.sSearchColumn) {
        this.addFilterNode(column, '=', sSearchValue);
      }
    });
  }

  /**
   * Crée le noeud de filtre et l'ajoute à this.oFilter.nodes
   * @param  {any} oSearchColumn
   * @param  {String} sCompareOperator
   * @param  {String} sValue
   * @returns void
   */
  public addFilterNode(oSearchColumn: any, sCompareOperator: String, sValue: String): void {

    if (this.vitisCheck.isDefAndNotNull(oSearchColumn)
      && this.vitisCheck.isString(sCompareOperator)
      && this.vitisCheck.isString(sValue)) {

      const oFilterNode = {
        column: oSearchColumn,
        compareOperator: sCompareOperator,
        value: sValue,
        displayValue: sValue
      };

      // Gesiton tableau de valeur avec libellé
      if (this.vitisCheck.isArray(oSearchColumn.searchSuggestions)) {
        for (let i = 0; i < oSearchColumn.searchSuggestions.length; i++) {
          const suggestion = oSearchColumn.searchSuggestions[i];
          if (this.vitisCheck.isDefAndNotNull(suggestion.label) &&
            this.vitisCheck.isDefAndNotNull(suggestion.value)) {
            if (suggestion.value === sValue) {
              oFilterNode.displayValue = suggestion.label;
            }
          }
        }
      }

      let nodeNotExist = true;
      for (let i = 0; i < this.oFilter.nodes.length; i++) {
        if (this.oFilter.nodes[i].column.field === this.sSearchColumn) {
          this.oFilter.nodes[i] = oFilterNode;
          nodeNotExist = false;
        }
      }

      if (nodeNotExist === true) {
        this.oFilter.nodes.push(oFilterNode);
      }

      // Applique le filtre à la grille
      this.oWorkspaceList.applyFilter();
    }
  }

  /**
   * Récupère les donéées au format DataTables
   * @param  {any} params
   * @returns Promise
   */
  private getResourceData_(params: any): Promise<any> {
    return new Promise((resolve, reject) => {

      let sRessource = this.modeOptions.ressource;

      if (this.modeOptions.search.statusFilter.request.ressource) {
        sRessource = this.modeOptions.search.statusFilter.request.ressource;
      }

      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: environment.properties.app_api_url + '/' + sRessource,
        ajaxLoader: false,
        params: params,
        success: (response: any) => {
          if (this.vitisCheck.isDefAndNotNull(response.data)
            && this.vitisCheck.isDefAndNotNull(response.data.status)
            && response.data.status == 1
            && this.vitisCheck.isDefAndNotNull(response.data.data)) {
            resolve(response.data.data);
          }
        }
      });
    });
  }

}
