import { Component, OnInit, Input, AfterViewInit, ViewChild, OnDestroy } from '@angular/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

declare var $: any;

@Component({
  selector: 'app-wklist-params',
  templateUrl: './params.component.html',
  styleUrls: ['./params.component.scss']
})
export class WklistParamsComponent implements OnInit, AfterViewInit, OnDestroy {

  @Input() params: any;

  @ViewChild('popover', { static: false }) popoverElem: any;

  public aDisplayParams: Array<string> = [];

  constructor(
    private vitisCheck: VitisCheck
  ) { }

  ngOnInit() {
    // Initialise le contenu
    this.initContent();
  }

  ngAfterViewInit() {
    // initialise la popover
    if (this.vitisCheck.isDefAndNotNull(this.popoverElem)) {
      this.initPopover(this.popoverElem.nativeElement);
    }
  }

  ngOnDestroy() {
    // détruit la popover
    if (this.vitisCheck.isDefAndNotNull(this.popoverElem)) {
      this.destroyPopover(this.popoverElem.nativeElement);
    }
  }

  /**
   * Initialise le contenu
   */
  public initContent() {
    try {
      const oParams = JSON.parse(this.params);
      for (const key in oParams) {
        if (oParams.hasOwnProperty(key)) {
          this.aDisplayParams.push(`<b>${key}</b> = ${oParams[key]}`);
        }
      }
    } catch (error) {
      this.aDisplayParams = this.params.split('|');
    }
  }

  /**
   * Initialise la popover
   * @param elem élément dom
   */
  public initPopover(elem: any) {
    const popoverContainer: any = $(elem);
    popoverContainer.popover({
      content: this.aDisplayParams.join('<br>'),
      trigger: 'hover',
      html: true
    });
  }

  /**
   * Supprime la popover
   * @param elem élément dom
   */
  public destroyPopover(elem: any) {
    const popoverContainer: any = $(elem);
    popoverContainer.popover('hide');
    popoverContainer.popover('disable');
  }
}
