import { EventEmitter } from '@angular/core';
import { NgbModalOptions } from '@ng-bootstrap/ng-bootstrap';

/**
 * Options boutons
 */
export class WorkspaceRelationListButton {
  event?: any;
  type: string; // button / group / modal
  label: string;
  sEvent?: string;
  name?: string;
  class?: string;
  icon?: string;
  buttons?: Array<WorkspaceRelationListButton>;
  modal_component?: string;
  modal_options?: NgbModalOptions; // https://ng-bootstrap.github.io/#/components/modal/api#NgbModalOptions
  refreshGridOnClose?: boolean; // recharge la grille lors de la fermeture (default: false)
  eventOnClose?: string; // Événement à lancer à la fermeture
  modal_need_selected_row?: boolean; // Vérifie qu'au moins 1 ligne de la grille est sélectionné (default: false)
  modal_label_no_row_error?: string; // Message d'erreur à afficher si aucune ligne n'a été sélectionné alors que c'était néccessaire
  privilege?: string; // Affichage du bouton en fonction d'un privilère
  display?: boolean; // Affichage du bouton (string = privilège obligatoire).
}

/**
 * Options des colonnes WorkspaceRelationlist
 */
export class WorkspaceRelationListColumn {
  title: string; // Titre à afficher
  field: string; // Nom de la colonne
  align?: string; // Alignement (right, left, center)
  sortable?: boolean; // Ordre modifiable
  translateTitle?: string; // Titre à traduire
  template?: string; // Template à utiliser
  resizeable?: boolean; // Non disponible
  width?: string; // Non disponible
  visible?: boolean; // Colonne visible (mais utilisable) vaut true par défaut
  searchable?: boolean; // Permet de désectiver le filtre personnalisé sur cette colonne
  searchType?: string; // Type de données (permet de mieux gérer les filtres)
  searchColumn?: string; // permet de filtrer sur un autre champs de la base
  searchWebService?: WorkspaceRelationListSearchWSColumn; // permet de définir une datasource pour remplir les suggestions
  searchSuggestions?: any; // Suggestions (false ou tableau)
  privilege?: string; // Affichage de la colonne en fonction d'un privilège
}

/**
 * Option pour utiliser un webservice comme suggestion d'un champs
 */
export class WorkspaceRelationListSearchWSColumn {
  label?: string; // si non défini la valeur sera utilisé
  value: string;
  resource: string; // module/objet
  params?: any; // sera encodé en json et utilisé pour la requête GET
}

/**
 * Option value statusFilter
 */
export class WorkspaceRelationListStatusFilterValue {
  label: string;
  value: string;
  textStyle?: string; // text-primary / text-danger / text-success / text-dark / text-info / text-secondary / text-warning
  showCounter?: boolean;
  annimation?: boolean;
}

export class WorkspaceRelationSourceListOptions {
  ressource: string; // (ex: gtf/orders) ressource source API à attaquer
  id_field: string; // (ex: order_id) colonne id de l'API
  id_ressource_field: string; // (ex: order_id) colonne id de la ressource mère de l'API
  titleModalInsert: string; // Titre de la modale qui affiche data grid source
  insert?: boolean; // affichage du bouton ajout
  columns: Array<WorkspaceRelationListColumn>;
}

/**
 * Options pour composant de type WorkspaceRelationList
 */
export class WorkspaceRelationListOptions {
  module?: string; // (ex: gtf) permet de charger les composant du module
  source?: WorkspaceRelationSourceListOptions; // (ex: gtf/orders) ressource source API à attaquer
  ressource?: string; // (ex: gtf/orders) ressource relation API à attaquer
  ressource_source?: string; // nom de la ressource source tel qu'afficher dans l'url
  fields_source: string[]; // Champs de la source afficher dans la ressource => Sert pour l'update des champs editable
  id_field?: string; // (ex: order_id) colonne id de l'API
  parent_id_field?: string; // cas de l'utilisation de workspaceRelationlist dans une section, permet soit de faire un remplacement dans l'URL avec :id_parent soit de faire un filtre sur l'élément parent
  disable_filter_parent?: boolean; // Permet de désactiver le filtrage via les noeud sur le parent automatiquement
  display?: boolean | string; // affichage du bouton fiche descriptive (string = privilège obligatoire)
  update?: boolean | string; // affichage du bouton mise à jour (string = privilège obligatoire)
  insert?: boolean | string; // affichage du bouton ajout (string = privilège obligatoire)
  edit?: boolean; // affichage du bouton save si on possède des champs editable
  delete?: boolean | string; // affichage du bouton suppression (string = privilège obligatoire)
  refresh?: boolean; // affichage du bouton rafraichissement (true par défaut)
  autorefresh?: WorkspaceRelationListAutorefreshOptions; // Options pour rafraichissement automatique
  form_path?: string; // Chemin vers le fichier formulaire
  js_path?: string; // Chemin vers le fichier formulaire
  order_by?: Array<any>; // [[numbero de la colonne], [asc / desc]]
  search?: {
    custom?: boolean
    form?: boolean
    default?: string // custom/form
    statusFilter?: {
      request?: {
        ressource?: string // ressource à utiliser pour les valeurs de filtres
        counter_key?: string // clé à utiliser pour la répartition
        params?: object // paramètre pour la requête de récupération des données
      }
      searchColumn: string
      values: Array<WorkspaceRelationListStatusFilterValue>
    }
  };
  columns: Array<WorkspaceRelationListColumn>;
  buttons: Array<WorkspaceRelationListButton>;
  edition_component?: string; // Utilisation d'un composant autre que simpleform pour l'insertion et l'edition
  height?: number; // Hauteur de la grille
  singleSelect?: boolean; // Désactivation de la selection multiple
  max_rows?: number; // Nombre maximum d'enregistrements à afficher
}

/**
 * Options pour rafraichissement automatique
 */
export class WorkspaceRelationListAutorefreshOptions {
  interval: number; // Intervalle de rafraichissement
  compare_columns?: Array<string>; // Colonnes à comparer
}

/**
 * Options pour workspaceRelationlist embarqué
 */
export class WorkspaceRelationListEmbeddedOptions {
  mode: {
    name: string;
    label: string;
    route: string;
    type: string;
    privilege: string;
    options: WorkspaceRelationListOptions;
  };
  events?: {
    onRefresh?: EventEmitter<any>;
    onSelect?: EventEmitter<any>;
    onDeselect?: EventEmitter<any>;
    onEdit?: EventEmitter<any>;
    onDisplay?: EventEmitter<any>;
    onCreate?: EventEmitter<any>;
    onDelete?: EventEmitter<any>;
  };
}