/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * Remplit le formulaire avec les infos de l'utilisateur
   * @param  {string} uid user id
   */
  var loadUser = function (uid) {

    let oUser = Object.assign({}, services.auth.getUserInfos());
    setTimeout(() => {
      scope.setFormValues(oUser);
    }, 500);
  }

  if (env.properties.unsubscribe === true) {
    angular.element("#unsubscribe_button").removeClass("hidden");
  }

  // Charge l'utilisateur courrant
  loadUser(services.auth.user_id);

  /**
   * Fonction appelée avant l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.beforeEvent = function () {
    const oFormValues = scope.getFormValues();
    return true
  }

  /**
   * Fonction appelée après l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.afterEvent = function () {
    const oFormValues = scope.getFormValues();

    // Reset les nouvelles infos utilisateur dans AuthService
    services.auth.setUserInfos(oFormValues);

    // Cache la modale
    $('#dashboard-simpleform-modal').modal('hide');
    return true
  }

  /**
   * Désactivation du compte de l'utilisateur.
   */
  scope.unsubscribeAccount = function () {
    const sUrl = env.properties.app_api_url + '/vitis/accounts/' + services.auth.user_id + '/unsubscribe';
    services.vitisRequest.ajaxRequestPromise({
      method: 'PUT',
      url: sUrl,
    }).then((resp) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          services.vitisAlert.fireToast({
            type: 'success',
            title: services.translate.instant('FORM_UNSUBSCRIBE_MSG_USER_USER')
          });
          setTimeout(()=>{
            // Déconnecte l'utilisateur
            services.auth.disconnect();
          }, 3000);
        }
      }
    });
  }
}
