/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * Remplit le formulaire avec les infos de l'utilisateur
   * @param  {string} uid user id
   */
  var loadUser = function (uid) {

    let oUser = Object.assign({}, services.auth.getUserInfos());
    setTimeout(() => {
      scope.setFormValues(oUser);
    }, 500);
  }

  // Charge l'utilisateur courrant
  loadUser(services.auth.user_id);

  /**
   * Fonction appelée avant l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.beforeEvent = function () {
    return new Promise((resolve, reject) => {
      const oFormValues = scope.getFormValues();
      if (scope.oFormValues.valid && services.vitisCheck.isString(oFormValues.password)) {
        if (oFormValues.password.length > 0) {

          // Message d'erreur si le mot de passe et la confirmation sont différents
          if (oFormValues.password !== oFormValues.password_confirm) {
            services.translate.get('ERROR_PASSWORD_CONFIRM_USER_USER').subscribe((res) => {
              services.vitisAlert.alert({
                'type': 'error',
                'text': res
              });
            });
          } else {

            // Message warning de changement de mot de passe
            services.translate.get('UPDATE_PASSWORD_USER_USER').subscribe((res) => {
              services.vitisAlert.alert({
                'type': 'warning',
                'html': res,
                'showCancelButton': true
              }).then((result) => {
                if (result.value) {
                  resolve();
                }
              });
            });
          }
        }
      }
    });
  }

  /**
   * Fonction appelée après l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.afterEvent = function () {
    const oFormValues = scope.getFormValues();

    // Déconnecte l'utilisateur
    services.auth.disconnect();

    // Cache la modale
    $('#dashboard-simpleform-modal').modal('hide');
    return true;
  }

  /**
   * Fonction appelée à l'initialisation du formulaire
   * @return {boolean|promise}
   */
  scope.initEvent = function () {}
}
