/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * Fonction appelée à l'initialisation du formulaire
   * @return {boolean|promise}
   */
  scope.confirmAccount = function () {
    return new Promise((resolve, reject) => {
      services.route.data.subscribe((oConfig) => {
        const sDatakey = scope.getFormField('datakey').getValue();
        if (sDatakey) {
          services.vitisRequest.ajaxRequestPromise({
            method: 'PUT',
            params: {
              datakey: sDatakey
            },
            url: env.properties.app_api_url + '/' + oConfig.options.ressource + '/' + scope.oFormValues.value.login + '/confirm',
          }).then((resp) => {
            if (resp.data) {
              if (resp.data.status === 1) {
                sToastMode = 'success';
                sText = services.translate.instant('FORM_USERS_REQUEST_CONFIRMATION_ACCOUNT_OK');
                services.vitisAlert.fireToast({
                  type: sToastMode,
                  title: sText
                });
                resolve();
                return;
              }
            }
            reject();
          }).catch(() => {
            reject();
          });
        } else {
          resolve();
        }
      });
    });
  }

  /**
   * Fonction appelée à l'initialisation du formulaire
   * @return {boolean|promise}
   */
  scope.abortAccount = function () {
    return new Promise((resolve, reject) => {
      services.route.data.subscribe((oConfig) => {

        const sDatakey = scope.getFormField('datakey').getValue();
        if (sDatakey) {
          services.vitisRequest.ajaxRequestPromise({
            method: 'PUT',
            params: {
              datakey: sDatakey
            },
            url: env.properties.app_api_url + '/' + oConfig.options.ressource + '/' + scope.oFormValues.value.login + '/abort',
          }).then((resp) => {
            if (resp.data) {
              if (resp.data.status === 1) {
                sToastMode = 'success';
                sText = services.translate.instant('FORM_USERS_REQUEST_CONFIRMATION_ACCOUNT_KO');
                services.vitisAlert.fireToast({
                  type: sToastMode,
                  title: sText
                });
                resolve();
                return;
              }
            }
            reject();
          }).catch(() => {
            reject();
          });
        } else {
          resolve();
        }
      })
    });
  }

  /**
   * Charge les données complémentaire si présente
   */
  scope.loadCompData = function () {
    setTimeout(function () {
      const sCompData = scope.getFormField('comp_data').getValue();
      if (sCompData) {

        const aCompData = JSON.parse(sCompData);
        const aFinalData = {};

        for (let sKey in aCompData) {
          const sText = services.translate.instant('FORM_USERS_USER_REQUESTS_COMP_DATA_' + sKey.toUpperCase());

          if (sText !== '') {
            aFinalData[sText] = aCompData[sKey];
          } else {
            aFinalData[sKey] = aCompData[sKey];
          }
        }

        const oCompData = {
          status: true,
          title: 'FORM_COMP_DATA_SECTION',
          values: aFinalData
        }
        scope.getFormField('comp_data_viewer').setValue(oCompData);
      }
    }, 500);
  }
}