import fs from 'fs';
import path from 'path';
import { Logger } from 'vitis-engine-lib';
import { FileConfig } from "./models";

/**
 * Service d'accès aux fichiers de conf jobs.json, tasks.json etc.
 */
export class FileConfigSrvc {

    private confDirPath?: string;
    private oLogger: Logger;

    constructor(oLogger: Logger) {
        this.oLogger = oLogger;
    }

    /**
     * Charge le fichier de config des jobs/tasks/cleaner
     * @param {string} sType jobs/tasks/cleaner
     */
    public getFileConfig(sType: string): FileConfig | undefined {

        let configFilePath: string;
        if (fs.existsSync(path.resolve(__dirname, `conf/${sType}.json`))) {
            this.confDirPath = path.resolve(__dirname, `conf`);
            configFilePath = path.resolve(__dirname, `conf/${sType}.json`);
        } else if (fs.existsSync(path.resolve(__dirname, `../conf/${sType}.json`))) {
            this.confDirPath = path.resolve(__dirname, `../conf`);
            configFilePath = path.resolve(__dirname, `../conf/${sType}.json`);
        } else if (fs.existsSync(path.resolve(`./conf/${sType}.json`))) {
            this.confDirPath = path.resolve(`./conf`);
            configFilePath = path.resolve(`./conf/${sType}.json`);
        } else if (fs.existsSync(path.resolve(`../conf/${sType}.json`))) {
            this.confDirPath = path.resolve(`../conf`);
            configFilePath = path.resolve(`../conf/${sType}.json`);
        } else {
            this.oLogger.log(`${sType} config file not found`, null, 'ERROR');
            return;
        }

        // Fichier de configuration
        const config = JSON.parse(String(fs.readFileSync(configFilePath)));
        return config;
    }

    /**
     * Met à jour un fichier de config jobs/tasks/cleaner
     * @param sType 
     * @param oFileConfig 
     */
    public updateFileConfig(sType: string, oFileConfig: FileConfig) {
        if (this.confDirPath) {

            const configFilePath = path.resolve(this.confDirPath, `${sType}.json`);

            if (fs.existsSync(configFilePath)) {
                try {
                    fs.writeFileSync(configFilePath, JSON.stringify(oFileConfig));
                } catch (error) {
                    this.oLogger.log(`${sType} config file cannot be updated`, null, 'ERROR');
                }
            } else {
                this.oLogger.log(`${sType} config file not found`, null, 'ERROR');
            }
        }
    }
}