/**
 *                        Jobs Runner
 * 
 * Ce fichier permet de lancer les jobs déclarés dans jobs.json
 * Les jobs correspondent à des commances système, en fonction
 * du système en cours, le script lancera command_win ou
 * comman_lin
 * 
 * Les périodes sont définies de la même manière que sur cron
 * mais une option par seconde est disponible :
 * ┌────────────── second (optional)
 * │ ┌──────────── minute
 * │ │ ┌────────── hour
 * │ │ │ ┌──────── day of month
 * │ │ │ │ ┌────── month
 * │ │ │ │ │ ┌──── day of week
 * │ │ │ │ │ │
 * │ │ │ │ │ │
 * * * * * * *
 * 
 */

import { propertiesSrvc } from "vitis-engine-lib";
import { Logger } from "vitis-engine-lib";
import { Db } from "vitis-engine-lib";
import { LocalFS } from "vitis-engine-lib";
import { RunnerSrvc } from "./runnerSrvc";
import { FileConfigSrvc } from "./fileconfigSrvc";

const reloadInterval = 10; // Intervalle de rechargement en secondes

/**
 * Interval sur reloadInterval pour recharger le fichier de conf et lancer la planification
 */
setInterval(() => {

  // Properties
  propertiesSrvc.getProperties().then((oProperties: any) => {

    const oLogger = new Logger(oProperties);
    // utilisation d'un ficheir spécifique pour logger via les runners
    oLogger.setLogFilePath(oProperties.vitis_runners_log_path)
    // désactive la console pour ne pas remplir le syslog
    oLogger.disableConsoleMessage();
    const oLocalFS = new LocalFS(oProperties, oLogger);
    const oDb = new Db(oProperties, {
      user: oProperties.db_superuser,
      host: oProperties.db_server,
      database: oProperties.db_name,
      password: oProperties.db_password,
      port: oProperties.db_port,
      autoSslConfig: true
    }, oLogger);

    const oFileConfigSrvc = new FileConfigSrvc(oLogger);
    const oRunnerSrvc = new RunnerSrvc(oProperties, oDb, oLogger, oLocalFS, oFileConfigSrvc);

    // Fichier de config des jobs
    const oJobsConfig = oFileConfigSrvc.getFileConfig('jobs');

    // Plannifie les jobs
    if (oJobsConfig && oJobsConfig.jobs && oJobsConfig.jobs.length) {
      oRunnerSrvc.startJobs(oJobsConfig.jobs, reloadInterval);
    }

    const oTasksConfig = oFileConfigSrvc.getFileConfig('tasks');

    // Lance les tâches
    if (oTasksConfig && oTasksConfig.tasks && oTasksConfig.tasks.length) {
      oRunnerSrvc.startTasks(oTasksConfig.tasks);
    }
  });

}, reloadInterval * 1000);
