import yargs from 'yargs';

/**
 * utilisation de yargs pour gérer l'aide sur engine
 */
export const args = yargs
  .usage('$0 <cmd> [OPTIONS]')
  .command('printTemplate', 'Imprime un template HTML en PDF',
    {
      'HTMLInputFileName': {
        describe: "Fichier contenant le HTML en entrée",
        demandOption: true,
        nargs: 1,
        type: "string",
      },
      'TmpFolder': {
        describe: "Dossier temporaire",
        demandOption: true,
        nargs: 1,
        type: "string",
      },
      'OutputFileName': {
        describe: "Fichier contenant le PDF en sorite",
        demandOption: true,
        nargs: 1,
        type: "string",
      },
      'Format': {
        describe: "A4, A3, A2, A0",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
      'OutputFormat': {
        describe: "pdf, jpeg, vectorpdf, imagepdf",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
      'Orientation': {
        describe: "portrait, paysage",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
      'JsonIncludes': {
        describe: "inclusions",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
      'JsonScope': {
        describe: "scope",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
      'JsonMaps': {
        describe: "Cartes à include",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
    })
  .command('printExcel', 'Génére un Excel à partir d\'un template json et d\'un scope json',
    {
      'JsonTemplate': {
        describe: "Template Json",
        demandOption: true,
        nargs: 1,
        type: "string",
        alias : "t"
      },
      'OutputFileName': {
        describe: "Fichier contenant le Excel en sortie",
        demandOption: true,
        nargs: 1,
        type: "string",
        alias: "o"
      },
      'JsonScope': {
        describe: "scope",
        demandOption: false,
        nargs: 1,
        type: "string",
        alias: "i"
      },
    })
  .command('concatPdf', 'Fusionne tous les pdf dans un dossier dans l\ordre alphabétique et écrit le résultat à l\'endroit souhaité',
    {
      'inputFolder': {
        describe: "Dossier contenant les pdf à fusionner",
        demandOption: true,
        nargs: 1,
        type: "string",
        alias: 'i',
      },
      'OutputFileName': {
        describe: "Fichier contenant le pdf final",
        demandOption: true,
        nargs: 1,
        type: "string",
        alias: 'o',
      }
    })
  .command('signPdf', 'Signe un PDF électroniquement avec un certificat P12',
    {
      'inputFile': {
        describe: "Chemin vers le PDF à signer",
        demandOption: true,
        nargs: 1,
        type: "string",
        alias: 'i',
      },
       'outputFile': {
        describe: "Chemin d'écriture du PDF signé",
        demandOption: true,
        nargs: 1,
        type: "string",
        alias: 'o',
      },
      'certFile': {
        describe: "chemin vers le certificat P12",
        demandOption: true,
        nargs: 1,
        type: "string",
        alias: 'c',
      },
      'certPass': {
        describe: "Mot de passe certificat P12",
        demandOption: false,
        nargs: 1,
        type: "string",
        alias: 'p',
      },
      'certLength': {
        describe: "Longueur certificat P12",
        demandOption: false,
        nargs: 1,
        type: "number",
        default : 8192,
        alias: 'k',
      },
      'reason': {
        describe: "Raison de la signature",
        demandOption: false,
        nargs: 1,
        type: "string",
        default: "Finalisation du document",
        alias: 'r',
      },
      'contact': {
        describe: "Mail pour contacter le signataire",
        demandOption: false,
        nargs: 1,
        type: "string",
        default: "fake@domain.com",
        alias: 'm',
      }, 
      'name': {
        describe: "Nom du signataire",
        demandOption: false,
        nargs: 1,
        type: "string",
        default: "John Doe",
        alias: 'n',
      },
      'location': {
        describe: "Lieu de la signature",
        demandOption: false,
        nargs: 1,
        type: "string",
        default: "somewhere",
        alias: 'l',
      }
    })
  .command('renderHtml', 'Compile un template HTML avec angular',
    {
      'HTMLInputFileName': {
        describe: "Fichier contenant le HTML en entrée",
        demandOption: true,
        nargs: 1,
        type: "string",
      },
      'OutputFileName': {
        describe: "Fichier contenant le HTML en sorite",
        demandOption: true,
        nargs: 1,
        type: "string",
      },
      'Format': {
        describe: "A4, A3, A2, A0",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
      'JsonIncludes': {
        describe: "inclusions",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
      'JsonScope': {
        describe: "scope",
        demandOption: false,
        nargs: 1,
        type: "string",
      },
    })
  .wrap(null)
  .help('help')
  .alias('h', 'help')
  .version(false)
  .locale('fr')
  .strict() // bloque les commandes non connues
  .argv;