<?php

namespace App\Module\Vitis\WebService\Controller;

use App\Module\Vitis\WebService\Controller\TreeController;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\DatabaseConnection;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\VitisRequest;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller qui permet de gérer le dossier partagé.
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 */
class SharedDirController extends TreeController {
  /**
   * Initialise le controler et instancie les services requis
   * @param Properties $oPropertiesSrvc Service d'accès aux properties
   * @param VitisRequest $oVitisRequest Service d'abstraction de la requête Symfony
   * @param Route $oRouteConfig Service d'accès à la configuration de la route
   * @param LoggerManager $oLoggerManager Service d'utilsation des logs
   * @param DatabaseConnection $oDatabaseConnection Service d'accès à la base de données
   * @param FileManager $oFileManager Service de gestion des fichiers
   */
  public function __construct(Properties $oPropertiesSrvc, VitisRequest $oVitisRequest, Route $oRouteConfig, LoggerManager $oLoggerManager, DatabaseConnection $oDatabaseConnection, FileManager $oFileManager) {
    parent::__construct($oPropertiesSrvc, $oVitisRequest, $oRouteConfig, $oLoggerManager, $oDatabaseConnection, $oFileManager);
    // création du shared_dir s'il n'existe pas
    $oFileAccessor = $oFileManager->getFileAccessor();
    if (!$oFileAccessor->file_exists($this->oProperties->get('shared_dir'))) {
      $oFileAccessor->mkdir($this->oProperties->get('shared_dir'), 0777, true);
    }
  }

  /**
   * get workspace directory tree
   * @return Response
   */
  public function getSharedDirTree(): Response {
    $oResponse = $this->getTree();
    $aContent = json_decode($oResponse->getContent(), true);

    // Si le système de fichier n'est pas local on redéfini le dossier parent (S3 n'a pas de dossier)
    if (isset($aContent['data']) && $this->oProperties->get('file_system') !== Properties::LOCAL_FS) {
      if (!isset($aContent['data']["content"])) {
        $aContent['data']["content"] = [];
        $aContent['data']["lastModification"] = "none";
        $aContent['data']["size"] = "";
        unset($aContent['data']["href"]);
        $oResponse->setContent(json_encode($aContent));
      }
    }

    return $oResponse;
  }

}
