<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Entity\DomainGroup;
use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;

/**
 *  Objet ORM pour v_group
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="v_group")
 */
class Group extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="group_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vitis).seq_common")
   */
  private $group_id;

  /**
   * @var string
   * @Column(type="string", length=50)
   */
  private $name;

  /**
   * @var int
   * @Column(type="integer", nullable=true)
   */
  private $nb_members;

  /**
   * @var ArrayCollection<User>
   * @ManyToMany(targetEntity="User")
   * @JoinTable(
   *   name="@prop(schema_vitis).user_group",
   *   joinColumns={@JoinColumn(name="group_id", referencedColumnName="group_id")},
   *   inverseJoinColumns={@JoinColumn(name="user_id", referencedColumnName="user_id")}
   * )
   */
  private $aUsers;

  /**
   * @var DomainGroup
   * @OneToOne(targetEntity=DomainGroup::class, mappedBy="vitisGroup", cascade={"persist", "remove"})
   */
  private $domainGroup;

  public function __construct() {
    $this->aUsers = new ArrayCollection();
    $this->nb_members = 0;
  }

  /**
   * @return int identifiant du groupe
   */
  public function getGroupId(): int {
    return $this->group_id;
  }
  /**
   * @return string nom du groupe
   */
  public function getName(): string {
    return $this->name;
  }
  /**
   * @param string $name nom du groupe
   * @return self
   */
  public function setName(string $name): self {
    if (empty($name)) {
      // random
    } else {
      $this->name = $name;
    }
    return $this;
  }
  /**
   * @return int[] liste des user_id
   */
  public function getUsers(): array{
    $aUserId = [];
    foreach ($this->aUsers as $oUser) {
      $aUserId[] = $oUser->getUserId();
    }
    return $aUserId;
  }
  /**
   * @return int nombre d'utilisateur dans le groupe
   */
  public function getNbMembers(): int {
    $this->nb_members = $this->aUsers->count();
    return $this->nb_members;
  }

  public function getGroupName(): ?string
  {
    if (isset($this->domainGroup)){
      return $this->domainGroup->getDomain()->getName();
    }
    return '';
  }

  public function getDomainGroup(): ?DomainGroup
  {
      return $this->domainGroup;
  }

  public function setDomainGroup(?DomainGroup $domainGroup): self
  {
      // unset the owning side of the relation if necessary
      if ($domainGroup === null && $this->domainGroup !== null) {
          $this->domainGroup->setVitisGroup(null);
      }

      // set the owning side of the relation if necessary
      if ($domainGroup !== null && $domainGroup->getVitisGroup() !== $this) {
          $domainGroup->setVitisGroup($this);
      }

      $this->domainGroup = $domainGroup;

      return $this;
  }

  public function getAutoRefresh(): ?bool
  {
      return isset($this->domainGroup)? $this->domainGroup->getAutoRefresh() : NULL;
  }
}