<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;

/**
 *  Objet ORM pour v_email_template
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="v_email_template")
 */
class MailTemplate extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="email_template_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vitis).seq_common")
   */
  private $email_template_id;

  /**
   * @var string
   * @Column(type="string", length=60)
   */
  private $name;
  /**
   * @var string
   * @Column(type="string")
   */
  private $subject;
  /**
   * @var string
   * @Column(type="string")
   */
  private $body;
  /**
   * @var string
   * @Column(name="`to`", type="string")
   */
  private $to;
  /**
   * @var string
   * @Column(type="string")
   */
  private $cc;
  /**
   * @var string
   * @Column(type="string")
   */
  private $cci;
  /**
   * @var string
   * @Column(type="string")
   */
  private $rt_emailcontext_id;
  /**
   * @var string
   * @Column(type="string")
   */
  private $json_scope;
  /**
   * @var string
   * @Column(type="string")
   */
  private $json_includes;
  /**
   * @var string
   * @Column(type="string")
   */
  private $context_documentation;
  /**
   * @var string
   * @Column(type="string")
   */
  private $context_label;
  /**
   * @var string
   * @Column(type="string")
   */
  private $smtp_login;
  /**
   * @var string
   * @Column(type="string")
   */
  private $smtp_password;

  /**
   * @return int identifiant du template
   */
  public function getMailTemplateId(): ?int {
    return $this->email_template_id;
  }
  /**
   * @return string nom du template
   */
  public function getName(): ?string {
    return $this->name;
  }
  /**
   * @param string $name nom du template
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }
  /**
   * @return string nom du template
   */
  public function getSubject(): ?string {
    return $this->subject;
  }
  /**
   * @param string $subject nom du template
   * @return self
   */
  public function setSubject(?string $subject): self {
    $this->subject = $subject;
    return $this;
  }
  /**
   * @return string nom du template
   */
  public function getBody(): ?string {
    return $this->body;
  }
  /**
   * @param string $body nom du template
   * @return self
   */
  public function setBody(?string $body): self {
    $this->body = $body;
    return $this;
  }
  /**
   * @return string nom du template
   */
  public function getTo(): ?string {
    return $this->to;
  }
  /**
   * @param string $to nom du template
   * @return self
   */
  public function setTo(?string $to): self {
    $this->to = $to;
    return $this;
  }
  /**
   * @return string nom du template
   */
  public function getCc(): ?string {
    return $this->cc;
  }
  /**
   * @param string $cc nom du template
   * @return self
   */
  public function setCc(?string $cc): self {
    $this->cc = $cc;
    return $this;
  }
  /**
   * @return string nom du template
   */
  public function getCci(): ?string {
    return $this->cci;
  }
  /**
   * @param string $cci nom du template
   * @return self
   */
  public function setCci(?string $cci): self {
    $this->cci = $cci;
    return $this;
  }
  /**
   * @return string nom du template
   */
  public function getContextName(): ?string {
    return $this->rt_emailcontext_id;
  }
  /**
   * @param string $rt_emailcontext_id nom du template
   * @return self
   */
  public function setContextName(?string $rt_emailcontext_id): self {
    $this->rt_emailcontext_id = $rt_emailcontext_id;
    return $this;
  }
  /**
   * @return string nom du template
   */
  public function getJsonScope(): ?string {
    return $this->json_scope;
  }
  /**
   * @return string nom du template
   */
  public function getJsonIncludes(): ?string {
    return $this->json_includes;
  }
  /**
   * @return string Documentation du contexte
   */
  public function getContextDocumentation(): ?string {
    return $this->context_documentation;
  }
  /**
   * @return string Documentation du contexte
   */
  public function getContextLabel(): ?string {
    return $this->context_label;
  }

  public function getMailParameters(): array{
    return [
      'to' => $this->to,
      'cc' => $this->cc,
      'cci' => $this->cci,
      'subject' => $this->subject,
      'aAttachments' => [],
      'bUseHTML' => true,
      'smtp_login' => $this->smtp_login,
      'smtp_password' => $this->smtp_password,
    ];
  }
  /**
   * @return string login smtp
   */
  public function getSmtpLogin(): ?string {
    return $this->smtp_login;
  }
  /**
   * @return string mot de smtp
   */
  public function getSmtpPassword(): ?string {
    return $this->smtp_password;
  }
}