<?php

namespace App\Module\Vitis\WebService\Query;

use Doctrine\DBAL\Platforms\AbstractPlatform;
use Doctrine\DBAL\Types\ConversionException;
use Doctrine\DBAL\Types\DateTimeType;

class DateTimeTypeWithMicroTime extends DateTimeType {
  /**
   * @var string
   */
  public const POSTRES_DATE_FORMAT = 'Y-m-d H:i:s.uO';

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'udatetime';
  }
  /**
   * {@inheritdoc}
   */
  public function convertToDatabaseValue($value, AbstractPlatform $platform) {
    if ($value === null) {
      return $value;
    }

    if ($value instanceof \DateTimeInterface) {
      return $value->format(self::POSTRES_DATE_FORMAT);
    }

    // théoriquement si on passe là ce sera pour l'exception
    return parent::convertToDatabaseValue($value, $platform);
  }
  /**
   * {@inheritdoc}
   */
  public function convertToPHPValue($value, AbstractPlatform $platform) {
    if (null === $value || $value instanceof \DateTime) {
      return $value;
    }

    $converted = \DateTime::createFromFormat(
      self::POSTRES_DATE_FORMAT,
      $value
    );

    if (!$converted) {
      throw ConversionException::conversionFailedFormat(
        $value,
        $this->getName(),
        self::POSTRES_DATE_FORMAT
      );
    }

    return $converted;
  }

  /**
   * Retourne un dateitme qui tien compte des microsecondes
   *
   * @return \DateTime
   */
  public static function getDatetime(): \DateTime {
    return \DateTime::createFromFormat('U.u', (string) microtime(true));
  }
}
