<?php

namespace App\Module\Vitis\WebService\Query\Functions;

use Doctrine\ORM\Query\AST\Functions\FunctionNode;
use Doctrine\ORM\Query\Lexer;
use Doctrine\ORM\Query\Parser;
use Doctrine\ORM\Query\SqlWalker;

/**
 * Classe abstraite pour les caster SQL.
 * Ex : field::geometry || function(field::integer)::text
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

abstract class AbstractVitisCaster extends FunctionNode {
  /**
   * @var string
   */
  protected const TYPE = '';

  /**
   * @var \Doctrine\ORM\Query\AST\ArithmeticFactor[]
   */
  protected $expressions = array();

  /**
   * Parse la requête DQL pour extraire l'appel d'une fonction
   *
   * @param Parser $parser Parser DQL, utilise Doctrine\ORM\Query\Lexer pour parser le DQL
   * @return void
   */
  public function parse(Parser $parser): void {
    $parser->match(Lexer::T_IDENTIFIER); // CastType
    $parser->match(Lexer::T_OPEN_PARENTHESIS); // (
    $this->expressions[] = $parser->ArithmeticFactor(); // value
    $parser->match(Lexer::T_CLOSE_PARENTHESIS); // )
  }

  /**
   * Retourne le texte à utiliser pour le SQL pour la classe parente
   *
   * @param SqlWalker $sqlWalker permet de parcourir la requete SQL
   * @return string chaine de caractère à utiliser à travers DBAL pour le SQL
   */
  public function getSql(SqlWalker $sqlWalker): string {
    $arguments = array();

    // permet d'attendre les events de remplacements imbriqué
    foreach ($this->expressions as $expression) {
      $arguments[] = $expression->dispatch($sqlWalker);
    }

    return implode(', ', $arguments) . '::' . static::TYPE; // utilise la valeur de la constante Type défini dans la classe enfant
  }
}
