<?php

namespace App\Module\Vitis\WebService\Service\Configuration;

use App\Module\Vitis\WebService\Service\Configuration\Properties;

/**
 * Service abstrait pour charger une configuration et la passer dans un tableau.
 * Contient la majorité des fonctions et appels aux services pour les controller.
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

abstract class AbstractConfigurationLoader {
  /**
   * @var string
   */
  protected const CONFIGURATION_FILE_PATH = '';

  /**
   * @var mixed[]
   */
  protected $aConfiguration;

  /**
   * @var Properties
   */
  protected $oPropertiesSrvc;

  /**
   * Parse le fichier Json de chaque module et l'ajoute au tableau des configuration
   * @return void
   */
  protected function loadConfiguration(): void {
    // calcul du dossier des modules
    $sConfigDir = $this->oPropertiesSrvc->replacePropertiesInString(static::CONFIGURATION_FILE_PATH);
    $aConfigTmp = [];
    foreach ($this->oPropertiesSrvc->get('_module_list') as $sModuleName) {
      $sConfigFilePath = str_replace('@Module', $sModuleName, $sConfigDir);
      if (file_exists($sConfigFilePath)) {
        $aJsonConfig = json_decode(file_get_contents($sConfigFilePath), true);
        $aConfigTmp = array_merge_recursive($aConfigTmp, $aJsonConfig);
      }
    }
    $this->aConfiguration = $aConfigTmp;
  }
  /**
   * Retourne la configuration demandé
   * @return array configuration demandé
   */
  public function getConfiguration(): array{
    return $this->aConfiguration;
  }
}
?>