<?php

namespace App\Module\Vitis\WebService\Service\Serializer;

use App\Module\Vitis\WebService\Service\DataAccessor\Fields\DateField;
use App\Module\Vitis\WebService\Service\Encoder\VitisJsonEncoder;
use App\Module\Vitis\WebService\Service\Encoder\VitisXmlEncoder;
use App\Module\Vitis\WebService\Service\Normalizer\VitisNormalizer;
use Symfony\Component\Serializer\Normalizer\DateTimeNormalizer;
use Symfony\Component\Serializer\Serializer;

/**
 * Service permettant la serialisation des sorties pour Vitis.
 *
 * @author Anthony Borghi   <anthony.borghi@veremes.com>
 * @author Alexandre Mironenkov   <alexandre.mironenkov@veremes.com>
 */

class VitisSerializer {
  /**
   * @var string
   */
  private $sSerializationFormat = 'json';
  /**
   * @var string
   */
  private $sDeserializationFormat = 'json';

  /**
   * @var Serializer
   */
  private $oSerializer;
  /**
   * @var VitisNormalizer
   */
  private $oNormalizer;
  /**
   * @var VitisXmlEncoder
   */
  private $oXmlEncoder;
  /**
   * @var VitisJsonEncoder
   */
  private $oJsonEncoder;

  /**
   */
  public function __construct() {

    $this->oNormalizer = new VitisNormalizer();
    $this->oXmlEncoder = new VitisXmlEncoder($this->oNormalizer);
    $this->oJsonEncoder = new VitisJsonEncoder();
    $aEncoders = [$this->oXmlEncoder, $this->oJsonEncoder];
    $aNormalizers = [];
    $aNormalizers[] = new DateTimeNormalizer([DateTimeNormalizer::FORMAT_KEY => DateField::DEFAULT_DB_FORMAT]);
    $aNormalizers[] = $this->oNormalizer;

    $this->oSerializer = new Serializer($aNormalizers, $aEncoders);
  }

  /**
   * Déserialise un chaine de caractère dans une entité Doctrine.
   * @param string $sData Donnée à déserialiser
   * @param string $sEntityClass namespace complet de l'entité à utiliser
   * @param array $aConfig config (default : [])
   * @return object Entité désérialisées
   */
  public function deserialize(string $sData, string $sEntityClass, array $aConfig = []): object {
    return $this->oSerializer->deserialize($sData, $sEntityClass, $this->sDeserializationFormat, $aConfig);
  }

  /**
   * Serialise la sortie.
   * @param object|array $aData Donnée à serialiser, entité doctrine ou tableau associatif
   * @return string Données sérialisées
   */
  public function serialize($aData, array $aContext = []): string {
    return $this->oSerializer->serialize($aData, $this->sSerializationFormat, $aContext);
  }

  /**
   * @param string $sFormat Format à utiliser pour la sérialisation (json|xml)
   * @return void
   */
  public function setSerializerFormat($sFormat): void {
    $this->sSerializationFormat = $sFormat;
  }
  /**
   * @return VitisNormalizer
   */
  public function getVitisNormalizer(): VitisNormalizer {
    return $this->oNormalizer;
  }
  /**
   * @return VitisXmlEncoder
   */
  public function getXmlEncoder(): VitisXmlEncoder {
    return $this->oXmlEncoder;
  }
  /**
   * @return VitisJsonEncoder
   */
  public function getJsonEncoder(): VitisJsonEncoder {
    return $this->oJsonEncoder;
  }
}
