/* global ol, nsUtils */

/**
 * Initialize the map
 * @param {object} opt_options
 * @param {string} opt_options.token
 * @param {string} opt_options.apiUrl
 * @param {string} opt_options.mapJSON
 * @param {array} opt_options.extent
 * @param {array} opt_options.features
 * @param {number} opt_options.featuresZoom
 * @param {array} opt_options.measureFeatures
 * @param {number} opt_options.resolutionCoeff
 * @param {number} opt_options.properties
 */
var initMap = function (opt_options) {

    // Convertit en float l'étendue
    if (isDef(opt_options.extent)) {
        for (var i = 0; i < opt_options.extent.length; i++) {
            opt_options.extent[i] = parseFloat(opt_options.extent[i]);
        }
    }

    if (!isDef(opt_options.apiUrl)) {
        callError('opt_options.apiUrl not defined');
    }
    if (!isDef(opt_options.token)) {
        callError('opt_options.token not defined');
    }
    if (!isDef(opt_options.mapId) && !isDef(opt_options.mapJSON)) {
        callError('neither opt_options.mapId nor opt_options.mapJSON defined');
    }
    if (!isDef(opt_options.extent) && !isDef(opt_options.features)) {
        callError('neither opt_options.extent nor opt_options.features defined');
    }

    if (opt_options.mapJSON && typeof opt_options.mapJSON === 'string') {
        opt_options.mapJSON = JSON.parse(opt_options.mapJSON);
    }

    if (opt_options.extent && typeof opt_options.extent === 'string') {
        opt_options.extent = opt_options.extent.split('|');
    }

    if (opt_options.features && typeof opt_options.features === 'string') {
        opt_options.features = opt_options.features.split('|');
    }

    window.oProperties = opt_options.properties;

    // Ajoute la définition OpenLayers des projections
    window.projections = new Projections();

    // Initialise la carte
    window.oPrintMap = new PrintMap({
        mapJSON: opt_options.mapJSON,
        extent: opt_options.extent,
        features: opt_options.features,
        featuresZoom: opt_options.featuresZoom,
        measureFeatures: opt_options.measureFeatures,
        resolutionCoeff: opt_options.resolutionCoeff,
        apiUrl: opt_options.apiUrl,
        token: opt_options.token
    });

    // Initialise la progression (timeout 100 le temps que la carte soit définie)
    setTimeout(function () {
        window.oProgress = new Progress(window.oPrintMap, opt_options['displayProgress']);
    }, 100);
};